/**************************
     This file is copyrighted, but license is hereby granted for personal,
     academic, and non-profit use.
     Commercial users should contact Xin Zhou xzhou82@gmail.com for permission.
 **************************/
#include <stdio.h>
#include <stdlib.h>
#include <search.h>
#include <string.h>
#include <sys/wait.h>
#include <sys/stat.h>
#include <limits.h>
#include <mysql.h>
#include <assert.h>
#include <math.h>
#include <time.h>
#include "sam.h"
#include "tabix.h"

#define MAXbpwidth 10
#define StartChrom_cgi "startChr"
#define StartCoord_cgi "startCoord"
#define StopChrom_cgi "stopChr"
#define StopCoord_cgi "stopCoord"
#define MaxProcessNum 20
#define MaxBigbeditem4geneplot 25000
#define min(a,b) ( (a) < (b) ? (a) : (b) )
#define max(a,b) ( (a) > (b) ? (a) : (b) )

#define GENOME "genome"
#define M_hide 0
#define M_hide_s "0"
#define M_show 1
#define M_show_s "1"
#define M_thin 2
#define M_thin_s "2"
#define M_full 3
#define M_full_s "3"
#define M_arc 4
#define M_arc_s "4"
#define M_trihm 5
#define M_trihm_s "5"
#define M_den 6
#define M_den_s "6"
#define M_error -1

/* array index of session table fields */
#define sid_session 0
#define sid_statusid 1
#define sid_statusnote 2
#define sid_clientctrl 3
#define sid_customTrackAttr 4
#define sid_juxType 5
#define sid_juxWhat 6
#define sid_juxNote 7
#define sid_dspStat 8
#define sid_correlation 9
#define sid_itemlist 10
#define sid_custmd 11
#define sid_mdconshow 12
#define sid_dataset 13
#define sid_pairwisecomp 14
#define sid_bed_n 15
#define sid_bed_c 16
#define sid_bedgraph_n 17
#define sid_bedgraph_c 18
#define sid_sam_n 19
#define sid_sam_c 20
#define sid_qdecor_n 21
#define sid_lr_n 22
#define sid_lr_c 23
#define sid_cat_n 24
#define sid_cat_c 25
#define sid_bigwighmtk_n 26
#define sid_bigwighmtk_c 27
#define sid_decorRenderingParams 28
#define sid_qtc 29
#define sid_gssparam 30
#define sid_stickynote 31
#define sid_bam_n 34
#define sid_bam_c 35
#define sid_reserved_3 36
#define sid_reserved_4 37
#define sid_reserved_5 38
#define sid_reserved_6 39
#define sid_reserved_7 40
#define sid_reserved_8 41
#define sid_reserved_9 42
#define sid_reserved_10 43

// all of following are tabix
#define FT_bed_n 0
#define FT_bed_c 1
#define FT_bedgraph_n 2
#define FT_bedgraph_c 3
#define FT_sam_n 4
#define FT_sam_c 5
#define FT_bam_n 17
#define FT_bam_c 18
#define FT_qdecor_n 8
#define FT_lr_n 9
#define FT_lr_c 10
#define FT_cat_n 12
#define FT_cat_c 13
// all of following are not tabix
#define FT_pwc 6
#define FT_htest 7
#define FT_tkgrp 11
#define FT_bigwighmtk_n 14
#define FT_bigwighmtk_c 15

#define RM_genome 0
#define RM_jux_n 1
#define RM_jux_c 2
#define RM_gsv_c 3
#define RM_gsv_kegg 4
#define RM_rpbr_beam 5
#define RM_yearmonthday 6

#define ymdayfile "yearmonthday" // a bed file

#define trashDir "/usr/lib/trash"
#define WWWT "/var/www/browser/t"
#define BINdir "../bin"
#define Mysqlserver "localhost" // 10.200.0.11
#define Mysqluser "root"
#define Mysqlpswd "xzhou"

// TODO
#define rpbr_file_url "http://epigenomegateway.wustl.edu/browser/repeat/_d"


/* miscellaneous */
#define SQUAWK 1 // set to 1 for squawking
#define CHECKCPUTIME 0 // 1 for benchmarking cpu time
#define Squawk stderr // where to squawk at, stdout or stderr
#define boolean short
#define TRUE 1
#define FALSE 0


/* FIXME badly need to unify all the bed/item/node structs with region
*/

struct slNode
    { 
    /* covers tnode, and track */
    struct slNode *next;
    char *name;
	char **arr;
    };
struct tnode
    { 
    /*
	for cgiparamsl:
	t1: key
	t2: value

	for hmtk json dump:
	t1: label
	t2: md anno
	t4: filetype
	*/
    struct tnode *next;
    char *t1;
    char *t2;
    char *t3;
    int t4;
    int num;
    float fnum;
    float fnum2;
    };
struct nsortItem
	{
	/* numerical sort */
	double value;
	void *ptr; // points to original item
	};
struct genericItem
	{
	/* to include beditem and readitem 
	when only coordinate is concerned
	and when bed/sam format cannot be sured
	*/
	struct genericItem *next;
	unsigned int start;
	unsigned int stop;
	};
struct beditem
	{
	/* such and the read has no field for chrs, because they are always attached to
	some structures that identifies chr
	*/
	struct beditem *next;
	unsigned int start;
	unsigned int stop;
	char *rest;
	};
struct beditem2
	{
	// only for repeatbrowser perhaps
	struct beditem2 *next;
	char *chrom;
	unsigned int start;
	unsigned int stop;
	char strand;
	char *rest; // may be used as ID
	double score;
	};
struct tabixHandle
	{
	struct tabixHandle *next;
	tabix_t *fin;
	char *name;
	};
struct readitem
	{
	struct readitem *next;
	unsigned int start;
	/* read stop is start + strlen(seq)
	this *stop* will only be used for computing density
	will not be reported to js for bed plotting
	the actual stop need to be determined by cigar
	*/
	unsigned int stop;
	char *id;
	uint32_t flag; // flag from bam
	char *flag_str; // flag str from sam
	char *seq; // read sequence
	uint32_t *cigar; // cigar from bam
	uint32_t n_cigar;
	char *cigar_str; // cigar str from sam
	char *mismatch;
	};
struct userData
	{
	struct readitem *sl;
	};


struct nnode
    { 
    // for genestruct
    struct nnode *next;
    int chrIdx;
    int start;
    int stop;
    int flank5start; // upstream flank start, only used in genesetview
    int flank5stop;
    int flank3start; // downstream flank start, only used in genesetview
    int flank3stop;
    char strand;
    char *name;
    double *data;
	int id;
    };

struct urlpiece
    { 
    // a piece of loooong url
    int offset;
    char *text;
    };
struct track
    { 
    /* both for track and density track */
    struct track *next;
    char *name;
	char *label;
    char *urlpath; // either URL or file path
    int ft;
	int mode;
	float pscore; // positive cutoff score
	float nscore; // negative cutoff score
    double *data; // array length: dsp.usedSummaryNumber
    int gid; // for grouping tracks in htest, passed from client
    boolean flag;
    pid_t pid; // for forking on a track
    char *tmpfile; // session+track name, will hold data made by child process and later, need to be made before forking so that both parent/child can see it
    };
struct geneParam
	{
	char *query_table; // [genetype]symbol table, if null, iterate over all tables
	int query_id; // item id, if -1, will use string match
	char *query_str;
	boolean use_chromid;
	boolean use_exactmatch;
	char *whichgenepart; // null for "genebody"
	char *origintype; // null for not adding flanking
	int flank5;
	int flank3;
	};
struct gene
	{
	struct gene *next;
	char *chrom;
	int chromIdx;
	/* initially start/stop will be set to txstart/txstop
	in case of applying flanking they will be changed
	*/
	unsigned int txstart; // transcription start/stop
	unsigned int txstop;
	unsigned int report_start;
	unsigned int report_stop;
	/* when param->whichgenepart is set to other than genebodym
	report_start/stop will be used, else they are same as txstart/stop
	*/
	char strand;
	int utr5start;
	int utr5width;
	int utr3start;
	int utr3width;
	int exoncount;
	int **exons;
    int flank5start; // upstream flank start, -1 for not in use
    int flank5stop;
    int flank3start;
    int flank3stop;
	char *name;
	int id; // in the bed file and []struct table
	char *type; // could be gene table name
	char *text; // could be desc
	boolean on_normalchr;
	};
struct region
    {
    struct region *prev;
    struct region *next;
    int chromIdx;
    int bstart; // *b*oundary of this region in genome
    int bstop;
    int dstart; // *d*isplayed portion of this region
    int dstop;
	char strand; // +/-
    int flank5start; // upstream flank start, only used in genesetview
    int flank5stop;
    int flank3start; // downstream flank start, only used in genesetview
    int flank3stop;
    int summarySize;
    char *coordString; // ?
    char *name; // only for gsv
	char *name2;
	boolean flag; // in case of gsv, true/false for whether is gene
    // will only be used when zoom level changed, to indicate if this region is start of view region
    boolean isViewStart;
    int viewStartCoord;
    double fvalue; // a value for sorting items
    };
struct point
    {
    int chromIdx;
    int coord;
    boolean hitBorder;
    };
/* |--left wing--|========DSP========|--right wing--|
            dsp start            dsp stop
   start/stop index of dsp, need to be exactly hmSpan in length
*/
struct chritemsl
	{
	struct chritemsl *next;
	int chromIdx;
	struct beditem *itemsl;
	};
struct displayedRegion
    {
    // start/stop will be used in determining head/tail
    // but will not be used for iteration purpose
    struct point *start;
    struct point *stop;
	struct chritemsl *chritemsl;
    struct region *head; // 5' end
    struct region *tail; // 3' end
    long entireLength;
    int usedSummaryNumber; // corresponds to number of pixels horizontally in heatmap
    boolean atbplevel; // if showing basepair, in this case, summary size will equal seq length
    int runmode;
    char *juxtkurlpath;
	int juxtkft; // file type of the track used in juxtaposition
	tabix_t *tabix; // tabix file handle for region computing during juxtaposition
	/* for restricting reading lots of items
	when showing longrange tracks for browser
	*/
	struct nnode *dspFilter;
    };
struct heatmap
    {
    struct displayedRegion *dsp;
    struct track *trackSl;
    struct slNode *genetrack_sl;
    // decor tracks grouped by file type (indicated by file type number)
    struct track *decor0;
    struct track *decor1;
    struct track *decor8;
    struct track *decor4;
    struct track *decor5;
    struct track *decor9;
    struct track *decor10;
	struct track *decor17;
	struct track *decor18;
    struct slNode *datasetSl; // list of datasets in use
    };



/**********************************
       global variables
 *********************************/
const char *CentralDB="WashUGb";
unsigned int hmSpan = 0;
MYSQL *conn=NULL;
struct chrInfoNode /* to be used in array */
    {
    char *name;
    unsigned int length;
    };
struct chrInfoNode **chrInfo; // an array
int lastChrIdx=-1;
unsigned int __skipped_num=0;
struct point leftBorder, rightBorder;
boolean trigger_getdata;
char *bbiDir;
char *seqPath;
char *session = NULL;
int statusId = -1;
struct slNode *genetrack_sl = NULL;
struct slNode *normalchr_sl=NULL;
struct tnode *cgiparamsl=NULL;

/* set to not null to tell current scaffold is year:monthDay
hard-coded: regions are month-based
january: 1 to 32, so that length is correct (32-1=31)
int **yearMonth=NULL;
int __startyear=-1, __stopyear=-1;
*/

/**********************************
          tiny functions 
 *********************************/
void done()
{
if(SQUAWK) fputs("done\n", stderr);
}
void errabort(char *msg)
{
fprintf(stderr, "Aborted: %s\n", msg);
exit(0);
}

boolean isCustom(int ft)
{
switch(ft) {
case FT_bedgraph_c:
case FT_cat_c:
case FT_bigwighmtk_c:
case FT_bed_c:
case FT_lr_c:
case FT_sam_c:
	return TRUE;
default:
	return FALSE;
}
}


long strMayPositiveInt(char *what)
{
// return -1 to indicate error
char *tail;
long value = strtol(what, &tail, 0);
if(tail[0] == '\0')
    return value;
return -1;
}


char *getSeq(tabix_t *fin, char *chrom, unsigned int start, unsigned int stop)
{
/* free after use
*/
int chridx, begin, end, len, i, j=0;
char *tmp;
assert(asprintf(&tmp,"%s:%d-%d",chrom,start,stop)>0);
if(ti_parse_region(fin->idx, tmp, &chridx, &begin, &end)<0)
	{
	fprintf(stderr, "%s: coord parsing error (%s)\n", __FUNCTION__, tmp);
	return NULL;
	}
free(tmp);
ti_iter_t iter=ti_queryi(fin, chridx, begin, end);
char *seq=malloc(sizeof(char)*(1+stop-start));
char delim[]="\t\n";
char *tok;
const char *row;
while((row=ti_read(fin, iter, &len)) != 0)
	{
	tmp=strdup(row);
	strtok(tmp, delim);
	assert((tok=strtok(NULL,delim))!=NULL);
	assert((begin=strMayPositiveInt(tok))!=-1);
	assert((tok=strtok(NULL,delim))!=NULL);
	assert((end=strMayPositiveInt(tok))!=-1);
	if((tok=strtok(NULL,delim))==NULL) continue;
	for(i=max(start,begin);i<min(stop,end);i++)
		seq[j++]=tok[i-begin];
	}
seq[j]='\0';
//fprintf(stderr, "%d-%d (%d), %d\n", start, stop, (stop-start), j);
return seq;
}

void slReverse(void *sl)
{
struct slNode **ppt = (struct slNode **)sl;
struct slNode *newList = NULL;
struct slNode *el, *next;

next = *ppt;
while (next != NULL)
{    
	el = next;
	next = el->next;
	el->next = newList;
	newList = el;
}    
*ppt = newList;
}

int slCount2(void *in)
{
// works for things similar as slNode
struct slNode *sl=(struct slNode *)in;
int a=0;
for(; sl!=NULL; sl=sl->next)
	a++;
return a;
}
void slFree(void *in)
{
// works for things similar as slNode
struct slNode *sl=(struct slNode *)in;
struct slNode *n;
while(sl!=NULL)
	{
	n=sl->next;
	free(sl);
	sl=n;
	}
}
int tnodesl_search4num(struct tnode *sl, char *what)
{
/* return -1 if not found
so it should work only for positive integer values
*/
for(; sl!=NULL; sl=sl->next)
	{
	if(strcasecmp(sl->t1, what)==0)
		return sl->num;
	}
return -1;
}

int nsort_compAsc(const void *a, const void *b)
{
// numerical sort, ascending
const struct nsortItem *aa=a;
const struct nsortItem *bb=b;
return (aa->value > bb->value) - (aa->value < bb->value);
}
int nsort_compDsc(const void *a, const void *b)
{
// numerical sort, descending
const struct nsortItem *aa=a;
const struct nsortItem *bb=b;
return (bb->value > aa->value) - (bb->value < aa->value);
}

struct genericItem *beditemsort_startAsc(struct genericItem *sl)
{
/* apply numerical sort to start of generic items
yield new list in ascending order
original list is *altered*
*/
if(sl==NULL) return NULL;
int count=0;
struct genericItem *t;
for(t=sl; t!=NULL; t=t->next) count++;
struct nsortItem *array=malloc(sizeof(struct nsortItem)*count);
if(array==NULL)
	{
	fprintf(stderr, "%s: out of mem\n", __FUNCTION__);
	exit(0);
	}
int i=0;
for(t=sl; t!=NULL; t=t->next)
	{
	array[i].value=t->start;
	array[i++].ptr=(void *)t;
	}
qsort(array, count, sizeof(struct nsortItem), nsort_compDsc);
struct genericItem *newsl=NULL;
for(i=0; i<count; i++)
	{
	t=(struct genericItem *)array[i].ptr;
	t->next=newsl;
	newsl=t;
	}
free(array);
return newsl;
}

struct genericItem *beditemsort_stopDsc(struct genericItem *sl)
{
/* apply numerical sort to stop of bed items
yield new list in descending order
original list is *altered*
*/
if(sl==NULL) return NULL;
int count=0;
struct genericItem *t;
for(t=sl; t!=NULL; t=t->next) count++;
struct nsortItem *array=malloc(sizeof(struct nsortItem)*count);
if(array==NULL)
	{
	fprintf(stderr, "%s: out of mem\n", __FUNCTION__);
	exit(0);
	}
int i=0;
for(t=sl; t!=NULL; t=t->next)
	{
	array[i].value=t->stop;
	array[i++].ptr=(void *)t;
	}
qsort(array, count, sizeof(struct nsortItem), nsort_compAsc);
struct genericItem *newsl=NULL;
for(i=0; i<count; i++)
	{
	t=(struct genericItem *)array[i].ptr;
	t->next=newsl;
	newsl=t;
	}
free(array);
return newsl;
}

char *makefilepath(char *trackname, int ft)
{
char *p;
switch(ft)
	{
	case FT_bed_c:
	case FT_bedgraph_c:
	case FT_lr_c:
	case FT_sam_c:
	case FT_bam_c:
	case FT_cat_c:
	case FT_bigwighmtk_c:
		return trackname;
	case FT_bed_n:
		assert(asprintf(&p, "%s%s.gz", bbiDir, trackname)>0);
		return p;
	case FT_bedgraph_n:
		assert(asprintf(&p, "%s%s.gz", bbiDir, trackname)>0);
		return p;
	case FT_qdecor_n:
		assert(asprintf(&p, "%s%s.gz", bbiDir, trackname)>0);
		return p;
	case FT_lr_n:
		assert(asprintf(&p, "%s%s.gz", bbiDir, trackname)>0);
		return p;
	case FT_cat_n:
		assert(asprintf(&p, "%s%s.gz", bbiDir, trackname)>0);
		return p;
	case FT_sam_n:
		assert(asprintf(&p, "%s%s.gz", bbiDir, trackname)>0);
		return p;
	case FT_bam_n:
		assert(asprintf(&p, "%s%s.bam", bbiDir, trackname)>0);
		return p;
	case FT_bigwighmtk_n:
		assert(asprintf(&p, "%s%s.bigWig", bbiDir, trackname)>0);
		return p;
	default:
		fprintf(stderr, "%s: %s %d\n", __FUNCTION__, trackname, ft);
		exit(0);
	}
return NULL;
}



float standardDeviation(double *data, int len)
{
if(len <= 1) return 0;
double ave = 0;
int i;
for(i=0; i<len; i++) ave += data[i];
ave /= len;
double std = 0;
for(i=0; i<len; i++)
    std += (data[i]-ave) * (data[i]-ave);
return sqrt(std/len);
}
int urlpiece_cmp(const void *a, const void *b)
{
const struct urlpiece *u1 = (const struct urlpiece *)a;
const struct urlpiece *u2 = (const struct urlpiece *)b;
return (u1->offset > u2->offset) - (u1->offset < u2->offset);
}
int getRegionNumber(struct displayedRegion *dsp)
{
int i=0;
struct region *r;
for(r=dsp->head; r!=NULL; r=r->next) i++;
return i;
}
void dspAddRegionHead(struct displayedRegion *dsp, struct region *r)
{
r->next = dsp->head;
r->prev = NULL;
dsp->head = r;
if(dsp->tail == NULL)
    dsp->tail = r;
else
    r->next->prev = r;
}
void dspAddRegionTail(struct displayedRegion *dsp, struct region *r)
{
r->prev = dsp->tail;
r->next = NULL;
dsp->tail = r;
if(dsp->head == NULL)
    dsp->head = r;
else
    r->prev->next = r;
}

int intMin(int a, int b)
{
if(a > b) return b;
return a;
}
int intMax(int a, int b)
{
if(a > b) return a;
return b;
}

int decorMode2num(char *str)
{
// convert string "thin" into "2"
if(strcasecmp(str, "thin")==0) return M_thin;
if(strcasecmp(str, "full")==0) return M_full;
if(strcasecmp(str, "density")==0) return M_den;
if(strcasecmp(str, "arc")==0) return M_arc;
if(strcasecmp(str, "trihm")==0) return M_trihm;
if(strcasecmp(str, "hide")==0) return M_hide;
if(strcasecmp(str, "show")==0) return M_show;
return M_error;
}

int *str2intArr(char *string, int howmany)
{
/* "1,2,3" to [1,2,3], for string such as "exonStarts"
   return pointer to an array
   free after use

   fields must be delimited by comma
 */
int *number = malloc(sizeof(int)*howmany);
char delim[] = ",";
char *tok = strtok(string, delim); assert(tok!=NULL);
assert((number[0]=strMayPositiveInt(tok))!=-1);
int i=1;
for(; i<howmany; i++)
    {
    tok = strtok(NULL, delim); assert(tok!=NULL);
    assert((number[i]=strMayPositiveInt(tok))!=-1);
    }
return number;
}

/*** cgi ***/
boolean cgiVarExists(char *what)
{
struct tnode *t;
for(t=cgiparamsl; t!=NULL; t=t->next)
	{
	if(strcasecmp(what, t->t1)==0) return TRUE;
	}
return FALSE;
}

char *cgiString(char *what)
{
/* return a copy of hash data
free after use
*/
struct tnode *t;
for(t=cgiparamsl; t!=NULL; t=t->next)
	{
	if(strcasecmp(what, t->t1)==0) return strdup(t->t2);
	}
return NULL;
}

int cgiInt(char *what)
{
struct tnode *t;
for(t=cgiparamsl; t!=NULL; t=t->next)
	{
	if(strcasecmp(what, t->t1)==0) return strtol(t->t2,NULL,0);
	}
fprintf(stderr, "integer-value param missing (%s)\n", what);
exit(0);
}

double cgiDouble(char *what)
{
struct tnode *t;
for(t=cgiparamsl; t!=NULL; t=t->next)
	{
	if(strcasecmp(what, t->t1)==0) return strtod(t->t2,NULL);
	}
fprintf(stderr, "float-value param missing (%s)\n", what);
exit(0);
}

void cgiVarSet(char *key, char *data)
{
struct tnode *t=malloc(sizeof(struct tnode));
t->t1=strdup(key);
t->t2=strdup(data);
t->next=cgiparamsl;
cgiparamsl=t;
}

void strDecode(char *in, char *out, int inLength)
{
char c;
int i;
for (i=0; i<inLength;++i)
	{    
	c = *in++;
	if (c == '+') 
		{
		*out++ = ' '; 
		}
	else if (c == '%') 
		{
		int code;
		if (sscanf(in, "%2x", &code) != 1)
			code = '?'; 
		in += 2;
		i += 2;
		*out++ = code;
		}    
	else 
		{
		*out++ = c; 
		}
	}
*out++ = 0; 
}

/*** mysql ***/
boolean mysqlConnect(const char *dbname)
{
conn=mysql_init(NULL);
if(mysql_real_connect(conn, Mysqlserver, Mysqluser, Mysqlpswd, dbname, 0, NULL, 0))
	return TRUE;
fprintf(stderr, "cannot connect to db (%s)\n", dbname);
return FALSE;
}

void mysqlUpdate(char *query)
{
/* replace sqlUpdate()
*/
if(mysql_query(conn, query))
	{
	fprintf(stderr, "%s\n", mysql_error(conn));
	exit(0);
	}
}

MYSQL_RES *mysqlGetResult(char *query)
{
/* replaces sqlGetResult() in kent source
upon failure, return NULL
*/
if(mysql_query(conn, query))
	{
	fprintf(stderr, "%s\n", mysql_error(conn));
	return NULL;
	}
MYSQL_RES *re=mysql_use_result(conn);
if(re==NULL)
	{
	fprintf(stderr, "mysql_use_result: %s (%s)\n", mysql_error(conn), query);
	return NULL;
	}
return re;
}

char *mysqlGetOnestring(char *query)
{
/* upon any failure, return NULL 
free after use
*/
MYSQL_RES *sr = mysqlGetResult(query);
if(sr == NULL)
    return NULL;
char **row=mysql_fetch_row(sr);
if(row==NULL || row[0]==NULL)
	{
	mysql_free_result(sr);
	return NULL;
	}
char *re = strdup(row[0]);
mysql_free_result(sr);
return re;
}

int mysqlGetPositiveInteger(char *query)
{
/* upon any failure, return -1 */
MYSQL_RES *sr = mysqlGetResult(query);
if(sr==NULL)
	return -1;
char **row;
if((row=mysql_fetch_row(sr)) == NULL)
    {
    mysql_free_result(sr);
    return -1;
    }
if(row[0] == NULL)
    {
	fprintf(stderr, "%s: row[0]==null\n", __FUNCTION__);
    mysql_free_result(sr);
    return -1;
    }
int value = strMayPositiveInt(row[0]);
mysql_free_result(sr);
return value;
}
int mysqlGetGenesGroupId()
{
// there should be a decor track group "Genes", fetch its ID
return mysqlGetPositiveInteger("select id from gfGrouping where name=\"Genes\"");
}

boolean mysqlResultAffirmative(char *query)
{
MYSQL_RES *sr = mysqlGetResult(query);
if(sr==NULL)
	return FALSE;
boolean re = (mysql_fetch_row(sr) != NULL);
mysql_free_result(sr);
return re;
}

boolean sessionExists(char *session)
{
char *query;
if(asprintf(&query, "select session from session where session=\"%s\"", session)<0) errabort("bae");
boolean re=mysqlResultAffirmative(query);
free(query);
return re;
}

boolean sessionStatusExists(char *session, int status)
{
if(!sessionExists(session)) return FALSE;
char *query;
if(asprintf(&query, "select statusId from session where session=\"%s\" and statusId=%d", session, status)<0) errabort("bae");
boolean re=mysqlResultAffirmative(query);
free(query);
return re;
}

char *getItemlistfromdb(char *session, int statusId)
{
char *query;
assert(asprintf(&query, "select itemlist from genesetRuntime where session=\"%s\" and statusId=%d", session, statusId)>0);
char *re = mysqlGetOnestring(query);
free(query);
return re;
}



void showRegionSl(struct displayedRegion *dsp)
{
fprintf(stderr, "showRegionSl:\nusedSummaryNumber: %d\nentireLength: %ld\n", dsp->usedSummaryNumber, dsp->entireLength);
struct region *r;
int i = 0;
for(r=dsp->head; r!=NULL; r=r->next)
    fprintf(stderr, "(%d) %s [%d,%d >> %d,%d]\tlen: %d\tsp:%d\n", i++, chrInfo[r->chromIdx]->name, r->bstart,r->dstart, r->dstop,r->bstop, r->dstop-r->dstart, r->summarySize);

fputs("showRegionSl: okay\n",stderr);
}
void showTrackSl(struct track *sl)
{
int i=0;
for(; sl!=NULL; sl=sl->next)
    fprintf(stderr, "%d\t%s\n", ++i, sl->name);
}



/**********************************
         small functions 
 *********************************/

struct beditem *slSort_beditem(struct beditem *sl, int(*compare)(const void *el1, const void *el2))
{
/* the returned list is generated in reverse order
so besure to sort in *reverse* order so as to get *forward* order from returned list
*/
if(sl==NULL) return NULL;
int count=0;
struct beditem *t;
for(t=sl; t!=NULL; t=t->next) count++;
struct beditem **itemArr=malloc(sizeof(struct beditem *)*count);
if(itemArr==NULL)
	{
	fprintf(stderr, "%s: out of mem\n", __FUNCTION__);
	exit(0);
	}
int i=0;
for(t=sl; t!=NULL; t=t->next)
	itemArr[i++]=t;
qsort(itemArr, count, sizeof(struct beditem *), compare);
struct beditem *newsl=NULL;
for(i=0; i<count; i++)
	{
	itemArr[i]->next=newsl;
	newsl=itemArr[i];
	}
free(itemArr);
return newsl;
}

int strMayChrID(char *what)
{
int i = 0;
for(; i<=lastChrIdx; i++)
    {
    if(strcasecmp(chrInfo[i]->name, what) == 0)
        return i;
    }
return -1;
}

int *dissectCoordString(char *item)
{
/* input a string, supposedly "chr:start-stop strand"
return 3-element int array: [chr idx, start, stop]
the value will be -1 to indicate any error in input

strand is optional, convert to:
1: forward
0: reverse
2: n/a, treat as 1 during use

string will be broken
chrom list must be ready before calling
*/
size_t s = strlen(item);
char strand = item[s-1];
char *chr, *start, *stop;
char delim[] = " \t-:";
if((chr=strtok(item, delim)) == NULL)
    return NULL;
if((start=strtok(NULL, delim)) == NULL)
    return NULL;
if((stop=strtok(NULL, delim)) == NULL)
    return NULL;
int *arr = malloc(sizeof(int)*4);
arr[0] = strMayChrID(chr);
if(arr[0]==-1)
	{
	// must not proceed
	arr[1]=arr[2]=-1;
	return arr;
	}
arr[1] = strMayPositiveInt(start);
if(arr[1]==-1 || arr[1]>chrInfo[arr[0]]->length) return NULL;
arr[2] = strMayPositiveInt(stop);
if(arr[2]==-1 || arr[2]>chrInfo[arr[0]]->length || arr[2]<=arr[1]) return NULL;
if(strand=='+' || strand=='-')
    arr[3] = strand=='+' ? 1 : 0;
else
    arr[3] = 2;
return arr;
}



boolean slLookup(void *sl, char *what, boolean caseSensitive)
{
/* caseSensitive: if true will use strcmp, else use strcasecmp
 */
struct slNode *this = (struct slNode *)sl;
for(; this!=NULL; this=this->next)
    {
    if(caseSensitive)
        {
        if(strcmp(this->name, what) == 0)
            return TRUE;
	}
    else
        {
	if(strcasecmp(this->name, what) == 0)
	    return TRUE;
	}
    }
return FALSE;
}

struct slNode *slSearch(void *sl, char *what)
{
struct slNode *this = (struct slNode *)sl;
for(; this!=NULL; this=this->next)
    {
    if(strcmp(this->name, what) == 0)
        return this;
    }
return NULL;
}


/*
boolean binaryFileIsInvalid(char *f, boolean isURL)
{
// takes file path or URL, return true if file has problem
boolean fail = FALSE;
if(isURL)
    {
    struct udcFile *tmp = udcFileMayOpen(f, "/var/www/trash");
    if(tmp == NULL)
        fail = TRUE;
    else
        udcFileClose(&tmp);
    }
else
    {
    FILE *fin = fopen(f, "rb");
    if(fin == NULL) 
        fail = TRUE;
    else
        fclose(fin);
    }
return fail;
}
*/


tabix_t *tabixOpen(char *urlpath)
{
/* open tabix file and load index!
to be used repeatedly, free after use
*/
tabix_t *fin=ti_open(urlpath,0);
if(fin==0)
	return NULL;
if(ti_lazy_index_load(fin)<0)
	return NULL;
return fin;
}

struct readitem *tabixQuery_sam(tabix_t *fin, char *querycoord)
{
/* tabix the sam file
XXX the sam/tabix format is abandoned, use bam instead
*/
int chridx, begin, end;
if(ti_parse_region(fin->idx, querycoord, &chridx, &begin, &end)<0)
	{
	fprintf(stderr, "%s: coord parsing error (%s)\n", __FUNCTION__, querycoord);
	// don't abort, this could be the case when a chr (eg chrM) is not in the data file
	return NULL;
	}
struct readitem *sl=NULL, *read;
ti_iter_t iter=ti_queryi(fin, chridx, begin, end);
const char *row;
char delim[]="\t";
char *tmpstr, *tok;
int len;
while((row=ti_read(fin, iter, &len)) != 0)
	{
	tmpstr=strdup(row);
	/** FIXME memory leak here **/
	if(tmpstr==NULL)
		{
		fprintf(stderr, "%s: mem\n", __FUNCTION__);
		exit(0);
		}
	read=malloc(sizeof(struct readitem));
	// read id
	if((tok=strtok(tmpstr, delim))==NULL) continue;
	read->id=tok;
	// flag
	if((tok=strtok(NULL, delim))==NULL) continue;
	read->flag_str=tok;
	// flag int value is left unassigned
	// chr name
	strtok(NULL,delim);
	// start
	if((tok=strtok(NULL, delim))==NULL) continue;
	if((read->start=strMayPositiveInt(tok))==-1) continue;
	read->start--;
	// mapq
	if((tok=strtok(NULL, delim))==NULL) continue;
	// cigar
	if((tok=strtok(NULL, delim))==NULL) continue;
	read->cigar=NULL; // bam
	read->cigar_str=tok;
	strtok(NULL,delim);
	strtok(NULL,delim);
	strtok(NULL,delim);
	// seq
	if((tok=strtok(NULL, delim))==NULL) continue;
	read->seq=tok;
	read->stop=read->start+strlen(tok);
	// qual
	strtok(NULL,delim);
	// optional alignment, only chop one field
	read->mismatch=NULL;
	while((tok=strtok(NULL,delim))!=NULL)
		{
		if(tok[0]=='M' && tok[1]=='D')
			{
			read->mismatch=tok;
			break;
			}
		}
	read->next=sl;
	sl=read;
	}
ti_iter_destroy(iter);

return sl;
}


static int bam_fetch_func(const bam1_t *b,void *data)
{
if (b->core.tid < 0) return 0;
const bam1_core_t *c = &b->core;

struct readitem *r=malloc(sizeof(struct readitem));
r->cigar_str=NULL; // sam
r->n_cigar=c->n_cigar;
r->cigar=malloc(sizeof(uint32_t)*c->n_cigar);
uint32_t *cigar = bam1_cigar(b);
int i;
for (i=0; i < c->n_cigar; ++i)
	(r->cigar)[i]=cigar[i]; // copy cigar

uint8_t *seq=bam1_seq(b);
r->seq=malloc(sizeof(char)*(c->l_qseq+1));
for(i=0; i<c->l_qseq; i++)
	(r->seq)[i]=bam_nt16_rev_table[bam1_seqi(seq,i)];
(r->seq)[i]='\0';

r->id=strdup(bam1_qname(b));
r->start=c->pos;
r->stop=c->pos+c->l_qseq;
r->flag=c->flag;
r->flag_str=NULL;
uint8_t *m=bam_aux_get(b,"MD");
if(m)
	r->mismatch=strdup((char *)m);
else
	r->mismatch=NULL;
struct userData *udata=(struct userData *)data;
r->next=udata->sl;
udata->sl=r;
return 0;
}

struct readitem *bamQuery_region(samfile_t *fp, bam_index_t *idx, char *coord)
{
// will not fill chromidx
int ref,beg,end;
bam_parse_region(fp->header,coord,&ref,&beg,&end);
if(ref<0)
	return NULL;
struct userData *d=malloc(sizeof(struct userData));
d->sl=NULL;
bam_fetch(fp->x.bam,idx,ref,beg,end,d,bam_fetch_func);
return d->sl;
}

bam_index_t *bam_parseIndex(char *urlpath, int ft)
{
if(ft==FT_bam_n)
	return bam_index_load(urlpath);
/* dissect url and move to that directory
*/
char *urlcopy=strdup(urlpath);
char delim[]="/";
char *tok=strtok(urlcopy,delim);
char *deposit_dir=trashDir;
struct stat sbuf;
while(1)
	{
	assert(asprintf(&deposit_dir,"%s/%s",deposit_dir,tok)>0);
	if(stat(deposit_dir,&sbuf)==-1)
		{
		// create new dir
		if(mkdir(deposit_dir,S_IRWXU)!=0)
			{
			fprintf(stderr,"failed to create caching directory for bam file\n");
			return NULL;
			}
		}
	tok=strtok(NULL,delim);
	if(tok==NULL)
		break;
	}
// ready to change dir
char *olddir=getcwd(NULL,0);
if(chdir(deposit_dir)!=0)
	{
	fprintf(stderr, "failed to change to bam caching dir: %s\n", deposit_dir);
	return NULL;
	}
bam_index_t *i=bam_index_load(urlpath);
if(chdir(olddir)!=0)
	{
	fputs("failed to change back dir afterwards\n",stderr);
	return NULL;
	}
return i;
}

struct beditem *tabixQuery_bed(tabix_t *fin, char *querycoord)
{
/* only query bed file
generic interval-based query over tabix file
query coord must be in the format of chr1:234-567
return NULL upon any error

the returned list is not sorted, it will be sorted on demand
as different sorting methods will be used,
e.g. sort by start, or stop
*/
int chridx, begin, end;
if(ti_parse_region(fin->idx, querycoord, &chridx, &begin, &end)<0)
	{
	fprintf(stderr, "%s: coord parsing error (%s)\n", __FUNCTION__, querycoord);
	// don't abort, this could be the case when a chr (eg chrM) is not in the data file
	return NULL;
	}
struct beditem *sl=NULL, *bi;
ti_iter_t iter=ti_queryi(fin, chridx, begin, end);
const char *row;
char delim[]="\t";
char *tmpstr, *tok, *rest;
int len;
boolean haveInvalid=FALSE;
while((row=ti_read(fin, iter, &len)) != 0)
	{
	tmpstr=strdup(row);
	/** FIXME memory leak here
	memory is allocated for tmpstr
	but it got chopped up
	don't know how to free its pieces
	*/
	if(tmpstr==NULL)
		{
		fprintf(stderr, "%s: mem\n", __FUNCTION__);
		exit(0);
		}
	bi=malloc(sizeof(struct beditem));
	// chr, discard
	if((rest=strchr(tmpstr, '\t'))==NULL)
		{ haveInvalid=TRUE; continue; }
	strtok(tmpstr, delim);
	tmpstr=++rest;
	// start
	if((rest=strchr(tmpstr, '\t'))==NULL)
		{ haveInvalid=TRUE; continue; }
	tok=strtok(tmpstr, delim);
	if((bi->start=strMayPositiveInt(tok))==-1)
		{ haveInvalid=TRUE; continue; }
	tmpstr=++rest;
	// stop
	if((rest=strchr(tmpstr, '\t'))==NULL)
		{ haveInvalid=TRUE; continue; }
	tok=strtok(tmpstr, delim);
	if((bi->stop=strMayPositiveInt(tok))==-1)
		{ haveInvalid=TRUE; continue; }
	bi->rest=++rest;

	bi->next=sl;
	sl=bi;
	}
ti_iter_destroy(iter);

if(haveInvalid)
	fprintf(stderr, "%s: failed to parse certain lines in %s\n", __FUNCTION__, fin->fn);

return sl;
}

double tabixQuery_bedgraph_singlepoint(tabix_t *fin, char *chrom, unsigned int start, unsigned int stop)
{
/* only get one data point, extremely simplified
return data directly
return 0 for any failure
*/
char *tmp;
assert(asprintf(&tmp, "%s:%d-%d", chrom, start, stop)>0);
int chridx, begin, end;
if(ti_parse_region(fin->idx, tmp, &chridx, &begin, &end)<0)
	return 0;
ti_iter_t iter=ti_queryi(fin, chridx, begin, end);
if(iter==0) return 0;
char delim[]="\t";
const char *row;
char *tok;
double sumarea=0, value;
int sumbp=0;
int istart, istop, jstart, jstop, len;
/* in fact, spnum should alway <= bp length, it should never be greater than bp length
*/
while((row=ti_read(fin, iter, &len)) != 0)
	{
	// chr, discard
	tmp=strdup(row);
	strtok(tmp, delim);
	if((tok=strtok(NULL,delim))==NULL) continue;
	if((istart=strMayPositiveInt(tok))==-1) continue;
	if((tok=strtok(NULL,delim))==NULL) continue;
	if((istop=strMayPositiveInt(tok))==-1) continue;
	if((tok=strtok(NULL,delim))==NULL) continue;
	//fprintf(stderr, ">%d %d\n", istart, istop);
	value=strtod(tok, NULL);
	jstart=max(start,istart);
	jstop=min(stop,istop);

	sumarea+=(jstop-jstart)*value;
	sumbp+=(jstop-jstart);
	}
ti_iter_destroy(iter);
if(sumbp==0) return 0;
return sumarea/((double)sumbp);
}

void tabixQuery_bedgraph(tabix_t *fin, char *chrom, unsigned int start, unsigned int stop, int spnum, double *data)
{
/* only query bedgraph file
data must be already initialized with 0
*/
char *tmp;
assert(asprintf(&tmp, "%s:%d-%d", chrom, start, stop)>0);

int chridx, begin, end;
if(ti_parse_region(fin->idx, tmp, &chridx, &begin, &end)<0)
	{
	fprintf(stderr, "%s: coord parsing error (%s)\n", __FUNCTION__, tmp);
	// don't abort, this could be the case when a chr (eg chrM) is not in the data file
	return;
	}
float step=((float)(stop-start))/spnum;
ti_iter_t iter=ti_queryi(fin, chridx, begin, end);
char delim[]="\t";
const char *row;
char *tok;
int len;
unsigned int i,istart,istop,jstart,jstop;
boolean atbplevel=spnum>=(stop-start);
double value;
double *sumvalue=NULL, *sumbp=NULL;
if(!atbplevel)
	{
	if((sumvalue=malloc(sizeof(double)*spnum))==NULL)
		{
		fprintf(stderr, "%s: sumvalue out of mem\n", __FUNCTION__);
		exit(0);
		}
	if((sumbp=malloc(sizeof(double)*spnum))==NULL)
		{
		fprintf(stderr, "%s: sumbp out of mem\n", __FUNCTION__);
		exit(0);
		}
	for(i=0; i<spnum; i++)
		{
		sumvalue[i]=0;
		sumbp[i]=0;
		}
	}
/* in fact, spnum should alway <= bp length, it should never be greater than bp length
*/
while((row=ti_read(fin, iter, &len)) != 0)
	{
	// chr, discard
	tmp=strdup(row);
	strtok(tmp, delim);
	if((tok=strtok(NULL,delim))==NULL) continue;
	if((istart=strMayPositiveInt(tok))==-1) continue;
	if((tok=strtok(NULL,delim))==NULL) continue;
	if((istop=strMayPositiveInt(tok))==-1) continue;
	if((tok=strtok(NULL,delim))==NULL) continue;
	//fprintf(stderr, ">%d %d\n", istart, istop);
	value=strtod(tok, NULL);
	jstart=max(start,istart);
	jstop=min(stop,istop);

	if(atbplevel)
		{
		for(i=jstart; i<jstop; i++)
			data[i-start]=value;
		}
	else
		{
		// i is summary point idx
		for(i=floor((jstart-start)/step); i<=floor((jstop-start)/step); i++)
			{
			if(i>=spnum) break;
			/* overlap of the bedgraph item with the bin
			*** type cast must be enforced ***/
			double ostart=max((double)start+(double)i*step, (double)jstart);
			double ostop=min((double)start+(double)(i+1)*step, (double)jstop);
			//if(ostart>=ostop) fprintf(stderr, "%f %f\n", ostart, ostop);
			sumvalue[i]+=value*(ostop-ostart);
			sumbp[i]+=ostop-ostart;
			}
		}
	}
ti_iter_destroy(iter);
if(!atbplevel)
	{
	for(i=0; i<spnum; i++)
		{
		if(sumbp[i]>0)
			data[i]=sumvalue[i]/sumbp[i];
		}
	free(sumvalue);
	free(sumbp);
	}
}

void *tabixQuery_regioncoord(tabix_t *fin, int ft, char *chr, unsigned int start, unsigned int stop)
{
char *query;
if(asprintf(&query, "%s:%d-%d", chr, start, stop)<0)
	fputs("mem",stderr);
if(ft==FT_sam_n||ft==FT_sam_c)
	return (void *)tabixQuery_sam(fin, query);
else if(ft==FT_bed_n||ft==FT_bed_c||ft==FT_lr_n||ft==FT_lr_c)
	return (void *)tabixQuery_bed(fin, query);
fprintf(stderr, "%s: ft not supported (%d)\n", __FUNCTION__, ft);
exit(0);
}

void *tabixQuery_region(tabix_t *fin, int ft, struct region *r)
{
return tabixQuery_regioncoord(fin, ft, chrInfo[r->chromIdx]->name, r->dstart, r->dstop);
}

void *tabixQuery_dsp(struct displayedRegion *dsp, char *urlpath, int ft)
{
/* only open file once, handle is used to perform queries over each region in dsp
but in case of bam, use samtools
return an array, length of region #
not sorted
free after use
*/
struct region *r;
int regioncount=0, dataidx=0;
for(r=dsp->head; r!=NULL; r=r->next)
	regioncount++;
void **data;

if(ft==FT_sam_n||ft==FT_sam_c||ft==FT_bam_n||ft==FT_bam_c)
	{
	data=malloc(sizeof(struct readitem *)*regioncount);
	}
else if(ft==FT_bed_n||ft==FT_bed_c||ft==FT_lr_n||ft==FT_lr_c)
	{
	data=malloc(sizeof(struct beditem *)*regioncount);
	}
else
	{
	fprintf(stderr, "%s: ft not supported (%d)\n", __FUNCTION__, ft);
	exit(0);
	}
assert(data!=NULL);

tabix_t *fin=NULL;
samfile_t *fp=NULL;
bam_index_t *bamidx=NULL;

if(ft==FT_bam_n || ft==FT_bam_c)
	{
	if((fp=samopen(urlpath,"rb",0))==0)
		{
		return NULL;
		}
	if((bamidx=bam_parseIndex(urlpath,ft))==NULL)
		return NULL;
	}
else
	{
	fin=tabixOpen(urlpath);
	if(fin==NULL)
		return NULL;
	}

for(r=dsp->head; r!=NULL; r=r->next)
	{
	//fprintf(stderr, "querying %s:%d-%d\n", chrInfo[r->chromIdx]->name,r->dstart,r->dstop);
	if(fin!=NULL)
		{
		data[dataidx]=tabixQuery_region(fin, ft, r);
		}
	else
		{
		char *tmp;
		assert(asprintf(&tmp,"%s:%d-%d",chrInfo[r->chromIdx]->name,r->dstart,r->dstop)>0);
		data[dataidx]=bamQuery_region(fp,bamidx,tmp);
		free(tmp);
		}
	//fputs("done\n", stderr);
	dataidx++;
	}

if(fin!=NULL)
	{
	ti_close(fin);
	}
else
	{
	bam_index_destroy(bamidx);
	samclose(fp);
	}

return data;
}

void tabixFreeResult(struct beditem *sl)
{
// TODO
struct beditem *bi,*prev=NULL;
for(bi=sl; bi!=NULL; bi=bi->next)
	{
	if(prev!=NULL) free(prev);
	prev=bi;
	}
if(prev!=NULL) free(prev);
}


void bigwigQuery(char *urlpath, char *chrom, unsigned int start, unsigned int stop, unsigned int spnum, double *data)
{
// space alien
// dummy path name conversion
int i;
char *dummyname=strdup(urlpath);
for(i=0; i<strlen(urlpath); i++)
	{
	if(urlpath[i]=='/')
		dummyname[i]='_';
	}
srand(time(0));
int rr=rand();
char *outfile;
assert(asprintf(&outfile, "%s/%s.%d", trashDir, dummyname, rr)>0);
free(dummyname);

char *command;
assert(asprintf(&command, "%s/bwquery %s %s %d %d %d %s", BINdir, urlpath, chrom, start, stop, spnum, outfile)>0);
if(system(command)==-1)
	{
	fprintf(stderr, "cannot run command (%s)\n", command);
	exit(0);
	}
free(command);

FILE *fin=fopen(outfile,"r");
char *line=malloc(1);
size_t s=0;
char delim[]="\n";
char *tok;
i=0;
while(getline(&line, &s, fin)!=-1)
	{
	assert((tok=strtok(line, delim))!=NULL);
	float v=strtod(tok,NULL);
	data[i++]=v;
	}
fclose(fin);
unlink(outfile);
free(outfile);

return;
}


double *tabixQuery_bedgraph_dsp(struct displayedRegion *dsp, char *urlpath, int ft)
{
/* fetch average data at summary points from bedgraph file, return pointer of array,
   free after use
   accepts bigWig files
   argument is url or path
   won't validate the file, must be done before calling this routine
 */

double *data = malloc(sizeof(double) * dsp->usedSummaryNumber);
int i;
for(i=0; i<dsp->usedSummaryNumber; i++) 
    data[i] = NAN;

if(ft==FT_bigwighmtk_n || ft==FT_bigwighmtk_c)
	{
	/* bigwig */
	struct region *r;
	i=0;
	for(r=dsp->head; r!=NULL; r=r->next)
		{
		if(r->summarySize==0)
			{
			if(SQUAWK)
				fprintf(stderr,"%s: skipped a 0-length region...\n", __FUNCTION__);
			continue;
			}
		bigwigQuery(urlpath, chrInfo[r->chromIdx]->name, r->dstart, r->dstop, r->summarySize, &data[i]);
		i += r->summarySize;
		}
	return data;
	}

tabix_t *fin=tabixOpen(urlpath);
if(fin==NULL)
	{
	fprintf(stderr, "%s(%s) failed\n", __FUNCTION__, urlpath);
	return NULL;
	}
i = 0;
struct region *r;
for(r=dsp->head; r!=NULL; r=r->next)
    {
    if(r->summarySize > 0)
        {
		tabixQuery_bedgraph(fin, chrInfo[r->chromIdx]->name, r->dstart, r->dstop, r->summarySize, &data[i]);
		}
    else
        {
		if(SQUAWK)
			fprintf(stderr,"%s: skipped a 0-length region...\n", __FUNCTION__);
		}
    i += r->summarySize;
    }
ti_close(fin);
return data;
}


void tabixQuery_categorical(tabix_t *fin, char *chrom, unsigned int start, unsigned int stop, int spnum, int *data)
{
/* similar as tabixQuery_bedgraph
data must be initialized to be -1 to indicate "no information"
*/
char *tmp;
assert(asprintf(&tmp, "%s:%d-%d", chrom, start, stop)>0);

int chridx, begin, end;
if(ti_parse_region(fin->idx, tmp, &chridx, &begin, &end)<0)
	{
	fprintf(stderr, "%s: coord parsing error (%s)\n", __FUNCTION__, tmp);
	// don't abort, this could be the case when a chr (eg chrM) is not in the data file
	return;
	}
float step=((float)(stop-start))/spnum;
ti_iter_t iter=ti_queryi(fin, chridx, begin, end);
char delim[]="\t";
const char *row;
char *tok;
int len;
unsigned int i,istart,istop,jstart,jstop;
boolean atbplevel=spnum>=(stop-start);

int value;
/* to properly display categorical data at low resolution
one summary point will need to only show category of max length
use the water-level method to do that...
*/
double plenArr[spnum]; // piece length
for(i=0; i<spnum; i++)
	plenArr[i]=0;

/* in fact, spnum should alway <= bp length, it should never be greater than bp length */
while((row=ti_read(fin, iter, &len)) != 0)
	{
	tmp=strdup(row);
	strtok(tmp, delim); // chr, discard
	if((tok=strtok(NULL,delim))==NULL) continue;
	if((istart=strMayPositiveInt(tok))==-1) continue;
	if((tok=strtok(NULL,delim))==NULL) continue;
	if((istop=strMayPositiveInt(tok))==-1) continue;
	if((tok=strtok(NULL,delim))==NULL) continue;
	if((value=strMayPositiveInt(tok))==-1) continue;
	jstart=max(start,istart);
	jstop=min(stop,istop);

	if(atbplevel)
		{
		for(i=jstart; i<jstop; i++)
			data[i-start]=value;
		}
	else
		{
		// i is summary point idx
		for(i=floor((jstart-start)/step); i<=floor((jstop-start)/step); i++)
			{
			if(i>=spnum) break;
			// compute the actual bp len of this item inside this summary point
			double bpl=min((double)start+(double)(i+1)*step, (double)jstop)-max((double)start+(double)i*step, (double)jstart);
			if(bpl>plenArr[i])
				{
				data[i]=value;
				plenArr[i]=bpl;
				}
			}
		}
	}
ti_iter_destroy(iter);
}

int *tabixQuery_categorical_dsp(struct displayedRegion *dsp, char *urlpath)
{
/* similar as tabixQuery_bedgraph_dsp
FIXME missing data?
 */
tabix_t *fin=tabixOpen(urlpath);
if(fin==NULL)
	{
	fprintf(stderr, "%s(%s) failed\n", __FUNCTION__, urlpath);
	return NULL;
	}
int *data = malloc(sizeof(int) * dsp->usedSummaryNumber);
int i;
for(i=0; i<dsp->usedSummaryNumber; i++) 
    data[i] = -1;
i = 0;
struct region *r;
for(r=dsp->head; r!=NULL; r=r->next)
    {
    if(r->summarySize > 0)
        {
		tabixQuery_categorical(fin, chrInfo[r->chromIdx]->name, r->dstart, r->dstop, r->summarySize, &data[i]);
		}
    else
        {
		if(SQUAWK)
			fprintf(stderr,"%s: skipped a 0-length region...\n", __FUNCTION__);
		}
    i += r->summarySize;
    }
ti_close(fin);
//if(SQUAWK) fprintf(stderr, "%s (%s)\n", __FUNCTION__, urlpath);
return data;
}

void printJson_dsp_spnum(struct displayedRegion *dsp, char *urlpath, int ft)
{
/* based on dsp, and spnum (summary size)
handles quantitative and categorical
including the enclosing square bracket
no validity check

FIXME whether to print out [] for regions with zero summarySize?
*/
if(ft==FT_cat_n || ft==FT_cat_c)
	{
	int *data=tabixQuery_categorical_dsp(dsp, urlpath);
	if(data==NULL)
		{
		printf("null");
		return;
		}
	int i=0, j=0;
	struct region *r;
	printf("[");
	for(r=dsp->head; r!=NULL; r=r->next)
		{
		if(r->summarySize == 0)
			continue;
		printf("[");
		for(j=0; j<r->summarySize; j++)
			printf("%d,", data[i++]);
		printf("],");
		}
	printf("],");
	return;
	}
double *data = tabixQuery_bedgraph_dsp(dsp, urlpath, ft);
if(data==NULL)
	{
	printf("null");
	return;
	}
int i=0, j=0;
struct region *r;
printf("[");
for(r=dsp->head; r!=NULL; r=r->next)
    {
    if(r->summarySize == 0)
        continue;
    printf("[");
    for(j=0; j<r->summarySize; j++)
		{
		if(isnan(data[i]))
			printf("NaN,");
		else
        	printf("%f,", data[i]);
		i++;
		}
    printf("],");
    }
printf("],");
}




/**************************************
            making of dsp
 **************************************/
boolean atbplevel(struct displayedRegion *dsp)
{
return dsp->usedSummaryNumber == dsp->entireLength;
}

struct chritemsl *chritemsl_searchidx(struct displayedRegion *dsp, int chrIdx)
{
struct chritemsl *ci;
for(ci=dsp->chritemsl; ci!=NULL; ci=ci->next)
	{
	if(ci->chromIdx==chrIdx)
		return ci;
	}
return NULL;
}

void chritemslAppend(struct displayedRegion *dsp, int chrIdx, struct beditem *data)
{
// add new node to head of dsp->chritemsl
struct chritemsl *ci = malloc(sizeof(struct chritemsl));
ci->chromIdx = chrIdx;
ci->itemsl = data;
ci->next = dsp->chritemsl;
dsp->chritemsl = ci;
}

void makeBorder(struct displayedRegion *dsp)
{
/* only works for genome or juxtaposition
   not gene set view
   update global variables leftBorder/rightBorder

very inefficient for the case of juxtaposition
need improvement
 */
if(dsp->runmode == RM_genome)
	{
	leftBorder.chromIdx = 0;
	leftBorder.coord = 0;
	rightBorder.chromIdx = lastChrIdx;
	rightBorder.coord = chrInfo[lastChrIdx]->length;
	return;
	}

if(dsp->runmode!=RM_jux_n && dsp->runmode!=RM_jux_c)
	{
	fprintf(stderr, "%s: wrong dsp runmode", __FUNCTION__);
	exit(0);
	}

/* juxtaposition
border are determined by first/last bed items from the list of chromosomes
to find that, all the items from a chr will be fetched to find either one of that
the data shall be cached in dsp->chritemsl in case
the chr will be used again in makeRegionSl_gf()

can we guarantee the sorting of the list generated by tabixQuery? no
the left most item is used for left border
the right most item is used for right border
must explicitly test this by coordinate comparison

always keep itemsl sorted with ascending start

TODO this should be improved by sorted bed list
 */
boolean notfound = TRUE;
struct beditem *itemsl, *slsorted, *it;
for(leftBorder.chromIdx=0; leftBorder.chromIdx<=lastChrIdx; leftBorder.chromIdx++)
	{
	itemsl=tabixQuery_regioncoord(dsp->tabix, dsp->juxtkft, chrInfo[leftBorder.chromIdx]->name, 0, chrInfo[leftBorder.chromIdx]->length);
	if(itemsl != NULL)
		{
		slsorted=(struct beditem *)beditemsort_startAsc((struct genericItem *)itemsl);
		/* use the first item in data as left border
		explicitly iterate to find the left most coord
		*/
		leftBorder.coord = slsorted->start;
		notfound = FALSE;
		// store for later use
		chritemslAppend(dsp, leftBorder.chromIdx, slsorted);
		if(SQUAWK)
			fprintf(stderr, "%s: juxtaposition left border - %s %d\n", __FUNCTION__, chrInfo[leftBorder.chromIdx]->name, leftBorder.coord);
		break;
		}
	}
if(notfound==TRUE)
	{
	fprintf(stderr, "%s: left border not found, the tabix file is empty?? %s\n", __FUNCTION__, dsp->tabix->fn);
	exit(0);
	}

// right border
//notfound = TRUE;
struct chritemsl *ci;
for(rightBorder.chromIdx=lastChrIdx; rightBorder.chromIdx>=0; rightBorder.chromIdx--)
	{
	/* dude! beware of such rare case
	when left/right borders are on same chr
	*/
	if((ci=chritemsl_searchidx(dsp, rightBorder.chromIdx))!=NULL)
		{
		slsorted=ci->itemsl;
		assert(slsorted!=NULL);
		}
	else
		{
		itemsl=tabixQuery_regioncoord(dsp->tabix, dsp->juxtkft, chrInfo[rightBorder.chromIdx]->name, 0, chrInfo[rightBorder.chromIdx]->length);
		slsorted=(struct beditem *)beditemsort_startAsc((struct genericItem *)itemsl);
		}
	if(slsorted != NULL)
		{
		// explicitly iterate to find right most stop coord
		unsigned int maxstop=0;
		for(it=slsorted; it!=NULL; it=it->next)
			{
			if(maxstop<it->stop)
				maxstop=it->stop;
			}
		rightBorder.coord = maxstop;
		if(SQUAWK)
			fprintf(stderr, "%s: juxtaposition right border - %s %d\n", __FUNCTION__, chrInfo[rightBorder.chromIdx]->name, rightBorder.coord);
		//notfound = FALSE;
		// store for later use
		if(ci==NULL)
			chritemslAppend(dsp, rightBorder.chromIdx, slsorted);
		return;
		}
	}
}

void setDspBoundary(struct displayedRegion *dsp, int startChr, int startCoord, int stopChr, int stopCoord)
{
dsp->start = malloc(sizeof(struct point));
dsp->stop = malloc(sizeof(struct point));
dsp->start->chromIdx = startChr;
dsp->start->coord = startCoord;
dsp->stop->chromIdx = stopChr;
dsp->stop->coord = stopCoord;
if(SQUAWK) fprintf(Squawk, "%s: %s %d %s %d\n", __FUNCTION__, chrInfo[startChr]->name, startCoord, chrInfo[stopChr]->name, stopCoord);
}



void computeEntireLength(struct displayedRegion *dsp)
{
/* region list defined by dsp->head and dsp->tail must be made prior to this
 */
dsp->entireLength = 0;
struct region *r;
for(r=dsp->head; r!=NULL; r=r->next)
    dsp->entireLength += r->dstop - r->dstart;
if(SQUAWK) fprintf(Squawk, "\tentire length computed: %ld...\n", dsp->entireLength);
}

void computeEntireLength_itemlist(struct displayedRegion *dsp)
{
// curbed by dsp->head and dsp->tail, tell tail by item name
dsp->entireLength = 0;
struct region *r;
for(r=dsp->head; ; r=r->next) 
    {
    dsp->entireLength += r->dstop - r->dstart;
    if(strcmp(r->name, dsp->tail->name) == 0) 
        break;
    }
if(SQUAWK) fprintf(Squawk, "entire length computed: %ld...\n", dsp->entireLength);
}
void computeSummarySize_bplevel(struct displayedRegion *dsp)
{
struct region *r;
dsp->usedSummaryNumber = dsp->entireLength;
for(r=dsp->head; r!=NULL; r=r->next)
    r->summarySize = r->dstop - r->dstart;
dsp->atbplevel = TRUE; // just in case
}

boolean trycomputeSummarySize_bplevel(struct displayedRegion *dsp, int spnum)
{
/* spnum: expected ceiling of summary points, usually hmSpan or 3*hmSpan?

   computeEntireLength must have been called before this
   if at bp level: 
      summary size will equal to seq length
      return true
   else:
      return false
 */
dsp->atbplevel = FALSE; // just in case
if(dsp->entireLength <= spnum)
    {
    dsp->atbplevel = TRUE;
    computeSummarySize_bplevel(dsp);
    }
return dsp->atbplevel;
}
boolean trycomputeSummarySize_bplevel_itemlist(struct displayedRegion *dsp, int spnum)
{
/* special case for trycomputeSummarySize_bplevel, need to use dsp->tail->name to terminate loop
 */
if(dsp->entireLength <= spnum)
    {
    dsp->atbplevel = TRUE;
    dsp->usedSummaryNumber = dsp->entireLength;
    struct region *r;
    for(r=dsp->head; ; r=r->next)
        {
        r->summarySize = r->dstop - r->dstart;
	if(strcmp(r->name, dsp->tail->name) == 0)
            return TRUE;
	}
    }
dsp->atbplevel = FALSE;
return FALSE;
}
void computeSummarySize(struct displayedRegion *dsp, int totalSP)
{
/* run after computeEntireLength()
   roll over regions starting from dsp->head, compute its summary size
   keep track of how many summary points have been used
   which is curbed by totalSP
   when that's going to be exhausted, truncate displayed region

   region's summary size is proportional to its length of entire length
 */
if(SQUAWK) fprintf(stderr, "\tcomputing summary size (might truncate dsp) expected %d...\n", totalSP);

if(trycomputeSummarySize_bplevel(dsp, totalSP))
    return;

dsp->usedSummaryNumber = 0;
struct region *r;
double sf=(double)totalSP/((double)dsp->entireLength); // sp per bp
for(r=dsp->head; r!=NULL; r=r->next)
	{
	int remainingSize = totalSP - dsp->usedSummaryNumber;
	int thisRegionSize = ceil((double)(r->dstop - r->dstart)*sf);
	if(thisRegionSize < remainingSize) // no equals...
		{
		r->summarySize = thisRegionSize;
		dsp->usedSummaryNumber += thisRegionSize;
		}
	else
		{
		if(SQUAWK)
			fprintf(stderr, "\ttotal # of summarySize exhausted (expecting %d, %d left)...\n", thisRegionSize, remainingSize);
		r->summarySize = remainingSize;
		int newbpwidth=ceil(remainingSize/sf);
		r->dstop=min(r->dstart+newbpwidth,r->dstop);
		dsp->usedSummaryNumber += remainingSize;
		/* must reset r->dstop !!!
		*/
		break;
		}
	}
// check if there's still regions in the list
if(r->next != NULL)
	{
	// truncate, reset stop point, recalculate entire length
	r->next = NULL;
	dsp->tail = r;
	if(SQUAWK)
		fprintf(stderr, "\tdisplayed region truncated to (%s:%d)...\n", chrInfo[r->chromIdx]->name, r->dstop);
	computeEntireLength(dsp);
	}
if(SQUAWK) fprintf(Squawk, "\tsummary size computed, %d used in total...\n", dsp->usedSummaryNumber);
}

void computeSummarySize_itemlist(struct displayedRegion *dsp)
{
/* ** special treatment for itemlist **

   must run computeEntireLength_itemlist() before this

   only do for items between head/tail

   hmSpan will be expected amount of total SP
   but be intelligent, if this is too little, increase to an reasonable amount
 */

if(trycomputeSummarySize_bplevel_itemlist(dsp, hmSpan))
    return;

// find smallest region
struct region *r;
int minRegionLen = dsp->head->dstop - dsp->head->dstart;
for(r=dsp->head->next; ; r=r->next)
    {
    if(r == NULL) break;
    if(strcmp(r->name, dsp->tail->name) == 0)
        break;
    int this = r->dstop - r->dstart;
    if(this < minRegionLen)
        minRegionLen = this;
    }

//fprintf(stderr, "min length: %d\n", minRegionLen);

// count number of items, there will be 1px spacing between two items
int itemcount=0;
for(r=dsp->head; r!=NULL; r=r->next) itemcount++;

// divide entireLength by such smallest region length, decide totalSP in this way
int totalSP;
float spfloat = (float)dsp->entireLength / minRegionLen;
if(spfloat < hmSpan)
    totalSP = hmSpan;
else
    totalSP = ceil(spfloat);
/*
else if(spfloat > hmSpan*3)
    totalSP = hmSpan * 3;
	*/

totalSP-=itemcount+1;

if(SQUAWK) fprintf(Squawk, "\tcomputing summary size (no truncation addup) expected %d...\n", totalSP);

dsp->usedSummaryNumber = 0;

for(r=dsp->head; ; r=r->next)
    {
    int thisRegionSize = max(1,floor(((float)(r->dstop-r->dstart)/dsp->entireLength)*totalSP));
    r->summarySize = thisRegionSize;
    dsp->usedSummaryNumber += thisRegionSize;
    if(strcmp(r->name, dsp->tail->name) == 0)
        break;
    }
if(SQUAWK) fprintf(Squawk, "\tsummary size computed, %d used in total...\n", dsp->usedSummaryNumber);
}



/*
struct displayedRegion *yearmonth2region(int startyear, int startcoord, int stopyear, int stopcoord)
{
assert(yearMonth!=NULL);
assert(startyear>=__startyear);
assert(stopyear<=__stopyear);
struct displayedRegion *dd=malloc(sizeof(struct displayedRegion));
dd->head=dd->tail=NULL;
int chridx, year, month, lookstart, lookstop, cumustart, cumustop, a,b;
struct region *r;
for(year=startyear; year<=stopyear; year++)
	{
	lookstart=year==startyear?startcoord:101;
	lookstop=year==stopyear?stopcoord:1231;
	for(chridx=0; chridx<=lastChrIdx; chridx++)
		{
		if(year==strMayPositiveInt(chrInfo[chridx]->name))
			break;
		}
	for(month=1;month<=12;month++)
		{
		cumustart=month*100+1;
		cumustop=month*100+yearMonth[year][month];
		a=max(lookstart,cumustart);
		b=min(lookstop,cumustop);
		if(a<b)
			{
			r=malloc(sizeof(struct region));
			r->bstart=cumustart;
			r->bstop=cumustop;
			r->dstart=a;
			r->dstop=b;
			r->isViewStart=FALSE;
			r->chromIdx=chridx;
			dspAddRegionTail(dd,r);
			}
		}
	}
return dd;
}
*/


void makeRegionSl_genome(struct displayedRegion *dsp)
{
/* setDspBoundary() must be run prior to this
   make list of regions between dsp->head and dsp->tail, 
   one region for each chromosome
 */
struct point *start = dsp->start, *stop = dsp->stop;
assert(start->chromIdx <= stop->chromIdx);
int i;
for(i=start->chromIdx; i<=stop->chromIdx; i++)
	{
	struct region *r = malloc(sizeof(struct region));
	r->bstart = 0;
	r->bstop = chrInfo[i]->length;
	r->dstart = (i==start->chromIdx) ? start->coord : r->bstart;
	r->dstop = (i==stop->chromIdx) ? stop->coord : r->bstop;
	r->isViewStart = FALSE;
	if(r->dstart == r->dstop)
		{ // omit zero-length region
		free(r);
		continue;
		}
	r->chromIdx = i;
	dspAddRegionTail(dsp, r);
	}
if(SQUAWK) fprintf(Squawk, "chromSl made...\n");
}


int requestRegionAddTail(struct displayedRegion *dsp, struct beditem *itemsl, int chrIdx, unsigned int lookstopcoord, int regionNumLimit)
{
/* only called by makeRegionSl_gf()
itemsl must have already been sorted by ascending start coord

lookstopcoord: 0 means no curb

request new regions from raw item data from chromosome query, as prepared by makeChritemsl()
region means a continuous segment connected by bed items
# of regions will be capped by given number
return # of regions added
new region will be added to tail of dsp region list

works on one chr for each call
 */

assert(itemsl!=NULL);
if(regionNumLimit<=0) return 0;

// first bed item
int rstart = itemsl->start;
int rstop = itemsl->stop;
if(lookstopcoord>0 && rstart>=lookstopcoord) return 0;
int count = 0;
struct region *r;
struct beditem *t;
for(t=itemsl->next; t!=NULL; t=t->next)
    {
    // merge bed items when they overlap or are head-tail connected
	if(lookstopcoord>0)
		{
		if(t->start >= lookstopcoord)
			break;
		}
    if(rstop >= t->start)
		{
        rstop = (t->stop > rstop) ? t->stop : rstop;
		}
    else
		{
		/* current item is outside of existing range
		yield new region with existing range */
        r = malloc(sizeof(struct region));
		r->bstart = r->dstart = rstart;
		r->bstop = r->dstop = rstop;
		r->chromIdx = chrIdx;
		r->isViewStart = FALSE;
		dspAddRegionTail(dsp, r);
		count++;
		if(count == regionNumLimit) return count;
		// range initiated with current item
		rstart = t->start;
		rstop = t->stop;
		}
    }
// itemsl exhausted, make one last region
r = malloc(sizeof(struct region));
r->bstart = r->dstart = rstart;
r->bstop = r->dstop = rstop;
r->chromIdx = chrIdx;
r->isViewStart = FALSE;
dspAddRegionTail(dsp, r);
count++;
return count;
}

void makeChritemsl(struct displayedRegion *dsp)
{
/* separated from makeRegionSl_gf() to reduce indent level

find bed items covered by given boundary
if there's anything within boundaries, fill them in dsp->chritemsl
won't change dsp->start, dsp->stop

keep itemsl sorted by ascending start
*/

// try to make chritemsl object for all chromosomes between dsp->start/stop
int i = dsp->start->chromIdx;
boolean foundsth=FALSE;
struct beditem *itemsl, *slsorted;
for(; i<=dsp->stop->chromIdx; i++)
    {
	if(chritemsl_searchidx(dsp, i)!=NULL)
		{
		foundsth=TRUE;
		continue;
		}
	itemsl=tabixQuery_regioncoord(dsp->tabix, dsp->juxtkft, chrInfo[i]->name, 0, chrInfo[i]->length);
	if(itemsl!=NULL)
		{
		slsorted=(struct beditem *)beditemsort_startAsc((struct genericItem *)itemsl);
		chritemslAppend(dsp, i, slsorted);
		foundsth=TRUE;
		}
	else
		{
		// got nothing from this query
		if(i==dsp->start->chromIdx && i==dsp->stop->chromIdx)
			{
			/* start/stop points are both within this chr
			not good, need to search remaining of this chr
			*/
			itemsl=tabixQuery_regioncoord(dsp->tabix, dsp->juxtkft, chrInfo[i]->name, dsp->stop->coord, chrInfo[i]->length);
			if(itemsl!=NULL)
				{
				slsorted=(struct beditem *)beditemsort_startAsc((struct genericItem *)itemsl);
				chritemslAppend(dsp, i, slsorted);
				foundsth=TRUE;
				}
			}
		}
	}

if(foundsth) return;

/*  haven't found anything with above queries
need to seek to chrs behind stop point and issue additional queries
but won't go to chrs before start point, as makeBorder() has already filled things there...
quit upon finding anything...
*/
for(i=dsp->stop->chromIdx+1; i<=lastChrIdx; i++)
	{
	if(chritemsl_searchidx(dsp, i)!=NULL) return;
	itemsl=tabixQuery_regioncoord(dsp->tabix, dsp->juxtkft, chrInfo[i]->name, 0, chrInfo[i]->length);
	if(itemsl!=NULL)
		{
		slsorted=(struct beditem *)beditemsort_startAsc((struct genericItem *)itemsl);
		chritemslAppend(dsp, i, slsorted);
		return;
		}
	}
/* it probably doesn't matter if execution would go here
as something must already been made in makeBorder()
*/
}

void makeRegionSl_gf(struct displayedRegion *dsp)
{
/* setDspBoundary() must be called prior to this
   first make dsp->chritemsl
   then determine dsp region by dsp->chritemsl
   curb region # by hmSpan
 */
if(SQUAWK)
    fputs("makeRegionSl_gf:\n", Squawk);

makeChritemsl(dsp);

struct chritemsl *ci;

if(SQUAWK)
    {
	// turn it off after debugging
	fputs("\tshow dsp->chritemsl ...\n", stderr);
	for(ci=dsp->chritemsl; ci!=NULL; ci=ci->next)
		fprintf(Squawk, "\t\t%s %d bed items\n", chrInfo[ci->chromIdx]->name, slCount2(ci->itemsl));
	fputs("\tdsp->chritemsl over\n", stderr);
    }

/*  will generate dsp regions based on chr raw data
try to exhaust bed data between dsp start/stop chromosomes
*/
int chrIdx;
int totalRegionAdded=0;
struct beditem *item;
for(chrIdx=dsp->start->chromIdx; chrIdx<=dsp->stop->chromIdx; chrIdx++)
	{
	if((ci=chritemsl_searchidx(dsp, chrIdx))!=NULL)
		{
		/* if the chr is dsp start/stop chr,
		need to curb the looking item by start/stop coord!!
		*/
		item=ci->itemsl;
		if(chrIdx==dsp->start->chromIdx)
			{
			for(; item!=NULL; item=item->next)
				{
				if(item->stop>dsp->start->coord)
					break;
				}
			}
		if(item==NULL) continue;
		unsigned int lookstopcoord= chrIdx==dsp->stop->chromIdx ? dsp->stop->coord : 0;

		totalRegionAdded += requestRegionAddTail(dsp, item, chrIdx, lookstopcoord, hmSpan-totalRegionAdded);
		if(totalRegionAdded >= hmSpan) break;
		}
	}
if(totalRegionAdded>0)
	{
	if(SQUAWK)
		fprintf(stderr, "\t%d regions generated between dsp start/stop\n", totalRegionAdded);
	}
else
	{
	/* no regions generated between dsp start/stop points
	look to the chrs on the right of dsp->stop
	*/
	for(chrIdx=dsp->stop->chromIdx+1; chrIdx<=lastChrIdx; chrIdx++)
		{
		if((ci=chritemsl_searchidx(dsp, chrIdx))!=NULL)
			{
			totalRegionAdded += requestRegionAddTail(dsp, ci->itemsl, chrIdx, 0, hmSpan-totalRegionAdded);
			if(totalRegionAdded >= hmSpan) break;
			}
		}
	if(totalRegionAdded==0)
		{
		/* still no regions added, start from the very first chr */
		for(chrIdx=0; chrIdx<=lastChrIdx; chrIdx++)
			{
			if((ci=chritemsl_searchidx(dsp, chrIdx))!=NULL)
				{
				totalRegionAdded += requestRegionAddTail(dsp, ci->itemsl, chrIdx, 0, hmSpan-totalRegionAdded);
				if(totalRegionAdded >= hmSpan) break;
				}
			}
		}
	}

 
/****** FIXME ******
   requestRegionAddTail() will set dstart/dstop same as bstart/bstop
   but for cases like drawing decor in juxtaposition mode
   dsp head/tail might have different dstart/dstop according to CGI param,

   if dsp->start is within dsp->head, reset head->dstart using start->coord,
   if dsp->stop is within dsp->tail, reset tail->dstop using stop->coord
   
   is this reliable??
 *******************/
if((dsp->start->chromIdx == dsp->head->chromIdx) &&
   (dsp->start->coord >= dsp->head->bstart) &&
   (dsp->start->coord <= dsp->head->bstop))
    {
    if(SQUAWK) fputs("\treset dsp->head->dstart using dsp->start->coord\n", Squawk);
    dsp->head->dstart = dsp->start->coord;
    }
if((dsp->stop->chromIdx == dsp->tail->chromIdx) &&
   (dsp->stop->coord >= dsp->tail->bstart) &&
   (dsp->stop->coord <= dsp->tail->bstop))
    {
    if(SQUAWK) fputs("\treset dsp->tail->dstop using dsp->stop->coord\n", Squawk);
    dsp->tail->dstop = dsp->stop->coord;
    }
if(SQUAWK) 
    {
    int i=0;
    struct region *r;
    for(r=dsp->head; r!=NULL; r=r->next) i++;
    fprintf(Squawk, "\tregionLst made, %d in total...\n", i);
    }
}



void extendRegionDstart(struct region *r, double spsize, int *spnum, boolean atbplevel)
{
/* extend region's dstart to left by trying fit in a number of summary points
   region's attributes must be initialized:
   - summarySize
   - bstart
   - dstart
 */
if(r->dstart <= r->bstart) return;
if(*spnum == 0) return;

int dist = (int)(spsize * *spnum); // dist to move in bp
if(atbplevel)
	{
	int this = r->dstart - r->bstart; // available bp
	if(this >= dist)
		{
		r->summarySize += *spnum;
		r->dstart -= dist;
		*spnum = 0;
		}
	else
		{
		r->dstart = r->bstart;
		r->summarySize += (int)(this/spsize);
		*spnum -= (int)(this/spsize);
		}
	return;
	}
double this = (r->dstart - r->bstart) / spsize;
if(this >= *spnum)
	{
	r->summarySize += *spnum;
	r->dstart -= dist;
	*spnum = 0;
	}
else
	{
	r->dstart = r->bstart;
	r->summarySize += ceil(this);
	*spnum -= ceil(this);
	}
}

void extendRegionDstop(struct region *r, double spsize, int *spnum, boolean atbplevel)
{
/* extend region's dstop to right by trying to region's attributes must be initialized:
   - summarySize
   - bstop
   - dstop
 */
if(r->dstop >= r->bstop) return;
if(*spnum == 0) return;

int dist = (int)(spsize * *spnum); // dist to move in bp
if(atbplevel)
	{
	int this = r->bstop - r->dstop;
	if(this >= dist)
		{
		r->summarySize += *spnum;
		r->dstop += dist;
		*spnum = 0;
		}
	else
		{
		r->dstop = r->bstop;
		r->summarySize += (int)(this/spsize);
		*spnum -= (int)(this/spsize);
		}
	return;
	}
double this = (r->bstop - r->dstop) / spsize;
if(this >= *spnum)
	{
	r->summarySize += *spnum;
	r->dstop += dist;
	*spnum = 0;
	}
else
	{
	r->dstop = r->bstop;
	r->summarySize += ceil(this);
	*spnum -= ceil(this);
	}
}



long moveBoundary_genome(struct displayedRegion *dsp, char side, long distance, int *spnum, double spsize)
{
if(SQUAWK)
    fprintf(stderr, "moveBoundary_genome distance: %ld spnum: %d spsize: %f\n", distance, *spnum, spsize);
/* move a point along genome, out side of existing list, 
   always need to extend/add new regions to list

   args:
       side: 'l' for moving dsp->head to left
             'r' for moving dsp->tail to right
       distance: distance in bp to be moved
       spnum: expected number of summary points to allocate
       spsize: # bp each summary point spans

   return 0 when all distance moved successfully
   else if hit border, return remaining distance not moved
 */
int oldpos;
while(distance > 0)
	{
	if(side == 'l')
		{
		struct region *head = dsp->head;
		oldpos = head->dstart;
		extendRegionDstart(head, spsize, spnum, dsp->atbplevel);
		distance -= oldpos - head->dstart;
		if(distance <= 0) // is this possible??
			return 0;
		if(*spnum == 0)
			return distance;
		if(head->chromIdx == leftBorder.chromIdx) // hit left border
			return distance;
		// not hitting left border, step into one chromosome on left...
		struct region *r = malloc(sizeof(struct region));
		r->chromIdx = head->chromIdx-1;
		r->bstart = 0;
		r->bstop = chrInfo[r->chromIdx]->length;
		r->dstart = r->dstop = r->bstop;
		r->summarySize = 0;
		r->isViewStart = FALSE;
		dspAddRegionHead(dsp, r);
		}
	else
		{
		struct region *tail = dsp->tail;
		oldpos = tail->dstop;
		extendRegionDstop(tail, spsize, spnum, dsp->atbplevel);
		distance -= tail->dstop - oldpos;
		if(distance <= 0)
			return 0;
		if(*spnum == 0)
			return distance;
		if(tail->chromIdx == rightBorder.chromIdx) // hit right border
			return distance;
		// not hitting right border, step into one chromosome on right...
		struct region *r = malloc(sizeof(struct region));
		r->chromIdx = tail->chromIdx +1;
		r->bstart=r->dstart=r->dstop= 0;
		r->bstop = chrInfo[r->chromIdx]->length;
		r->summarySize = 0;
		r->isViewStart = FALSE;
		dspAddRegionTail(dsp, r);
		}
	}
return 0;
}




struct region *newRegionFromBeditem(int chrIdx, struct beditem *item)
{
// dstart/dstop will be unspecified!!
struct region *r = malloc(sizeof(struct region));
r->chromIdx = chrIdx;
r->bstop = item->stop;
r->bstart = item->start;
r->summarySize = 0;
r->isViewStart = FALSE;
return r;
}





void moveBoundary_gf_left(struct displayedRegion *dsp, long distance, int spnum)
{
/* juxtaposition
dsp region list must already have summary size computed
allowed moving distance and summary point number given

cached chr itemsl will be used, but will be sorted by stop coord!
 */
struct region *hRegion = dsp->head;
if(SQUAWK) fprintf(Squawk, "\tmoveBoundary_gf_left: from %s:%d to left by (%ld, %d)...\n", chrInfo[hRegion->chromIdx]->name, hRegion->dstart, distance, spnum);

double spsize = (double)distance / spnum;

// remaining distance in head region
extendRegionDstart(hRegion, spsize, &spnum, dsp->atbplevel);
if(spnum <= 0) return;

// fetch new bed items on left in *same chr* as head region
fprintf(stderr, "looking for %d...\n", hRegion->chromIdx);
struct chritemsl *ci=chritemsl_searchidx(dsp, hRegion->chromIdx);
assert(ci!=NULL); // believe it can never be null
// sort by descending stop coord
struct beditem *itemsl=(struct beditem *)beditemsort_stopDsc((struct genericItem *)ci->itemsl);
// replace existing record
ci->itemsl=itemsl;
struct beditem *item;

/* try extending current head with overlaping bed items */
for(item=itemsl; item!=NULL; item=item->next)
	{
	if(item->stop >= hRegion->bstart)
		{
		if(item->start < hRegion->bstart)
			{
			// head bstart extendable with new item
			hRegion->bstart = item->start;
			}
		}
	else
		{
		// new item beyond head, stop
		break;
		}
	}
extendRegionDstart(hRegion, spsize, &spnum, dsp->atbplevel);
if(spnum <= 0) return;

if(item != NULL)
	{
	/* form new regions using remaining bed items in this chromosome 
	will append to head
	*/
	struct region *r = newRegionFromBeditem(hRegion->chromIdx, item);
	r->dstart = r->dstop = r->bstop; // set displayed portion as zero length
	// extend this region
	for(item=item->next; item!=NULL; item=item->next)
		{
		if(item->stop >= r->bstart)
			{
			if(item->start < r->bstart) // extend current region
				r->bstart = item->start;
			}
		else
			{
			// current region stops
			extendRegionDstart(r, spsize, &spnum, dsp->atbplevel);
			dspAddRegionHead(dsp, r);
			if(spnum <= 0) return;

			// if the program continues, create another region, initiate using current bed item
			r = newRegionFromBeditem(hRegion->chromIdx, item);
			r->dstart = r->dstop = r->bstop;
			}
		}
	extendRegionDstart(r, spsize, &spnum, dsp->atbplevel);
	dspAddRegionHead(dsp, r);
	if(spnum <= 0) return;
	}

/* looping through remaining chromosomes on left of h->chromIdx
   form new regions,
   add to head
 */
int chrIdx = hRegion->chromIdx -1;
//fputs("entering while loop...\n", stderr);
while(chrIdx >= 0)
    {
	itemsl=NULL;
	if((ci=chritemsl_searchidx(dsp, chrIdx))!=NULL)
		{
		itemsl=(struct beditem *)beditemsort_stopDsc((struct genericItem *)ci->itemsl);
		ci->itemsl=itemsl;
		}
	else
		{
		struct beditem *sl2=tabixQuery_regioncoord(dsp->tabix, dsp->juxtkft, chrInfo[chrIdx]->name, 0, chrInfo[chrIdx]->length);
		if(sl2!=NULL)
			{
			itemsl=(struct beditem *)beditemsort_stopDsc((struct genericItem *)sl2);
			chritemslAppend(dsp, chrIdx, itemsl);
			}
		}
    if(itemsl == NULL)
        {
        chrIdx--;
        continue;
		}

    // received data for this chr, repeat above practice
	item=itemsl;
    struct region *r = newRegionFromBeditem(chrIdx, item);
    r->dstart = r->dstop = r->bstop;
    for(item=item->next; item!=NULL; item=item->next)
        {
		if(item->stop >= r->bstart)
			{
			if(item->start < r->bstart)
				r->bstart = item->start;
			}
		else
			{
			extendRegionDstart(r, spsize, &spnum, dsp->atbplevel);
			dspAddRegionHead(dsp, r);
			if(spnum <= 0) return;
			r = newRegionFromBeditem(chrIdx, item);
			r->dstart = r->dstop = r->bstop;
			}
		}
    extendRegionDstart(r, spsize, &spnum, dsp->atbplevel);
    dspAddRegionHead(dsp, r);
    if(spnum <= 0) return;
    chrIdx--;
    }
}



void moveBoundary_gf_right(struct displayedRegion *dsp, long distance, int spnum) 
{
/* growing on right
must re-sort all itemsl as it might have been modified in moveBoundary_gf_left()
*/
struct region *tRegion = dsp->tail;
if(SQUAWK) fprintf(Squawk, "\tmoveBoundary_gf_right: from %s:%d to right by (%ld, %d)...\n", chrInfo[tRegion->chromIdx]->name, tRegion->dstop, distance, spnum);

double spsize = (double)distance / spnum;

extendRegionDstop(tRegion, spsize, &spnum, dsp->atbplevel);
if(spnum <= 0) return;

// look for new bed items on right in tail chromosome
struct chritemsl *ci=chritemsl_searchidx(dsp, tRegion->chromIdx);
assert(ci!=NULL);
struct beditem *itemsl=(struct beditem *)beditemsort_startAsc((struct genericItem *)ci->itemsl);
struct beditem *item;
ci->itemsl=itemsl;
/* try extending tail with overlaping bed items */
for(item=itemsl; item!=NULL; item=item->next)
	{
	if(item->start <= tRegion->bstop)
		{
		if(item->stop > tRegion->bstop)
			{
			// tail extendable by this item
			tRegion->bstop = item->stop;
			}
		}
	else
		{
		// item beyond tail, stop
		break;
		}
	}
extendRegionDstop(tRegion, spsize, &spnum, dsp->atbplevel);
if(spnum <= 0) return;

if(item != NULL)
	{
	/* form new regions using remaining bed items in this chromosome */
	struct region *r = newRegionFromBeditem(tRegion->chromIdx, item);
	r->dstart = r->dstop = r->bstart;
	for(item=item->next; item!=NULL; item=item->next)
		{
		if(item->start <= r->bstop)
			{
			if(item->stop > r->bstop)
				r->bstop = item->stop;
			}
		else
			{
			// current region stops
			extendRegionDstop(r, spsize, &spnum, dsp->atbplevel);
			dspAddRegionTail(dsp, r);
			if(spnum <= 0) return;
			
			// if program continues, make new region
			r = newRegionFromBeditem(tRegion->chromIdx, item);
				r->dstart = r->dstop = r->bstart;
			}
		}
		extendRegionDstop(r, spsize, &spnum, dsp->atbplevel);
		dspAddRegionTail(dsp, r);
		if(spnum <= 0) return;
	}

/* looping through remaining chromosomes on right of tRegion->chromIdx
   form new regions,
   add to tail
 */
int chrIdx = tRegion->chromIdx +1;
while(chrIdx <= lastChrIdx)
    {
	itemsl=NULL;
	if((ci=chritemsl_searchidx(dsp, chrIdx))!=NULL)
		{
		itemsl=(struct beditem *)beditemsort_startAsc((struct genericItem *)ci->itemsl);
		ci->itemsl=itemsl;
		}
	else
		{
		struct beditem *sl2=tabixQuery_regioncoord(dsp->tabix, dsp->juxtkft, chrInfo[chrIdx]->name, 0, chrInfo[chrIdx]->length);
		if(sl2!=NULL)
			{
			itemsl=(struct beditem *)beditemsort_startAsc((struct genericItem *)sl2);
			chritemslAppend(dsp, chrIdx, sl2);
			}
		}
    if(itemsl == NULL)
        {
		chrIdx++;
		continue;
		}
	item=itemsl;
    struct region *r = newRegionFromBeditem(chrIdx, item);
    r->dstart = r->dstop = r->bstart;
    for(item=item->next; item!=NULL; item=item->next)
        {
		if(item->start <= r->bstop)
			{
			if(item->stop > r->bstop)
				r->bstop = item->stop;
			}
		else
			{
			extendRegionDstop(r, spsize, &spnum, dsp->atbplevel);
			dspAddRegionTail(dsp, r);
			if(spnum <= 0) return;
			r = newRegionFromBeditem(chrIdx, item);
				r->dstart = r->dstop = r->bstart;
			}
        }
    extendRegionDstop(r, spsize, &spnum, dsp->atbplevel);
    dspAddRegionTail(dsp, r);
    if(spnum <= 0) return;
    chrIdx++;
    }
}


void computeUsedSummaryNumber(struct displayedRegion *dsp)
{
struct region *r;
dsp->usedSummaryNumber = 0;
for(r=dsp->head; r!=NULL; r=r->next)
    dsp->usedSummaryNumber += r->summarySize;
}

void finalizeDsp_gf(struct displayedRegion *dsp)
{
/* call this after running setDspBoundary()
   trigger_move will not come to this step, only others will

          |--left wing--|========DSP========|--right wing--|

   DSP will have hmSpan number of summary points,
   # of summary points for left/right wing will be given by their
   length proportional to dsp.entireLength

   make expansion to wing regions:
   - make regions (head/tail) if it's not there yet
   - compute entire length for DSP
   - seek left boundary to left by entireLength
   - compute left wing # of summary points (leftSPnum)
   - set dspStart
   - seek right boundary to right by entireLength
   - compute right wing # of summary points (rightSPnum)
   - recompute entire length for expanded DSP
   - recompute summary size for expanded DSP using (leftSPnum+hmSpan+rightSPnum)
 */
if(SQUAWK) fputs("finalizeDsp_gf()...\n", Squawk);
if(dsp->head == NULL)
    {
    if(dsp->tail == NULL)
        makeRegionSl_gf(dsp);
    else
        errabort("dsp->head is null but dsp->tail is NOT NULL");
    }
// set all rest regions as NOT being VIEW START
struct region *r;
for(r=dsp->head->next; r!=NULL; r=r->next)
    r->isViewStart = FALSE;
// and all the *wing* regions will be set as similar

// dsp->head is view start
dsp->head->isViewStart = TRUE;
dsp->head->viewStartCoord = dsp->head->dstart;

/* in order to get resolution size, first compute summary size of existing region list
   then use this resolution size to compute spnum for left/right wings
 */
computeEntireLength(dsp);
computeSummarySize(dsp, hmSpan);


// to remember if it should be at bp level, do special things after making wings...

moveBoundary_gf_left(dsp, dsp->entireLength, dsp->usedSummaryNumber);
moveBoundary_gf_right(dsp, dsp->entireLength, dsp->usedSummaryNumber);

computeEntireLength(dsp);

if(dsp->atbplevel)
    {
    // reassign summary size for each region using region seq length
    computeSummarySize_bplevel(dsp);
    }
else
    {
    computeUsedSummaryNumber(dsp);
    }
}




void finalizeDsp_genome(struct displayedRegion *dsp)
{
if(SQUAWK) fputs("finalizeDsp_genome()...\n", Squawk);
if(dsp->head == NULL)
    {
    if(dsp->tail == NULL)
        makeRegionSl_genome(dsp);
    else
        errabort("dsp->head is null but dsp->tail is NOT NULL");
    }
// set all rest regions as NOT being VIEW START
struct region *r;
for(r=dsp->head; r!=NULL; r=r->next)
    r->isViewStart = FALSE;
// and all the *wing* regions will be set as similar

// dsp->head is view start
dsp->head->isViewStart = TRUE;
dsp->head->viewStartCoord = dsp->head->dstart;

computeEntireLength(dsp);

if(trycomputeSummarySize_bplevel(dsp, hmSpan))
    {
    computeUsedSummaryNumber(dsp);
    if(SQUAWK) fputs("\tnow at bp level\n", stderr);
    int a = dsp->usedSummaryNumber;
    moveBoundary_genome(dsp, 'l', dsp->entireLength, &a, 1);
    a = dsp->usedSummaryNumber;
    moveBoundary_genome(dsp, 'r', dsp->entireLength, &a, 1);
    computeUsedSummaryNumber(dsp);
    return;
    }

// assign hmSpan points to current dsp
computeSummarySize(dsp, hmSpan);


double sf = ((double)dsp->entireLength / dsp->usedSummaryNumber);

int spn = hmSpan;
moveBoundary_genome(dsp, 'l', dsp->entireLength, &spn, sf);
spn = hmSpan;
moveBoundary_genome(dsp, 'r', dsp->entireLength, &spn, sf);
computeEntireLength(dsp);
computeUsedSummaryNumber(dsp);
}




/**********************************
         move/zoom
 **********************************/

boolean dspAtLeftBorder(struct displayedRegion *dsp)
{
/* if at left border, return TRUE
   if downstream of left border, return FALSE
   else, squawk and die
 */
if(dsp->head->chromIdx > leftBorder.chromIdx)
    return FALSE;
if(dsp->head->chromIdx == leftBorder.chromIdx)
    {
    if(dsp->head->dstart == leftBorder.coord)
        return TRUE;
    if(dsp->head->dstart > leftBorder.coord)
        return FALSE;
    errabort("dsp left boundary beyond left border");
    }
errabort("dsp left boundary beyond left border");
return FALSE; // merely not to let compiler complain
}


boolean dspAtRightBorder(struct displayedRegion *dsp)
{
if(dsp->tail->chromIdx < rightBorder.chromIdx)
    return FALSE;
if(dsp->tail->chromIdx == rightBorder.chromIdx)
    {
    if(dsp->tail->dstop == rightBorder.coord)
        return TRUE;
    if(dsp->tail->dstop < rightBorder.coord)
        return FALSE;
    errabort("dsp right boundary beyond right border");
    }
errabort("dsp right boundary beyond right border");
return FALSE; // merely...
}


long moveBoundary_list_which(struct displayedRegion *dsp, char which, char side, long distance)
{
/* same code for genome/gf 

   args:
      which: h/t telling if it's head or tail that moves
      side: l/r to which side

   ** sort out the logic carefully ***
   move boundary within list of regions defined by dsp->head and dsp->tail
   and look at dstart/dstop of each region
   direction: 'l' for moving point to left, 'r' to right

   return current point
 */
if(which!='h' && which!='t') errabort("moveBoundary_list_which: which must be h/t");
if(side!='l' && side!='r') errabort("moveBoundary_list_which: side must be l/r");
if(SQUAWK) fprintf(stderr, "\tmoveBoundary_list_which: which (%c) side (%c) dist (%ld)\n", which, side, distance);
struct region *glider;
if(which == 'h')
    {
    // moving head, only modify dstart
    glider = dsp->head;
    if(side == 'r')
        { // shrink
        while(distance > 0)
            {
	    int movableDist = glider->bstop - glider->dstart;
	    if(distance < movableDist)
	        {
	        glider->dstart += distance;
	        distance = 0;
	        break;
	        }
	    distance -= movableDist;
	    glider->dstart = glider->bstop;
	    if(glider->next == NULL) break;
	    glider = glider->next;
	    glider->dstart = glider->bstart;
            }
	}
    else
        { // expand
	while(distance > 0)
	    {
	    int movableDist = glider->dstart - glider->bstart;
	    if(distance < movableDist)
	        {
	        glider->dstart -= distance;
	        distance = 0;
	        break;
	        }
	    distance -= movableDist;
	    glider->dstart = glider->bstart;
	    if(glider->prev == NULL) break;
	    glider = glider->prev;
	    glider->dstart = glider->dstop = glider->bstop; // enforced
            }
        }
    dsp->head = glider;
    }
else
    {
    // moving tail, only modify dstop
    glider = dsp->tail;
    if(side == 'r')
        { // expand
        while(distance > 0)
            {
	    int movableDist = glider->bstop - glider->dstop;
	    if(distance < movableDist)
	        {
	        glider->dstop += distance;
	        distance = 0;
	        break;
	        }
	    distance -= movableDist;
	    glider->dstop = glider->bstop;
	    if(glider->next == NULL) break;
	    glider = glider->next;
	    glider->dstop = glider->dstart = glider->bstart; // enforced
            }
	}
    else
        { // shrink
        while(distance > 0)
	    {
	    int movableDist = glider->dstop - glider->bstart;
	    if(distance < movableDist)
	        {
	        glider->dstop -= distance;
	        distance = 0;
	        break;
	        }
	    distance -= movableDist;
	    glider->dstop = glider->bstart;
	    if(glider->prev == NULL) break;
	    glider = glider->prev;
	    glider->dstop = glider->bstop; // enforced
            }
        }
    dsp->tail = glider;
    }
return distance;
}

void moveBoundary_list(struct displayedRegion *dsp, char side, long distance)
{
// wrapper of moveBoundary_list_which
if(side!='l' && side!='r') errabort("moveBoundary_list: side must be l/r");
if(side == 'l') // move head to right
    moveBoundary_list_which(dsp, 'h', 'r', distance);
else // move tail to left
    moveBoundary_list_which(dsp, 't', 'l', distance);
}





/***********************
   ITEMlist-specific functions
 **********************/
void genesetRuntime_insert(char *session, int statusId, char *itemlist)
{
char *query;
if(asprintf(&query, "delete from genesetRuntime where session='%s' and statusId=%d", session, statusId)<0) errabort("bae");
mysqlUpdate(query);
free(query);
if(asprintf(&query, "insert into genesetRuntime values('%s', %d, '%s')", session, statusId, itemlist)<0) errabort("bae");
mysqlUpdate(query);
free(query);
}
int itemlistsort_lenAsc(const void *a, const void *b)
{
// item length, ascending
const struct region *aa = (const struct region *)a;
const struct region *bb = (const struct region *)b;
int alen = aa->bstop - aa->bstart;
int blen = bb->bstop - bb->bstart;
return (alen > blen) - (alen < blen);
}
int itemlistsort_lenDesc(const void *a, const void *b)
{
// item length, descending
const struct region *aa = (const struct region *)a;
const struct region *bb = (const struct region *)b;
int alen = aa->bstop - aa->bstart;
int blen = bb->bstop - bb->bstart;
return (alen < blen) - (alen > blen);
}
int itemlistsort_tkmeanAsc(const void *a, const void *b)
{
// average track score, ascending
const struct region *aa = (const struct region *)a;
const struct region *bb = (const struct region *)b;
if(isnan(aa->fvalue))
	{
	if(isnan(bb->fvalue))
		return 0;
	return -1;
	}
if(isnan(bb->fvalue)) return 1;
return (aa->fvalue > bb->fvalue) - (aa->fvalue < bb->fvalue);
}
int itemlistsort_tkmeanDesc(const void *a, const void *b)
{
// average track score, descending
const struct region *aa = (const struct region *)a;
const struct region *bb = (const struct region *)b;
if(isnan(aa->fvalue))
	{
	if(isnan(bb->fvalue))
		return 0;
	return 1;
	}
if(isnan(bb->fvalue)) return -1;
return (aa->fvalue < bb->fvalue) - (aa->fvalue > bb->fvalue);
}
int itemlistsort_coord(const void *a, const void *b)
{
// item length, coordinate
const struct region *aa = (const struct region *)a;
const struct region *bb = (const struct region *)b;
if(aa->chromIdx != bb->chromIdx)
    return (aa->chromIdx > bb->chromIdx) - (aa->chromIdx < bb->chromIdx);
if(aa->bstart != bb->bstart)
    return (aa->bstart > bb->bstart) - (aa->bstart < bb->bstart);
return (aa->bstop > bb->bstop) - (aa->bstop < bb->bstop);
}
int getChromIdxByItemname(struct displayedRegion *dsp, char *itemname)
{
// regionSL must have been made
struct region *r;
for(r=dsp->head; r!=NULL; r=r->next)
    {
    if(strcmp(r->name, itemname) == 0)
        return r->chromIdx;
    }
errabort("getChromIdxByItemname: item name not found in regionSL");
return -1;
}
void trimRegionSl_itemlist(struct displayedRegion *dsp, char *startItem, int startCoord, char *stopItem, int stopCoord)
{
/* regionSl between dsp->head/tail must be made
   and will seek for boundary inside this list
 */
struct region *r;
boolean headmissing=TRUE, tailmissing=TRUE;
for(r=dsp->head; r!=NULL; r=r->next)
    {
    if(strcmp(r->name, startItem) == 0)
        {
	r->dstart = startCoord;
	dsp->head = r;
	headmissing = FALSE;
	}
    if(strcmp(r->name, stopItem) == 0)
        {
	r->dstop = stopCoord;
	dsp->tail = r;
	tailmissing = FALSE;
	break;
	}
    }
if(headmissing)
	{
	fprintf(stderr, "trimRegionSl_itemlist: head (%s) not found\n", startItem);
	exit(0);
	}
if(tailmissing)
	{
	fprintf(stderr, "trimRegionSl_itemlist: tail (%s) not found\n", stopItem);
	exit(0);
	}
if(SQUAWK) fprintf(Squawk, "trimRegionSl_itemlist: head (%s) tail (%s)...\n", dsp->head->name, dsp->tail->name);
}

boolean dspRestoreByCleanItemlist(struct displayedRegion *dsp, char *session, int statusId)
{
/* itemlist is what's stored in collateitemlist.itemlist, must be clean and safe,
use it to restore dsp, will make regionSl for all items, and might be modified later on
upon error, return FALSE

name,chridx,start,stop,5fstart,5fstop,3fstart,3fstop,strand,isgene

this is complete item list
*/
char *savedString = getItemlistfromdb(session, statusId);
if(savedString == NULL)
    return FALSE;
char delim[] = ",";
struct region *r;
char *tok = strtok(savedString, delim);
assert(tok!=NULL);
while(tok != NULL)
    {
    r = malloc(sizeof(struct region));
	r->name=strdup(tok);
    assert((tok=strtok(NULL, delim))!=NULL);
    assert((r->chromIdx=strMayPositiveInt(tok))!=-1);
    assert((tok=strtok(NULL, delim))!=NULL);
    assert((r->bstart=r->dstart=strMayPositiveInt(tok))!=-1);
    assert((tok=strtok(NULL, delim))!=NULL);
    assert((r->bstop=r->dstop=strMayPositiveInt(tok))!=-1);
	// following values could be -1 for no flanking
    assert((tok=strtok(NULL, delim))!=NULL);
    r->flank5start=strMayPositiveInt(tok);
    assert((tok=strtok(NULL, delim))!=NULL);
    r->flank5stop=strMayPositiveInt(tok);
    assert((tok=strtok(NULL, delim))!=NULL);
    r->flank3start=strMayPositiveInt(tok);
    assert((tok=strtok(NULL, delim))!=NULL);
    r->flank3stop=strMayPositiveInt(tok);
    r->isViewStart = FALSE;
    assert((tok=strtok(NULL, delim))!=NULL);
	r->strand=tok[0];
    assert((tok=strtok(NULL, delim))!=NULL);
	r->flag=tok[0]=='1';
    dspAddRegionTail(dsp, r);
    tok = strtok(NULL, delim);
    }
return TRUE;
}

void makeDspWing_itemlist(struct displayedRegion *dsp)
{
/* do this after dsp->head/tail are made
   first flip head->isViewStart

   make wings by summary size
 */
if(SQUAWK) fputs("makeDspWing_itemlist()...\n", stderr);
dsp->head->isViewStart = TRUE;
dsp->head->viewStartCoord = dsp->head->dstart;
computeEntireLength_itemlist(dsp);

if(trycomputeSummarySize_bplevel_itemlist(dsp, hmSpan))
    {
    moveBoundary_list_which(dsp, 'h', 'l', dsp->entireLength);
    moveBoundary_list_which(dsp, 't', 'r', dsp->entireLength);
    dsp->head->prev = NULL;
    dsp->tail->next = NULL;
    computeEntireLength(dsp);
    computeSummarySize_bplevel(dsp);
    return;
    }
    
computeSummarySize_itemlist(dsp);

float spsize = (float)dsp->entireLength / dsp->usedSummaryNumber;
int spnum = hmSpan;
struct region *r;

// seek to left to make left wing
r = dsp->head;
extendRegionDstart(r, spsize, &spnum, dsp->atbplevel);
if(spnum > 0)
    {
    r = r->prev;
    while(r != NULL)
        {
	r->summarySize = 0;
	r->dstart = r->dstop = r->bstop;
	extendRegionDstart(r, spsize, &spnum, dsp->atbplevel);
	dsp->head = r;
	if(spnum <= 0)
	    break;
	r = r->prev;
	}
    }
dsp->head->prev = NULL;

// seek to right to make right wing
r = dsp->tail;
spnum = hmSpan;
extendRegionDstop(r, spsize, &spnum, dsp->atbplevel);
if(spnum > 0)
    {
    r = r->next;
    while(r != NULL)
        {
	r->summarySize = 0;
	r->dstart = r->dstop = r->bstart;
	extendRegionDstop(r, spsize, &spnum, dsp->atbplevel);
	dsp->tail = r;
	if(spnum <= 0)
	    break;
	r = r->next;
	}
    }
dsp->tail->next = NULL;

computeEntireLength(dsp);

dsp->usedSummaryNumber = 0;
for(r=dsp->head; r!=NULL; r=r->next)
    dsp->usedSummaryNumber += r->summarySize;
}





/********************************
      draw decor tracks
      apply to both genome/gf mode
 *******************************/

void parseTrackParam(char *param, int ft, struct track **sl)
{
char delim[] = ",";
struct track *tt;
char *tok = strtok(param, delim);
while(tok != NULL)
    {
	tt=malloc(sizeof(struct track));
	tt->ft=ft;
	tt->data=NULL;
	switch(ft)
		{
		case FT_bed_n:
			tt->name=strdup(tok);
			tt->urlpath=makefilepath(tok, FT_bed_n);
			assert((tok=strtok(NULL,delim)) != NULL);
			assert((tt->mode=strMayPositiveInt(tok))!=-1);
			break;
		case FT_bed_c:
			tt->name=strdup(tok);
			assert((tok=strtok(NULL,delim)) != NULL);
			tt->label = strdup(tok);
			assert((tok=strtok(NULL,delim)) != NULL);
			tt->urlpath=strdup(tok);
			assert((tok=strtok(NULL,delim)) != NULL);
			assert((tt->mode=strMayPositiveInt(tok))!=-1);
			break;
		case FT_bedgraph_n:
			tt->name=strdup(tok);
			tt->urlpath=makefilepath(tok, FT_bedgraph_n);
			assert((tok=strtok(NULL,delim)) != NULL);
			assert((tt->mode=strMayPositiveInt(tok))!=-1);
			break;
		case FT_bedgraph_c:
			tt->name=strdup(tok);
			assert((tok=strtok(NULL,delim)) != NULL);
			tt->label = strdup(tok);
			assert((tok=strtok(NULL,delim)) != NULL);
			tt->urlpath=strdup(tok);
			assert((tok=strtok(NULL,delim)) != NULL);
			assert((tt->mode=strMayPositiveInt(tok))!=-1);
			break;
		case FT_sam_n:
			tt->name=strdup(tok);
			tt->urlpath=makefilepath(tok,FT_sam_n);
			assert((tok=strtok(NULL,delim))!=NULL);
			assert((tt->mode = strMayPositiveInt(tok))!=-1);
			break;
		case FT_sam_c:
			tt->name=strdup(tok);
			assert((tok=strtok(NULL,delim)) != NULL);
			tt->label=strdup(tok);
			assert((tok=strtok(NULL,delim)) != NULL);
			tt->urlpath=strdup(tok);
			assert((tok=strtok(NULL,delim)) != NULL);
			assert((tt->mode=strMayPositiveInt(tok))!=-1);
			break;
		case FT_bam_n:
			tt->name=strdup(tok);
			tt->urlpath=makefilepath(tok,FT_bam_n);
			assert((tok=strtok(NULL,delim))!=NULL);
			assert((tt->mode = strMayPositiveInt(tok))!=-1);
			break;
		case FT_bam_c:
			tt->name=strdup(tok);
			assert((tok=strtok(NULL,delim)) != NULL);
			tt->label=strdup(tok);
			assert((tok=strtok(NULL,delim)) != NULL);
			tt->urlpath=strdup(tok);
			assert((tok=strtok(NULL,delim)) != NULL);
			assert((tt->mode=strMayPositiveInt(tok))!=-1);
			break;
		case FT_qdecor_n:
			tt->name=strdup(tok);
			tt->urlpath=makefilepath(tok, FT_qdecor_n);
			assert((tok=strtok(NULL, delim))!=NULL);
			assert((tt->mode=strMayPositiveInt(tok))!=-1);
			break;
		case FT_lr_n:
			tt->name=strdup(tok);
			tt->urlpath=makefilepath(tok, FT_lr_n);
			assert((tok=strtok(NULL,delim)) != NULL);
			assert((tt->mode=strMayPositiveInt(tok))!=-1);
			assert((tok=strtok(NULL,delim)) != NULL);
			tt->pscore = strtod(tok, NULL);
			assert((tok=strtok(NULL,delim)) != NULL);
			tt->nscore = strtod(tok, NULL);
			break;
		case FT_lr_c:
			tt->name = strdup(tok);
			assert((tok=strtok(NULL,delim)) != NULL);
			tt->label = strdup(tok);
			assert((tok=strtok(NULL,delim)) != NULL);
			tt->urlpath = strdup(tok);
			assert((tok=strtok(NULL,delim)) != NULL);
			assert((tt->mode = strMayPositiveInt(tok))!=-1);
			assert((tok=strtok(NULL,delim)) != NULL);
			tt->pscore = strtod(tok, NULL);
			assert((tok=strtok(NULL,delim)) != NULL);
			tt->nscore = strtod(tok, NULL);
			break;
		case FT_cat_n:
			tt->name=strdup(tok);
			tt->urlpath=makefilepath(tok, FT_cat_n);
			assert((tok=strtok(NULL,delim)) != NULL);
			assert((tt->mode=strMayPositiveInt(tok))!=-1);
			break;
		case FT_cat_c:
			tt->name=strdup(tok);
			assert((tok=strtok(NULL,delim)) != NULL);
			tt->label = strdup(tok);
			assert((tok=strtok(NULL,delim)) != NULL);
			tt->urlpath=strdup(tok);
			assert((tok=strtok(NULL,delim)) != NULL);
			assert((tt->mode=strMayPositiveInt(tok))!=-1);
			break;
		case FT_bigwighmtk_n:
			tt->name=strdup(tok);
			tt->urlpath=makefilepath(tok, FT_bigwighmtk_n);
			assert((tok=strtok(NULL,delim)) != NULL);
			assert((tt->mode=strMayPositiveInt(tok))!=-1);
			break;
		case FT_bigwighmtk_c:
			tt->name=strdup(tok);
			assert((tok=strtok(NULL,delim)) != NULL);
			tt->label = strdup(tok);
			assert((tok=strtok(NULL,delim)) != NULL);
			tt->urlpath=strdup(tok);
			assert((tok=strtok(NULL,delim)) != NULL);
			assert((tt->mode=strMayPositiveInt(tok))!=-1);
			break;
		default:
			fprintf(stderr, "%s: unknown ft %d\n", __FUNCTION__, ft);
			exit(0);
		}
    tt->next = *sl;
    *sl = tt;
    tok = strtok(NULL, delim);
    }
}

void parseParamHtest(struct track *tracksl)
{
int grpnum = cgiInt("htestgrpnum");
char *tok; char delim[] = ",";
char *param;
struct track *track;
int i;
for(i=1; i<=grpnum; i++)
    {
    if(asprintf(&param, "htestgrp%d", i)<0) errabort("bae");
    tok = strtok(cgiString(param), delim); assert(tok!=NULL);
    while(tok != NULL)
	{
	for(track=tracksl; track!=NULL; track=track->next)
	    {
	    if(strcmp(track->name, tok)==0)
		{
		track->gid = i;
		break;
		}
	    }
	tok = strtok(NULL, delim);
	}
    }
}






void genestruct2json(struct gene *g)
{
if(g==NULL)
	{
	printf("[]");
	return;
	}
printf("[%d,%d,%d,%d,[", g->utr5start,g->utr5width,g->utr3start,g->utr3width);
int i=0;
for(; i<g->exoncount; i++)
	printf("[%d,%d]%s", (g->exons)[i][0],(g->exons)[i][1],i==(g->exoncount-1)?"":",");
printf("]]");
}

struct gene *makeGene(char *s_strand,char *s_txstart,char *s_txend,char *s_cdsstart,char *s_cdsend,char *s_exoncount,char *s_exonstarts,char *s_exonends)
{
/* 
strand must be +/-
assume to take output of [gene]struct table record
*/
int txstart = strMayPositiveInt(s_txstart);
if(txstart==-1)
	return NULL;
int txstop = strMayPositiveInt(s_txend);
if(txstop==-1)
	return NULL;
int cdsstart = strMayPositiveInt(s_cdsstart);
if(cdsstart==-1)
	return NULL;
int cdsstop = strMayPositiveInt(s_cdsend);
if(cdsstop==-1)
	return NULL;
char strand = s_strand[0];
int exoncount = strMayPositiveInt(s_exoncount);
if(exoncount==-1)
	return NULL;
// exons, dissect exon start/ends/frame string, put into arrays
int *exonstartarr = str2intArr(s_exonstarts, exoncount);
int *exonstoparr = str2intArr(s_exonends, exoncount);

struct gene *gene=malloc(sizeof(struct gene));
gene->next=NULL;
gene->txstart=txstart;
gene->txstop=txstop;
gene->strand=strand;
gene->flank5start=gene->flank5stop=gene->flank3start=gene->flank3stop=-1;

// must determine utr first then make exons as some 'exons' are actually utr

/* 5' UTR
start is always transcription start
if 1st exon stop is upstream of cds start, utr stops at 1st exon stop
else, utr stops at cds start
print out start/length
*/
if(strand == '+')
	{
	gene->utr5start=txstart;
	if(cdsstart > exonstoparr[0])
		{
		gene->utr5width=exonstoparr[0]-txstart;
		// make 1st exon unusable...
		exonstartarr[0] = -1;
		exonstoparr[0]=-1;
		}
	else
		{
		gene->utr5width=cdsstart-txstart;
		// reset first exon start if it is beyond cdsstart
		if(cdsstart > exonstartarr[0])
			exonstartarr[0] = cdsstart;
		}
	}
else
	{
	int utrstart;
	if(exonstoparr[exoncount-1] > cdsstop)
		{
		utrstart = exonstartarr[exoncount-1];
		// make last exon usuable...
		exonstartarr[exoncount-1]=-1; exonstoparr[exoncount-1]=-1;
		}
	else
		{
		utrstart = cdsstop;
		// reset last exon stop if it is beyond cdsstop
		if(exonstoparr[exoncount-1] > cdsstop)
			exonstoparr[exoncount-1] = cdsstop;
		}
	gene->utr5start=utrstart;
	gene->utr5width=txstop-utrstart;
	}
/* 3' UTR
 stop is always transcription stop
 if last exon start is upstream of cds stop, utr starts at cds stop
 else, utr starts at last exon start
 */
if(strand == '+')
	{
	int utrstart;
	if(exonstartarr[exoncount-1] > cdsstop)
		{
		utrstart = exonstartarr[exoncount-1];
		// make last exon usuable
		exonstartarr[exoncount-1]=-1; exonstoparr[exoncount-1]=-1;
		}
	else
		{
		utrstart = cdsstop;
		if(exonstoparr[exoncount-1] > cdsstop)
			exonstoparr[exoncount-1] = cdsstop;
		}
	gene->utr3start=utrstart;
	gene->utr3width=txstop-utrstart;
	}
else
	{
	gene->utr3start=txstart;
	if(cdsstart > exonstoparr[0])
		{
		gene->utr3width=exonstoparr[0]-txstart;
		// make first exon unusable
		exonstartarr[0]=-1; exonstoparr[0]=-1;
		}
	else
		{
		gene->utr3width=cdsstart-txstart;
		if(exonstartarr[0] < cdsstart)
			exonstartarr[0] = cdsstart;
		}
	}

/* exons
awkward as some exons might be removed
*/
gene->exoncount=0;
int j;
for(j=0; j<exoncount; j++)
	{
	if(exonstartarr[j]!=-1 && exonstoparr[j]!=-1)
		gene->exoncount++;
	}
gene->exons=malloc(gene->exoncount*sizeof(int *));
int k=0;
for(j=0; j<exoncount; j++)
	{
	if(exonstartarr[j]!=-1 && exonstoparr[j]!=-1)
		{
		int *this=malloc(2*sizeof(int));
		this[0]=exonstartarr[j];
		this[1]=exonstoparr[j];
		(gene->exons)[k++]=this;
		}
	}
free(exonstartarr);
free(exonstoparr);
return gene;
}




void tabixQuery_regiondensity(tabix_t *fin, samfile_t *fp, bam_index_t *bamidx, int ft, struct region *r, double *data)
{
/* fetch bed items for a region and compute density
only deal with coord, so use generic item
*/
int startArr[r->summarySize], stopArr[r->summarySize], i;
float pieceLength = (float)(r->dstop-r->dstart) / r->summarySize;
for(i=0; i<r->summarySize; i++)
	{
	startArr[i] = (int)(r->dstart + pieceLength*i);
	stopArr[i] = (int)(r->dstart + pieceLength*(i+1));
	}

struct genericItem *itemsl, *item;
if(fin!=NULL)
	itemsl=(struct genericItem *)tabixQuery_region(fin, ft, r);
else
	{
	char *tmp;
	assert(asprintf(&tmp,"%s:%d-%d",chrInfo[r->chromIdx]->name,r->dstart,r->dstop)>0);
	itemsl=(struct genericItem *)bamQuery_region(fp,bamidx,tmp);
	free(tmp);
	}

for(item=itemsl; item!=NULL; item=item->next)
	{
	// collapse this bed item onto the density counter
	for(i=0; i<r->summarySize; i++)
		{
		if(item->start > stopArr[i]) continue;
		if(item->stop < startArr[i]) break;
		if(intMax(item->start,startArr[i]) < intMax(item->stop,stopArr[i])) data[i]++;
		}
	}
// TODO free
}

void printJson_itemDensity(struct displayedRegion *dsp, char *urlpath, int ft)
{
/* decor density data printed out, not storing
*/
struct region *r;
int i;

tabix_t *fin=NULL;
samfile_t *fp=NULL;
bam_index_t *bamidx=NULL;

if(ft==FT_bam_n||ft==FT_bam_c)
	{
	if((fp=samopen(urlpath,"rb",0))==0)
		{
		printf("null");
		return;
		}
	if((bamidx=bam_parseIndex(urlpath,ft))==NULL)
		{
		printf("'abort':'cannot load bam index %s',", urlpath);
		return;
		}
	}
else
	{
	fin=tabixOpen(urlpath);
	if(fin==NULL)
		{
		printf("'abort':'bad tabix file %s',", urlpath);
		return;
		}
	}

printf("[");
for(r=dsp->head; r!=NULL; r=r->next)
    {
    double *data = malloc(sizeof(double)*r->summarySize);
    for(i=0; i<r->summarySize; i++) 
        data[i] = 0;
    tabixQuery_regiondensity(fin, fp, bamidx, ft, r, data);
    printf("[");
    for(i=0; i<r->summarySize; i++)
		printf("%d,", (int)data[i]);
    printf("],");
    free(data);
    }
printf("],");

if(fin!=NULL)
	ti_close(fin);
else
	{
	bam_index_destroy(bamidx);
	samclose(fp);
	}
//if(SQUAWK) fprintf(stderr, "ok: %s(%s)\n", __FUNCTION__, urlpath);
}



void printJson_bed_scorefilter(struct displayedRegion *dsp, char *urlpath, int ft, float pscoreCutoff, float nscoreCutoff)
{
/* args:
     pscoreCutoff/nscoreCutoff: score cutoff

will apply region filtering when dsp->dspFilter is not null
will only fetch items whose mate is within the region list
to restrict the amount of retrieved data with hi-c tracks

difference with printJson_bedstack()
- it won't deal with gene tracks, only long-range??
- always apply score cutoff
- no stacking
- may apply within region filtering

set a global variable to tell how many items are skipped
*/

if(0) fprintf(stderr, "%s %f %f\n", urlpath, pscoreCutoff, nscoreCutoff);

__skipped_num=0;

char delim[] = "\t";
char delim2[] = ",";

/* this is guaranteed to be long-range */
struct beditem *item;
struct beditem **data=(struct beditem **)tabixQuery_dsp(dsp, urlpath, ft);

int dataidx=0; // array idx of data corresponding to region

struct region *r;
struct nnode *fi;
boolean outsidedspfilter;

boolean applyDspFilter=dsp->dspFilter!=NULL;

if(0)
	{
	for(fi=dsp->dspFilter; fi!=NULL; fi=fi->next)
		fprintf(stderr, "filter %s %d %d\n", chrInfo[fi->chrIdx]->name, fi->start, fi->stop);
	}

char *name, *id, *strand; // to be used at each bed item
printf("[");
for(r=dsp->head; r!=NULL; r=r->next)
    {
    printf("[");
    for(item=data[dataidx]; item!=NULL; item=item->next)
        {
		if((name=strtok(item->rest,delim))==NULL) continue;
		if(strcmp(name, ".")==0) continue;
		if((id=strtok(NULL,delim))==NULL) continue;
		strand=strtok(NULL,delim);

		// apply score cutoff
		char *namec = strdup(name);
		char *matecoordstr = strtok(namec, delim2);
		if(matecoordstr==NULL) continue;
		char *tmp = strtok(NULL, delim2);
		if(tmp==NULL) continue; // supposed to be the score
		float thisscore = strtod(tmp, NULL);

		if(thisscore >= 0)
			{
			if(thisscore<pscoreCutoff) continue;
			}
		else
			{
			if(thisscore>nscoreCutoff) continue;
			}

		if(applyDspFilter)
			{
			int *matecoord = dissectCoordString(matecoordstr);
			/* 2013/2/16 forgot to check if this function returns NULL ended up errors, blast it...
			TODO should use chrom name, not idx
			*/
			if(matecoord==NULL || matecoord[0]==-1 || matecoord[1]==-1 || matecoord[2]==-1) continue;
			outsidedspfilter=TRUE;
			for(fi=dsp->dspFilter; fi!=NULL; fi=fi->next)
				{
				if(matecoord[0]==fi->chrIdx && (max(matecoord[1],fi->start)<=min(matecoord[2],fi->stop)))
					{
					outsidedspfilter=FALSE;
					break;
					}
				}
			free(matecoord);
			if(outsidedspfilter)
				{
				__skipped_num++;
				continue;
				}
			}

		printf("[,,%s,", id);
		if(name != NULL)
			printf("'%s',", name);
		else
			printf(",");
		if(strand != NULL)
			printf("'%c',", strand[0]=='+'?'>':(strand[0]=='-'?'<':strand[0]));
		else
			printf(",");
		printf(",,,%d,%d,,,,,,,],",item->start,item->stop);
		}
	dataidx++;
    printf("],");
    }
printf("],");

/*
dataidx=0;
for(r=dsp->head; r!=NULL; r=r->next)
	tabixFreeResult(data[dataidx++]);
	*/
free(data);

if(SQUAWK) fprintf(stderr, "ok: %s(%s) %d skipped\n", __FUNCTION__, urlpath, __skipped_num);
return;
}


int printJson_bedstack(struct displayedRegion *dsp, char *urlpath, int ft, char *decorname, float *pscoreCutoffPtr, float *nscoreCutoffPtr)
{
/* args:
     file: path or url
	 ft: file type
     decorname: NULL if it is custom
     scoreCutoffPtr: pointer to float or NULL
       if NULL, won't apply cutoff

   nasty similarity with printJson_bed_scorefilter()

   score filtering is only reserved for future use!

   do a preliminary stacking only using item coordinate,
   stacking for each region separately,

   for gene track, tell from belonging to gene group and the hasStruct field is True
   will retrieve symbol and struct data for each item

   for gene part tracks like promoters, tell from belonging to gene group,
   but hasStruct field is False, will only retrieve symbol

   !! symbol or struct table name is assumed to be [x]symbol, [x]struct
      where [x] is decorname

   only purpose of stacking here is to prevent from sending too many items to clientside
   TODO return # skipped items
        identify region with skipped items, report to JS
   TODO should be extendable for generic genomic feature track with structure
        where only struct data will be retrieved, not symbol
   TODO better error report
 */


boolean queryStruct = FALSE;
boolean querySymbol = FALSE;
// in case track is promoter track, symbol table will take name of parent track
char *symbolTableBasename = NULL;
char *structTableBasename = NULL;

char *query;

/* decide whether it is a gene track
and if to query gene structure */
if(decorname!=NULL && genetrack_sl!=NULL)
    {
    int genegrpid = mysqlGetGenesGroupId();
	if(genegrpid!=-1)
		{
		if(asprintf(&query, "select grp from decorInfo where name=\"%s\"", decorname)<0) errabort("bae");
		int thisgrpid = mysqlGetPositiveInteger(query);
		free(query);
		if(thisgrpid == genegrpid)
			{
			// will query symbol, is gene or gene part (promoter stuff)
			querySymbol = TRUE;
			if(asprintf(&query, "select parent from decorInfo where name=\"%s\"", decorname)<0) errabort("bae");
			char *parentName = mysqlGetOnestring(query);
			free(query);
			if(parentName == NULL)
				{
				// no parent, so it is a gene track, see if it has structure
				symbolTableBasename = decorname;
				if(asprintf(&query, "select hasStruct from decorInfo where name='%s'", decorname)<0) errabort("bae");
				char *re = mysqlGetOnestring(query);
				free(query);
				if(re[0] == '1')
					{
					queryStruct = TRUE;
					structTableBasename = decorname;
					}
				}
			else
				{
				// has parent, so symbol table takes name of parent track
				symbolTableBasename = parentName;
				// currently gene part doesn't have structure, but if things change, extend here
				}
			}
		else
			{
			/* TODO extend here for generic genomic feature tracks that don't belong to gene group
			   test if the track has structure
			 */
			}
		}
	}

int stackNumber=300, stackIdx; // TODO make it a parameter
int stack[stackNumber];
//int id_strand=0, id_txstart=1, id_txend=2, id_cdsstart=3, id_cdsend=4, id_exoncount=5, id_exonstarts=6, id_exonends=7;
char delim[] = "\t";
char delim2[] = ",";
int returnedCount=0,skipCount=0;
MYSQL_RES *sr;
char **row;

float pscoreCutoff=0, nscoreCutoff=0;
if(pscoreCutoffPtr != NULL)
    {
    pscoreCutoff = *pscoreCutoffPtr;
    nscoreCutoff = *nscoreCutoffPtr;
    }

struct genericItem *item;
struct genericItem **data=(struct genericItem **)tabixQuery_dsp(dsp, urlpath, ft);

boolean is_sambam=(ft==FT_sam_n||ft==FT_sam_c||ft==FT_bam_n||ft==FT_bam_c);

int dataidx=0;

printf("[");
char *name=NULL, *id=NULL, *strand=NULL; // to be used at each bed item, but not sam read
struct region *r;
for(r=dsp->head; r!=NULL; r=r->next)
    {
    // stacking using item coordinate, so do separately for each region
    for(stackIdx=0; stackIdx<stackNumber; stackIdx++)
		stack[stackIdx] = 0; // will be item coordinate
	printf("[");
	struct genericItem *itemsl=beditemsort_startAsc(data[dataidx]);
	for(item=itemsl; item!=NULL; item=item->next)
        {
		if(!is_sambam)
			{
			/** is bed item **/
			// chop up item->rest
			struct beditem *item2=(struct beditem *)item;
			if((name=strtok(item2->rest,delim))==NULL) continue;
			if((id=strtok(NULL,delim))==NULL) continue;
			strand=strtok(NULL,delim);
			// done chopping

			if(strcmp(name, ".") == 0) name = NULL;

			/* see if to apply score cutoff
			lots of freaky assumptions here, only apply to long-range data
			a copy of the name string will be chopped
			likely to be revised in future */
			if(pscoreCutoffPtr != NULL)
				{
				// only long-range track has score, it need to be chopped off from the name
				char *namec = strdup(name);
				strtok(namec, delim2);
				char *tmp = strtok(NULL, delim2);
				if(tmp == NULL)
					continue;
				float thisscore = strtod(tmp, NULL);
				if(thisscore >= 0)
					{
					if(thisscore<pscoreCutoff) continue;
					}
				else
					{
					if(thisscore>nscoreCutoff) continue;
					}
				}
			}

	    // stacking
		boolean notStacked = TRUE;
	    for(stackIdx=0; stackIdx<stackNumber; stackIdx++)
			{
			if(stack[stackIdx] <= item->start)
				{
				// slot found
				stack[stackIdx] = item->stop;
				notStacked = FALSE;
				returnedCount++;
				break;
				}
			}
		if(notStacked) 
			{
			skipCount++;
			continue;
			}

		// compose json array
		if(is_sambam)
			{
			/** sam read output **/
			struct readitem *read=(struct readitem *)item;
			/*** do not output read stop */
			printf("[,,'%s',,,,,,%d,,,{'cigar':[", read->id, read->start);
			// must cigar
			{
			int i,op,cl;
			for(i=0; i<read->n_cigar;i++)
				{
				op=(read->cigar)[i] & BAM_CIGAR_MASK;
				cl=(read->cigar)[i] >> BAM_CIGAR_SHIFT;
				printf("['");
				switch(op)
					{
					case BAM_CMATCH: printf("M");break;
					case BAM_CINS: printf("I");break;
					case BAM_CDEL: printf("D");break;
					case BAM_CREF_SKIP: printf("N"); break;
					case BAM_CSOFT_CLIP: printf("S");break;
					case BAM_CHARD_CLIP: printf("R");break;
					case BAM_CPAD: printf("P");break;
					default:printf("?");break;
					}
				printf("',%d],",cl);
				}
			}
			// must flag
			printf("],'flag':%d,",read->flag);
			// must seq
			printf("'seq':'%s'",read->seq);
			// optional mismatch
			if(read->mismatch!=NULL)
				printf(",'mismatch':'%s'",read->mismatch);
			printf("},,,,,false],");
			// with default bam file the read sequence sometimes contain invalid character, need double quote
			}
		else
			{
			/** bed item output **/
			printf("[,,%s,", id);
			if(name != NULL)
				printf("'%s',", name);
			else
				printf(",");
			if(strand != NULL)
				printf("'%c',", strand[0]=='+'?'>':(strand[0]=='-'?'<':strand[0]));
			else
				printf(",");
			printf(",,,%d,%d,,,,,,",item->start,item->stop);
			if(querySymbol && symbolTableBasename!=NULL)
				{
				if(asprintf(&query, "select symbol from %ssymbol where id=%s", symbolTableBasename, id)<0) errabort("bae");
				char *symbol = mysqlGetOnestring(query);
				free(query);
				if(symbol != NULL)
					printf("'%s'", symbol);
				free(symbol);
				}
			printf(",");

			if(queryStruct && structTableBasename!=NULL)
				{
				if(asprintf(&query, "select strand,txStart,txEnd,cdsStart,cdsEnd,exonCount,exonStarts,exonEnds from %sstruct where id=%s", structTableBasename, id)<0) errabort("bae");
				sr=mysqlGetResult(query);
				assert(sr!=NULL);
				free(query);
				row = mysql_fetch_row(sr);
				if(row==NULL ||
					row[0]==NULL ||
					row[1]==NULL ||
					row[2]==NULL ||
					row[3]==NULL ||
					row[4]==NULL ||
					row[5]==NULL ||
					row[6]==NULL ||
					row[7]==NULL)
					{
					// don't break program
					}
				else
					{
					struct gene *g=makeGene(row[0],row[1],row[2],row[3],row[4],row[5],row[6],row[7]);
					genestruct2json(g);
					}
				mysql_free_result(sr);
				}
			printf("],");
			}
	    }
	dataidx++; // never forget this...
    printf("],");
    }
printf("],");

/*
dataidx=0;
for(r=dsp->head; r!=NULL; r=r->next)
	tabixFreeResult(data[dataidx++]);
	*/
free(data);

if(SQUAWK) fprintf(stderr, "%s(%s), %d returned, %d skipped\n", __FUNCTION__, urlpath, returnedCount, skipCount);
return skipCount;
}





void printJsonDecor(struct displayedRegion *dsp, struct track *tt)
{
/* grand wrapper for decor processing routines
   takes one decor track, handles it according to its mode and file type
   expandable to support other file types and modes
 */
if(tt->mode==M_show)
    {
    if(tt->ft == FT_qdecor_n)
        printJson_dsp_spnum(dsp, tt->urlpath, FT_qdecor_n);
    else
        errabort("printJsonDecor: decor mode show, but file type is not native qdecor");
    return;
    }
if(tt->mode==M_den)
    {
    printJson_itemDensity(dsp, tt->urlpath, tt->ft);
    return;
    }
switch(tt->ft)
	{
	case FT_lr_n:
	case FT_lr_c:
		/* long-range track
		will always apply score cutoff
		if in arc/trihm mode and over chiapet panel, will apply within region filtering
		*/
		printJson_bed_scorefilter(dsp, tt->urlpath, tt->ft, tt->pscore, tt->nscore);
    	return;
	case FT_bed_n:
    	printJson_bedstack(dsp, tt->urlpath, tt->ft, tt->name, NULL,NULL);
		return;
	case FT_bed_c:
    	printJson_bedstack(dsp, tt->urlpath, tt->ft, NULL, NULL,NULL);
		return;
	case FT_sam_n:
	case FT_sam_c:
	case FT_bam_n:
	case FT_bam_c:
    	printJson_bedstack(dsp, tt->urlpath, tt->ft, NULL,NULL,NULL);
		return;
	default:
    	fprintf(stderr,"printJsonDecor: unknown ft (%d)\n", tt->ft);
		exit(0);
	}
}







char strMayGetgeneStrand(struct slNode *genetrack_sl, char *what)
{
/* query gene table for given string, if found, return its strand +/-
   else, return '.'

   The strand must be +/-
 */
struct slNode *table;
char *query;
for(table=genetrack_sl; table!=NULL; table=table->next)
    {
    if(asprintf(&query, "select t.strand from %sstruct as t,%ssymbol as s where (s.name=\"%s\" or s.symbol=\"%s\") and s.id=t.id", table->name, table->name, what, what)<0) errabort("bae");
    char *re = mysqlGetOnestring(query);
    if(re != NULL)
        return re[0];
    }
return '.';
}




long getScaffoldLength(char *name)
{
/* TODO use bsearch
to overcome speed issue when there's 80000 scaffolds
*/
char *query;
assert(asprintf(&query, "select childLength from scaffoldInfo where child='%s'", name)>0);
long len = mysqlGetPositiveInteger(query);
free(query);
return len;
}

void reviveScaffold()
{
/* session and status obtained from global variable
   given session and status, revive scaffold from table scaffoldRuntime
 */
char *query;
if(asprintf(&query, "select count,names from scaffoldRuntime where session='%s' and statusId=%d", session, statusId)<0) errabort("bae");
MYSQL_RES *sr = mysqlGetResult(query);
assert(sr!=NULL);
free(query);
char **row = mysql_fetch_row(sr);
assert(row[0] != NULL);
assert(row[1] != NULL);
assert((lastChrIdx=strMayPositiveInt(row[0]))!=-1);
chrInfo = malloc(sizeof(struct chrInfoNode *) * lastChrIdx);
lastChrIdx--;
int i = 0;
char delim[] = ",";
char *tok = strtok(row[1], delim);
while(tok != NULL)
    {
    chrInfo[i] = (struct chrInfoNode *)malloc(sizeof(struct chrInfoNode));
	chrInfo[i]->name=strdup(tok);
    tok = strtok(NULL, delim);
    assert(tok != NULL);
    assert((chrInfo[i]->length=strMayPositiveInt(tok))!=-1);
    i++;
    tok = strtok(NULL, delim);
    }
assert(i == lastChrIdx+1);
mysql_free_result(sr);
if(SQUAWK)
    fputs("reviveScaffold() done\n", stderr);
if(0)
    {
    for(i=0; i<=lastChrIdx; i++)
        fprintf(stderr, "%s %u\n", chrInfo[i]->name, chrInfo[i]->length);
    }
}

void scaffoldAddnew(char *name)
{
/* runtime adding new scaffold into chrInfo and runtime table,
   not reviving!
   lucky we are dealing with a bunch of global variables...
 */
// make new and extended chrInfo array
if(SQUAWK)
    fprintf(stderr, "scaffoldAddnew() adding %s...\n", name);
lastChrIdx++;
struct chrInfoNode **new = malloc(sizeof(struct chrInfoNode *) * (lastChrIdx+1));
int i = 0;
for(; i<lastChrIdx; i++)
    {
    new[i] = malloc(sizeof(struct chrInfoNode));
    new[i]->name = strdup(chrInfo[i]->name);
    new[i]->length = chrInfo[i]->length;
    }
new[lastChrIdx] = malloc(sizeof(struct chrInfoNode));
new[lastChrIdx]->name = strdup(name);
new[lastChrIdx]->length = getScaffoldLength(name);
assert(new[lastChrIdx]->length != -1);
free(chrInfo);
chrInfo = new;
if(SQUAWK)
    fputs("chrInfo extended\n", stderr);
// update runtime table
char *query;
if(asprintf(&query, "select names from scaffoldRuntime where session='%s' and statusId=%d", session, statusId)<0) errabort("bae");
char *tmp = mysqlGetOnestring(query);
free(query);
if(asprintf(&query, "update scaffoldRuntime set count=%d,names='%s%s,%u,' where session='%s' and statusId=%d", lastChrIdx+1, tmp, chrInfo[lastChrIdx]->name, chrInfo[lastChrIdx]->length, session, statusId)<0) errabort("bae");
mysqlUpdate(query);
free(query);
}


struct gene *queryGene(struct geneParam *param)
{
/* query gene with an input string
will yield a list
normalchr_sl must be ready
genetrack_sl is needed if no specific table is appointed
*/
struct slNode *_genetrack;
if(param->query_table==NULL)
	_genetrack=genetrack_sl;
else
	{
	_genetrack=malloc(sizeof(struct slNode));
	_genetrack->name=param->query_table;
	_genetrack->next=NULL;
	}
struct slNode *table, *rawresult=NULL, *re;
char *query, **row;
int i;
MYSQL_RES *sr;
for(table=_genetrack; table!=NULL; table=table->next)
	{
	if(param->query_id!=-1)
		{
		// using exact gene id
		assert(asprintf(&query, "select st.*,sy.* from %sstruct as st,%ssymbol as sy where sy.id=%d and sy.id=st.id", table->name, table->name, param->query_id)>0);
		}
	else
		{
		// using string keyword
		if(param->use_exactmatch)
			assert(asprintf(&query, "select st.*,sy.* from %sstruct as st,%ssymbol as sy where (sy.name=\"%s\" or sy.symbol=\"%s\") and sy.id=st.id", table->name, table->name, param->query_str, param->query_str)>0);
		else
			assert(asprintf(&query, "select st.*,sy.* from %sstruct as st,%ssymbol as sy where (sy.name like \"%%%s%%\" or sy.symbol like \"%%%s%%\") and sy.id=st.id", table->name, table->name, param->query_str, param->query_str)>0);
		}
	sr = mysqlGetResult(query);
	assert(sr!=NULL);
	free(query);
	while((row=mysql_fetch_row(sr))!=NULL)
		{
		/* 0-10 are fields from [x]struct
		11-13 are fields from [x]symbol
		*/
		re=malloc(sizeof(struct slNode));
		re->arr=malloc(20*sizeof(char *));
		for(i=0; i<14; i++)
			(re->arr)[i]=strdup(row[i]);
		(re->arr)[15]=table->name;
		re->next=rawresult;
		rawresult=re;
		}
	mysql_free_result(sr);
	}
struct gene *result=NULL, *g;
for(re=rawresult; re!=NULL; re=re->next)
	{
	row=re->arr;
	g=makeGene(row[2],row[3],row[4],row[5],row[6],row[7],row[8],row[9]);
	if(g==NULL) continue;
	g->on_normalchr=slLookup(normalchr_sl,row[1],FALSE);
	if(param->use_chromid)
		{
		g->chrom=NULL;
		g->chromIdx=strMayChrID(row[1]);
		if(g->chromIdx==-1)
			{
			scaffoldAddnew(row[1]);
			g->chromIdx=lastChrIdx;
			}
		}
	else
		{
		g->chrom=row[1];
		g->chromIdx=-1;
		}
	g->name=row[12];
	g->id=strMayPositiveInt(row[0]);
	g->text=row[13];
	g->type=row[15];

	if(!param->use_chromid)
		{
		g->next=result;
		result=g;
		continue;
		}
	long thischrlen = chrInfo[g->chromIdx]->length;
	/* below sets report_start/stop and flanking according to which genepart to report
	do not process "exons/introns" as gene already reports all exons
	*/
	if(strcmp(param->whichgenepart, "genebodypromoter") == 0)
		{
		if(g->strand == '+')
			{
			g->report_stop=g->txstop;
			g->flank5stop=g->txstart;
			g->flank5start = g->report_start = max(0,g->txstart-3000);
			}
		else
			{
			g->report_start=g->txstart;
			g->flank5start = g->txstop;
			g->flank5stop= g->report_stop = min(thischrlen, g->txstop+3000);
			}
		}
	else if(strcmp(param->whichgenepart,"promoter")==0)
		{
		if(g->strand=='+')
			{
			g->report_start=max(0,g->txstart-3000);
			g->report_stop=g->txstart;
			}
		else
			{
			g->report_start=g->txstop;
			g->report_stop=min(thischrlen,g->txstop+3000);
			}
		}
	else if(strcmp(param->whichgenepart,"utr5")==0)
		{
		g->report_start=g->utr5start;
		g->report_stop=g->utr5start+g->utr5width;
		}
	else if(strcmp(param->whichgenepart,"utr3")==0)
		{
		g->report_start=g->utr3start;
		g->report_stop=g->utr3start+g->utr3width;
		}
	else if(strcmp(param->whichgenepart, "custom") == 0)
		{
		if(strcmp(param->origintype, "txstart") == 0)
			{
			if(g->strand == '+')
				{
				g->flank5stop=g->flank3start=g->txstart;
				g->report_stop=g->flank3stop=min(thischrlen,g->txstart+param->flank3);
				g->report_start=g->flank5start=max(0,g->txstart-param->flank5);
				}
			else
				{
				g->flank3stop=g->flank5start=g->txstop;
				g->report_start=g->flank3start=max(0,g->txstop-param->flank3);
				g->report_stop=g->flank5stop=min(thischrlen,g->txstop+param->flank5);
				}
			}
		else if(strcmp(param->origintype, "txstop") == 0)
			{
			if(g->strand == '+')
				{
				g->flank5stop=g->flank3start=g->txstop;
				g->report_start=g->flank5start=max(0,g->txstop-param->flank5);
				g->report_stop=g->flank3stop=min(thischrlen,g->txstop+param->flank3);
				}
			else
				{
				g->flank5start=g->flank3stop=g->txstart;
				g->report_stop=g->flank5stop=min(thischrlen,g->txstart+param->flank5);
				g->report_start=g->flank3start=max(0,g->txstart-param->flank3);
				}
			}
		else if(strcmp(param->origintype, "genebody") == 0)
			{
			if(g->strand == '+')
				{
				g->flank5stop=g->txstart;
				g->report_start=g->flank5start=max(0,g->txstart-param->flank5);
				g->flank3start=g->txstop;
				g->report_stop=g->flank3stop=min(thischrlen,g->txstop+param->flank3);
				}
			else
				{
				g->flank3stop=g->txstart;
				g->flank5start=g->txstop;
				g->report_start=g->flank3start=max(0,g->txstart-param->flank3);
				g->report_stop=g->flank5stop=min(thischrlen,g->txstop+param->flank5);
				}
			}
		}
	else
		{
		g->report_start=g->txstart;
		g->report_stop=g->txstop;
		}
	g->next=result;
	result=g;
	}
return result;
}




void makeDspBoundaryFromCgi(struct displayedRegion *dsp)
{
char *s=cgiString(StartChrom_cgi);
assert(s!=NULL);
int startChr = strMayChrID(s);
assert(startChr!=-1);
int startCoord = cgiInt(StartCoord_cgi);
s=cgiString(StopChrom_cgi);
assert(s!=NULL);
int stopChr = strMayChrID(s);
assert(stopChr!=-1);
int stopCoord = cgiInt(StopCoord_cgi);
setDspBoundary(dsp, startChr, startCoord, stopChr, stopCoord);
}

void printJson_hmtk4datasets(struct slNode *sl)
{
/*** use of hash here **

argument is a list of data set name

assemble hmtk info from multiple tables to:
- reduce amount of json text to transfer
- reduce number of mysql queries
tables:
- track2Label for label
- track2Annotation for metadata annotation
- track2Ft for filetype
- track2Categorical for categorical info
*/
// count # of hmtk
char *query, **row;
int count=0;
struct slNode *t;
struct slNode *tksl=NULL, *newtk; // as there's no way to iterate the hash
struct tnode *record;
for(t=sl; t!=NULL; t=t->next)
	{
	assert(asprintf(&query, "select count(tkname) from %s", t->name)>0);
	int c=mysqlGetPositiveInteger(query);
	assert(c!=-1);
	count+=c;
	free(query);
	}
assert(hcreate((size_t)count*1.25)!=0);
// for each dataset
for(t=sl; t!=NULL; t=t->next)
	{
	// label
	assert(asprintf(&query, "select b.name,b.label from track2Label as b,%s as a where a.tkname=b.name", t->name)>0);
	MYSQL_RES *sr=mysqlGetResult(query);
	assert(sr!=NULL);
	free(query);
	while((row=mysql_fetch_row(sr))!=NULL)
		{
		assert(row[0]!=NULL);
		assert(row[1]!=NULL);
		record=malloc(sizeof(struct tnode));
		record->t1=strdup(row[1]);
		record->t2=NULL; record->t4=-1;
		ENTRY ent;
		ent.key=strdup(row[0]);
		ent.data=(char *)record;
		hsearch(ent, ENTER);
		newtk=malloc(sizeof(struct slNode));
		newtk->name=ent.key;
		newtk->next=tksl;
		tksl=newtk;
		}
	mysql_free_result(sr);

	// md anno
	assert(asprintf(&query, "select b.name,b.attridx from track2Annotation as b,%s as a where a.tkname=b.name", t->name)>0);
	sr=mysqlGetResult(query);
	assert(sr!=NULL);
	free(query);
	while((row=mysql_fetch_row(sr))!=NULL)
		{
		assert(row[0]!=NULL);
		assert(row[1]!=NULL);
		ENTRY ent;
		ent.key=strdup(row[0]);
		ENTRY *ent2=hsearch(ent,FIND);
		if(ent2==NULL)
			{
			// enter
			record=malloc(sizeof(struct tnode));
			record->t2=strdup(row[1]);
			record->t1=NULL; record->t4=-1;
			ent.data=(char *)record;
			hsearch(ent, ENTER);
			newtk=malloc(sizeof(struct slNode));
			newtk->name=ent.key;
			newtk->next=tksl;
			tksl=newtk;
			}
		else
			{
			record=(struct tnode *)(ent2->data);
			record->t2=strdup(row[1]);
			free(ent.key);
			}
		}
	mysql_free_result(sr);

	// filetype, all tracks must have file type
	assert(asprintf(&query, "select b.name,b.ft from track2Ft as b,%s as a where a.tkname=b.name", t->name)>0);
	sr=mysqlGetResult(query);
	assert(sr!=NULL);
	free(query);
	while((row=mysql_fetch_row(sr))!=NULL)
		{
		assert(row[0]!=NULL);
		assert(row[1]!=NULL);
		ENTRY ent;
		ent.key=strdup(row[0]);
		ENTRY *ent2=hsearch(ent,FIND);
		assert(ent2!=NULL);
		record=(struct tnode *)(ent2->data);
		record->t4=strMayPositiveInt(row[1]);
		free(ent.key);
		}
	mysql_free_result(sr);
	}


// json output
// meanwhile query info for categorical tracks
printf("'hmtkinfo':{");
for(newtk=tksl; newtk!=NULL; newtk=newtk->next)
	{
	ENTRY ent;
	ent.key=newtk->name;
	ENTRY *ent2=hsearch(ent, FIND);
	assert(ent2!=NULL);
	record=(struct tnode *)(ent2->data);
	printf("'%s':{label:\"%s\", md:[%s], ft:%d,", newtk->name, record->t1, record->t2, record->t4);
	if(record->t4==FT_cat_n)
		{
		assert(asprintf(&query, "select info from track2Categorical where name='%s'", newtk->name)>0);
		char *re=mysqlGetOnestring(query);
		free(query);
		if(re!=NULL)
			{
			printf("cateInfo:{%s}", re);
			free(re);
			}
		}
	printf("},");
	free(newtk->name);
	free(record->t1);
	free(record->t2);
	free(record);
	}
printf("},");
slFree(newtk);
hdestroy();
}




struct slNode *findDatasetByHmtksl(struct slNode *tksl)
{
// input is list of hmtk names
struct tnode *dsl=NULL, *d; // complete list of data set
MYSQL_RES *sr=mysqlGetResult("select tablename from dataset");
assert(sr!=NULL);
char **row;
while((row=mysql_fetch_row(sr))!=NULL)
	{
	d=malloc(sizeof(struct tnode));
	d->t1=strdup(row[0]);
	d->t4=FALSE;
	d->next=dsl;
	dsl=d;
	}
assert(dsl!=NULL);
mysql_free_result(sr);
char *query;

struct slNode *tk;
for(tk=tksl; tk!=NULL; tk=tk->next)
	{
	for(d=dsl; d!=NULL; d=d->next)
		{
		if(!d->t4)
			{
			assert(asprintf(&query, "select tkname from %s where tkname='%s'", d->t1, tk->name)>0);
			if(mysqlResultAffirmative(query))
				{
				d->t4=TRUE;
				}
			free(query);
			}
		}
	}
struct slNode *dsinuse=NULL, *d2; // datasets in use
for(d=dsl; d!=NULL; d=d->next)
	{
	if(!d->t4) continue;
	d2=malloc(sizeof(struct slNode));
	d2->name=strdup(d->t1);
	d2->next=dsinuse;
	dsinuse=d2;
	}
slFree(dsl);
return dsinuse;
}


void recoverDsp_cgiParam(struct displayedRegion *dsp)
{
dsp->usedSummaryNumber = 0;
char *param = cgiString("regionLst");
assert(param!=NULL);
struct region *r = NULL;
char delim[] = ",";
char *tok = strtok(param, delim);
assert(tok!=NULL);
while(tok != NULL)
	{
	r = malloc(sizeof(struct region));
	r->chromIdx = strMayChrID(tok);
	if(r->chromIdx == -1)
		{
		scaffoldAddnew(tok);
		r->chromIdx = lastChrIdx;
		printf("'newscaffold':['%s',%u],", tok, chrInfo[lastChrIdx]->length);
		}
	assert((tok=strtok(NULL, delim))!=NULL);
	assert((r->bstart=r->dstart=strMayPositiveInt(tok))!=-1);
	assert((tok=strtok(NULL, delim))!=NULL);
	assert((r->bstop=r->dstop=strMayPositiveInt(tok))!=-1);
	assert((tok=strtok(NULL, delim))!=NULL);
	assert((r->summarySize=strMayPositiveInt(tok))!=-1);
	if(dsp->runmode==RM_gsv_c || dsp->runmode==RM_gsv_kegg)
		{
		assert((tok=strtok(NULL, delim))!=NULL);
		r->name=strdup(tok);
		}

	dsp->usedSummaryNumber += r->summarySize;
	r->isViewStart = FALSE;
	dspAddRegionTail(dsp, r);
	tok = strtok(NULL, delim);
	}
dsp->head->dstart = cgiInt("startCoord");
dsp->tail->dstop = cgiInt("stopCoord");
}


void defaultScaffold()
{
char *tmp = mysqlGetOnestring("select defaultScaffold from config");
// need to count number of scaffold, and make new string (mix of name and length)
char *old="", *new;
boolean atfirst=TRUE;
int i = 0; // # of names
char delim[]=",";
char *tok = strtok(tmp, delim);
while(tok!=NULL)
	{
	i++;
	int len = getScaffoldLength(tok);
	assert(len != -1);
	assert(asprintf(&new, "%s%s,%d,", old, tok, len)>0);
	if(atfirst) atfirst=FALSE;
	else free(old);
	old = new;
	tok = strtok(NULL, delim);
	}
char *query;
assert(asprintf(&query, "insert into scaffoldRuntime values('%s',%d,%d,'%s')", session, statusId, i, old)>0);
mysqlUpdate(query);
free(query);
free(old);
reviveScaffold();
}


char *dspStat2readable(char *dspstat, char *juxtype)
{
// to human readable, free after use
char delim[]=",";
char *startChr;
if((startChr=strtok(dspstat, delim)) == NULL)
	errabort("expecting start chr from dspStat string but got null...");
char *startCoord;
if((startCoord=strtok(NULL, delim)) == NULL)
	errabort("expecting start coord from dspStat string but got null...");
char *stopChr;
if((stopChr=strtok(NULL, delim)) == NULL)
	errabort("expecting stop chr from dspStat string but got null...");
char *stopCoord;
if((stopCoord=strtok(NULL, delim)) == NULL)
	errabort("expecting stop coord from dspStat string but got null...");
char *re;
if(juxtype[0]=='3' || juxtype[0]=='4')
	{
	// gsv
	if(strcmp(startChr,stopChr) == 0)
		assert(asprintf(&re, "at %s", startChr)>0);
	else
		assert(asprintf(&re, "from %s to %s", startChr, stopChr)>0);
	}
else
	{
	if(strcmp(startChr,stopChr) == 0)
		assert(asprintf(&re,"%s from %s to %s", startChr, startCoord, stopCoord)>0);
	else
		assert(asprintf(&re,"from %s(%s) to %s(%s)", startChr, startCoord, stopChr, stopCoord)>0);
	}
return re;
}



void sessionstatus2json(char *oldsession)
{
// database connection to the genomedb must be ready
printf("'statusLst':['%s',", oldsession);
char *query, **row;
// see if session exists
assert(asprintf(&query, "select statusId from session where session=\"%s\"", oldsession)>0);
if(!mysqlResultAffirmative(query))
	{
	free(query);
	printf("],");
	return;
	}
free(query);

// retrieve all saved status under this session and return
if(asprintf(&query, "select * from session where session='%s'", oldsession)<0) errabort("bae");
MYSQL_RES *sr = mysqlGetResult(query);
assert(sr!=NULL);
while((row=mysql_fetch_row(sr)) != NULL)
	{
	printf("[%s, \"%s\", '%s',", 
		row[sid_statusid],
		row[sid_statusnote],
		dspStat2readable(row[sid_dspStat], row[sid_juxType])
		);
	// number of native tracks (hmtk plus decor)
	struct track *sl=NULL, *tt;
	if(row[sid_bedgraph_n]!=NULL) parseTrackParam(row[sid_bedgraph_n], FT_bedgraph_n, &sl);
	if(row[sid_bedgraph_c]!=NULL) parseTrackParam(row[sid_bedgraph_c], FT_bedgraph_c, &sl);
	if(row[sid_bigwighmtk_n]!=NULL) parseTrackParam(row[sid_bigwighmtk_n], FT_bigwighmtk_n, &sl);
	if(row[sid_bigwighmtk_c]!=NULL) parseTrackParam(row[sid_bigwighmtk_c], FT_bigwighmtk_c, &sl);
	if(row[sid_cat_n]!=NULL) parseTrackParam(row[sid_cat_n], FT_cat_n, &sl);
	if(row[sid_cat_c]!=NULL) parseTrackParam(row[sid_cat_c], FT_cat_c, &sl);
	if(row[sid_bed_n]!=NULL) parseTrackParam(row[sid_bed_n], FT_bed_n, &sl);
	if(row[sid_bed_c]!=NULL) parseTrackParam(row[sid_bed_c], FT_bed_c, &sl);
	if(row[sid_qdecor_n]!=NULL) parseTrackParam(row[sid_qdecor_n], FT_qdecor_n, &sl);
	if(row[sid_sam_n]!=NULL) parseTrackParam(row[sid_sam_n], FT_sam_n, &sl);
	if(row[sid_sam_c]!=NULL) parseTrackParam(row[sid_sam_c], FT_sam_c, &sl);
	if(row[sid_lr_n]!=NULL) parseTrackParam(row[sid_lr_n], FT_lr_n, &sl);
	if(row[sid_lr_c]!=NULL) parseTrackParam(row[sid_lr_c],FT_lr_c, &sl);
	if(row[sid_bam_n]!=NULL) {
	fprintf(stderr, "BAMN: %s\n", row[sid_bam_n]);
	parseTrackParam(row[sid_bam_n], FT_bam_n, &sl);
	fputs("dddddone\n", stderr);
	}
	if(row[sid_bam_c]!=NULL) parseTrackParam(row[sid_bam_c], FT_bam_c, &sl);
	int tknum=0;
	for(tt=sl; tt!=NULL; tt=tt->next) tknum++;

	printf("%d,'%s'],", tknum, row[sid_juxNote]);
	}
mysql_free_result(sr);
free(query);
printf("],");
}

void repeatbrowser_parseExpTrack(char *tknamelst, int ft, struct track **sl)
{
struct track *t;
char delim[]=",";
char *tok=strtok(tknamelst,delim);
while(tok!=NULL)
	{
	t=malloc(sizeof(struct track));
	t->name=t->label=strdup(tok);
	assert(asprintf(&(t->urlpath),"/srv/epgg/data/data/repeatbrowser/hg19_repeatmasker/genome_bedgraph/%s.gz",tok)>0);
	t->ft=ft;
	assert((tok=strtok(NULL,delim))!=NULL);
	assert((t->mode=strMayPositiveInt(tok))!=-1);
	t->next=*sl;
	*sl=t;
	tok=strtok(NULL,delim);
	}
}

struct tabixHandle *repeatbrowser_parseTrack(char *tknamelst, int type)
{
/* type:
	1 - read density on genome
	2 - read density on consensus, iteres all
	3 - read density on consensus, iteres unique
	4 - repeat genome copy density on a consensus
all of them are bedgraph
*/

struct tabixHandle *sl=NULL, *t;
char *path;
if(type==4)
	{
	path="/srv/epgg/data/data/repeatbrowser/hg19_repeatmasker/repeat_genome_density.gz";
	t=malloc(sizeof(struct tabixHandle));
	if((t->fin=tabixOpen(path))==NULL)
		{
		fprintf(stderr, "error with file %s\n", path);
		exit(0);
		}
	t->next=NULL;
	return t;
	}
	
char delim[]=",";
char *tok=strtok(tknamelst,delim);
while(tok!=NULL)
	{
	if(type==1)
		assert(asprintf(&path,"/srv/epgg/data/data/repeatbrowser/hg19_repeatmasker/genome_bedgraph/%s.gz",tok)>0);
	else if(type==2)
		assert(asprintf(&path,"/srv/epgg/data/data/repeatbrowser/hg19_repeatmasker/iteres_all/%s.gz",tok)>0);
	else
		assert(asprintf(&path,"/srv/epgg/data/data/repeatbrowser/hg19_repeatmasker/iteres_uniq/%s.gz",tok)>0);
	t=malloc(sizeof(struct tabixHandle));
	if((t->fin=tabixOpen(path))==NULL)
		{
		fprintf(stderr, "error with file '%s'\n", path);
		exit(0);
		}
	t->name=strdup(tok);
	t->next=sl;
	sl=t;
	free(path);
	tok=strtok(NULL,delim);
	}
if(sl==NULL)
	{
	fprintf(stderr, "%s wrong with type %d\n", __FUNCTION__, type);
	exit(0);
	}
return sl;
}


struct beditem2 *getRepeats4subfam()
{
/* args:
chrlst: chr1,len1,chr2,len2,...

!!! bbiDir must be ready!!!
which is from the sukn database that the rpbr depends on
*/
struct tnode *chrsl=NULL, *tt;
char delim[]=",\t";
char *tok=strtok(cgiString("chrlst"), delim);
while(tok!=NULL)
	{
	tt=malloc(sizeof(struct tnode));
	tt->t1=tok;
	assert((tok=strtok(NULL,delim))!=NULL);
	assert(asprintf(&(tt->t2), "%s:0-%s", tt->t1, tok)>0);
	tt->next=chrsl;
	chrsl=tt;
	tok=strtok(NULL,delim);
	}
char *file;
assert(asprintf(&file, "/srv/epgg/data/data/subtleKnife/hg19/%s%s%s.gz", cgiString("rpbr_class"), cgiString("rpbr_family"), cgiString("rpbr_subfam"))>0);
tabix_t *fin=tabixOpen(file);
assert(fin!=NULL);
struct beditem2 *sl=NULL, *t;
struct beditem *sl2, *it;
for(tt=chrsl; tt!=NULL; tt=tt->next)
	{
	sl2=tabixQuery_bed(fin, tt->t2);
	for(it=sl2; it!=NULL; it=it->next)
		{
		t=malloc(sizeof(struct beditem2));
		t->chrom=strdup(tt->t1);
		t->start=it->start;
		t->stop=it->stop;
		strtok(it->rest, delim); // throw name
		assert((tok=strtok(NULL,delim))!=NULL);
		t->rest=strdup(tok); // id
		assert((tok=strtok(NULL,delim))!=NULL);
		t->strand=tok[0];
		t->next=sl;
		sl=t;
		}
	slFree(sl2);
	}
slFree(chrsl);
ti_close(fin);
return sl;
}


char *gsv_dsp_makestring(struct displayedRegion *dsp)
{
char *old = "", *new;
boolean atfirst=TRUE;
struct region *r;
for(r=dsp->head; r!=NULL; r=r->next)
	{
	assert(asprintf(&new, "%s%s,%d,%d,%d,%d,%d,%d,%d,%c,%d,", 
		old,
		r->name,
		r->chromIdx,
		r->bstart, r->bstop,
		r->flank5start, r->flank5stop,
		r->flank3start, r->flank3stop,
		r->strand,
		r->flag?1:0)>0);
	if(atfirst) atfirst=FALSE;
	else free(old);
	old = new;
	}
return old;
}

struct region *gsv_duplicateregion(struct region *r1)
{
struct region *r3=malloc(sizeof(struct region));
r3->chromIdx=r1->chromIdx;
r3->name=r1->name;
r3->strand=r1->strand;
r3->flag=r1->flag;
r3->bstart=r3->dstart=r1->bstart;
r3->bstop=r3->dstop=r1->bstop;
r3->flank5start=r1->flank5start;
r3->flank3start=r1->flank3start;
r3->flank5stop=r1->flank5stop;
r3->flank3stop=r1->flank3stop;
return r3;
}

struct geneParam *initGeneParam()
{
struct geneParam *p=malloc(sizeof(struct geneParam));
p->query_table=NULL;
p->query_id=-1;
p->query_str=NULL;
p->use_chromid=TRUE;
p->use_exactmatch=TRUE;
p->whichgenepart="genebody";
p->origintype=NULL;
p->flank5=0;
p->flank3=0;
return p;
}



int main()
{


clock_t cpuTimeStart, cpuTimeTemp;
if(CHECKCPUTIME) cpuTimeStart = clock();

if(SQUAWK) fputs(">>>>>\n", Squawk);


{
	/* initiate cgi hash 
	this program only deals with "get", not "post"
	all parameters passed via cgi must be escaped and be decoded here
	** no empty value allowed, e.g. k1=v1&k2=&k3=v3&...
	as =& will be used to chop up things...
	TODO use singly linked list instead of gnu hash to get rid of entry limit
	*/
	char *tmp=getenv("QUERY_STRING");
	if(tmp==NULL)
		{
		if(SQUAWK) fputs("cgi got no param\n", stderr);
		return 1;
		}
	if(strstr(tmp, "NODECODE")!=tmp)
		{
		int len=strlen(tmp);
		// decode
		char *tmp2 = malloc(sizeof(char) * (len+1));
		strDecode(tmp, tmp2, len);
		tmp=tmp2;
		}
	char delim[]="=&";
	struct tnode *t;
	char *tok=strtok(tmp,delim);
	assert(tok!=NULL);
	while(tok!=NULL)
		{
		t=malloc(sizeof(struct tnode));
		t->t1=strdup(tok); // name
		tok=strtok(NULL,delim); // value
		if(tok==NULL)
			{
			fprintf(stderr,"!! param '%s' has no value\n", t->t1);
			return 1;
			}
		t->t2=strdup(tok); // value
		tok=strtok(NULL,delim);
		t->next=cgiparamsl;
		cgiparamsl=t;
		}
	if(SQUAWK) fputs("cgi param parsed\n", stderr);
	if(0)
		{
		for(t=cgiparamsl; t!=NULL; t=t->next)
			fprintf(stderr, "%s => %s\n", t->t1, t->t2);
		}
}

/* things that do not require db connection
*/

if(cgiVarExists("loaddatahub"))
	{
	/* simply read a datahub file
	*/
	printf("Content-Type:text/plain\n\n");
	printf("%s__SEPARATOR__",cgiString("horcrux"));
	srand(time(0));
	int r = rand();
	char *descriptorfile;
	assert(asprintf(&descriptorfile, "%s/%d.hub", trashDir, r)>0);
	char *command;
	assert(asprintf(&command, "wget %s -O %s -q", cgiString("url"), descriptorfile)>0);
	if(system(command) == -1)
		{
		printf("ERROR:cannot run wget");
		return 1;
		}
	// tell if the file is empty
	FILE *fin = fopen(descriptorfile, "rb");
	if(fin == NULL)
		{
		printf("ERROR:cannot open temp file");
		return 1;
		}
	fseek(fin, 0, SEEK_END);
	if(ftell(fin) == 0)
		{
		printf("ERROR:error with descriptor file (no content or invalid URL)");
		return 1;
		}
	rewind(fin);
	char *line=malloc(1); size_t s=1;
	while(getline(&line, &s, fin) != -1)
		{
		// skip comments
		if(line[0]=='#' || line[0]=='\n') continue;
		printf("%s",line);
		}
	fclose(fin);
	unlink(descriptorfile);
	done();
	return 1;
	}

printf("Content-Type:application/json\n\n{");


if(cgiVarExists("refreshcusttkcache"))
	{
	// just one track
	char delim[]=",";
	char *tok;
	if(cgiVarExists("filelst"))
		{
		tok=strtok(cgiString("filelst"),delim);
		while(tok!=NULL)
			{
			if(unlink(tok)==-1)
				{
				fprintf(stderr, "cannot unlink %s\n", tok);
				}
			tok=strtok(NULL,delim);
			}
		}
	if(cgiVarExists("dirlst"))
		{
		tok=strtok(cgiString("dirlst"),delim);
		while(tok!=NULL)
			{
			if(rmdir(tok)==-1)
				{
				fprintf(stderr,"cannot rmdir %s\n", tok);
				}
			tok=strtok(NULL,delim);
			}
		}

	char *url=cgiString("url");
	if(url==NULL)
		{
		printf("'error':'missing url'}");
		return 1;
		}
	int ft=cgiInt("ft");
	char *chrom=cgiString("chrom");
	int start=cgiInt("start");
	double *x=malloc(sizeof(double));
	if(ft==FT_bigwighmtk_c)
		{
		bigwigQuery(url,chrom,start,start+1,1,x);
		// TODO how to detect invalid bigwig file?
		}
	else if(ft==FT_bam_c)
		{
		samfile_t *fp=samopen(url,"rb",0);
		if(fp==0)
			{
			printf("'error':'failed to read bam file',");
			}
		else 
			{
			if(bam_parseIndex(url,ft)==NULL)
				{
				printf("'error':'failed to refresh cache',");
				}
			samclose(fp);
			}
		}
	else if(ft==FT_bedgraph_c)
		{
		tabix_t *fin=tabixOpen(url);
		if(fin==NULL)
			{
			printf("'error':'failed to refresh cache',");
			}
		else
			{
			ti_close(fin);
			}
		}
	else 
		{
		tabix_t *fin=tabixOpen(url);
		if(fin==NULL) printf("'error':'failed to refresh cache',");
		else ti_close(fin);
		}
	printf("}");
	done();
	return 1;
	}


/* keys to spit back */
char *horcrux;
if(cgiVarExists("horcrux"))
	{
	horcrux=cgiString("horcrux");
	printf("'horcrux':'%s',", horcrux);
	}
else
	horcrux="fake";

// try to put this into specific context where hengeview happens??
if(cgiVarExists("hengekey"))
	printf("'hengekey':'%s',",cgiString("hengekey"));


/** following things all require db connection **/

if(cgiVarExists("load_dblst"))
    {
	// sukn only
    if(!mysqlConnect(CentralDB))
		{
		printf("'mysqldberror':1}");
		return 0;
		}
    MYSQL_RES *sr = mysqlGetResult("select dbName,dbLabel,organism,isMajor,fontcolor from dbTable");
	assert(sr!=NULL);
    char **row;
	printf("'dblst':[");
    while((row=mysql_fetch_row(sr)) != NULL)
		printf("{'dbname':'%s','label':'%s','organism':'%s','ismajor':%s,'fontcolor':'%s'},", row[0], row[1], row[2],row[3],row[4]);
	printf("],");
    mysql_free_result(sr);
	mysql_close(conn);
	if(cgiVarExists("load_dblst_quit"))
		{
		printf("}");
		done();
		return 1;
		}
    }



if(cgiVarExists("repeatbrowser"))
	{
	/**** __rb__ repeatbrowser
	exit when done
	***/
	if(cgiVarExists("rpbrDbname"))
		{
		if(!mysqlConnect(cgiString("rpbrDbname")))
			{
			printf("'mysqldberror':1}");
			return 0;
			}
		}
	if(cgiVarExists("viewkey"))
		printf("'key':'%s',", cgiString("viewkey"));

	if(cgiVarExists("getfileinfo"))
		{
		char *query;
		char *file=cgiString("getfileinfo");
		assert(asprintf(&query, "select info from tk2info where name='%s'", file)>0);
		printf("'text':'%s','file':'%s'}", mysqlGetOnestring(query),file);
		free(query);
		done();
		return 1;
		}
	if(cgiVarExists("geoaccidlst"))
		{
		/* to fetch ratio data for a group of geo/tks to display in the bigmap
		*/
		char *query, **row;
		char delim[]=",";
		char *tok=strtok(cgiString("geoaccidlst"),delim);
		while(tok!=NULL)
			{
			assert(asprintf(&query, "select subfam_id,ratio_1,ratio_2 from ratiomatrix where geo_id=%s", tok)>0);
			MYSQL_RES *sr=mysqlGetResult(query);
			free(query);
			printf("%s:{", tok);
			while((row=mysql_fetch_row(sr))!=NULL)
				printf("%s:[%s,%s],", row[0],row[1],row[2]);
			printf("},");
			mysql_free_result(sr);
			tok=strtok(NULL,delim);
			}
		printf("}");
		done();
		return 1;
		}
	if(cgiVarExists("getsubfamcopiesonly"))
		{
		/* to fetch genome copies of a subfam for display
		*/
		struct beditem2 *sl=getRepeats4subfam();
		printf("'genomecopies':[");
		struct beditem2 *t;
		for(t=sl; t!=NULL; t=t->next)
			{
			printf("['%s',%d,%d,'%c'],", t->chrom, t->start, t->stop, t->strand);
			}
		printf("]}");
		slFree(sl);
		done();
		return 1;
		}
	if(cgiVarExists("getsubfamcopieswithtk"))
		{
		struct beditem2 *rsl=getRepeats4subfam();
		int itemtotalcount=0;
		struct beditem2 *t;
		for(t=rsl; t!=NULL; t=t->next) itemtotalcount++;

		/* see which tracks to query
			tabix handles are opened in child but **not parent**
			see below
		struct tabixHandle *treatSl=NULL, *inputSl=NULL, *tt;
		assert(asprintf(&query,"select treatment,control from geo2xx where geo='%s'",geoAcc)>0);
		sr=mysqlGetResult(query);
		row=mysql_fetch_row(sr);
		if(row!=NULL)
			{
			if(row[0]==NULL)
				{
				fprintf(stderr, "geo %s gets no treatment track!\n", geoAcc);
				exit(0);
				}
			treatSl=repeatbrowser_parseTrack(row[0],1);
			if(row[1]!=NULL)
				inputSl=repeatbrowser_parseTrack(row[1],1);
			}
		mysql_free_result(sr);
		*/

		char *geoAcc=cgiString("geo");

		char *query, **row;

		struct tabixHandle *tt;
		assert(asprintf(&query,"select treatment,control from geo2info where geo='%s'",geoAcc)>0);
		MYSQL_RES *sr=mysqlGetResult(query);
		row=mysql_fetch_row(sr);
		char *treatTkstr=strdup(row[0]);
		char *inputTkstr=NULL;
		if(row[1]!=NULL) inputTkstr=strdup(row[1]);
		mysql_free_result(sr);

		/** filters will be added here
		to preserve those repeats that passes filter
		**/

		printf("'genomecopies':[");

		/* now tracks are ready, fetch data and print json
		single-process
		not_in_use
		for(t=itemsl; t!=NULL; t=t->next)
			{
			printf("['%s',%d,%d,'%c',%s,[", t->chrom, t->start, t->stop, t->strand, t->rest);
			// compute score for each treat track, return as a nested array
			for(tt=treatSl; tt!=NULL; tt=tt->next)
				printf("%f,",tabixQuery_bedgraph_singlepoint(tt->fin, t->chrom, t->start, t->stop));
			printf("]");
			if(inputSl!=NULL)
				{
				printf(",[");
				for(tt=inputSl; tt!=NULL; tt=tt->next)
					printf("%f,", tabixQuery_bedgraph_singlepoint(tt->fin, t->chrom, t->start, t->stop));
				printf("]");
				}
			printf("],");
			}
		*/

		/** _fork_ **/
		double itemperprocess=500;
		int processcount=(int)ceil(itemtotalcount/itemperprocess);
		char *depositfiles[processcount];
		int pidarr[processcount];
		struct beditem2 *itemPlist[processcount];
		struct beditem2 *ttt;

		// initialize stuff for forking
		int i,j;
		srand(time(0));
		int basenum=rand();
		t=rsl;
		for(i=0; i<processcount; i++)
			{
			assert(asprintf(&(depositfiles[i]), "%s/%d.%d", trashDir, basenum, i)>0);
			pidarr[i]=-1;
			itemPlist[i]=NULL;
			for(j=0; j<itemperprocess; j++)
				{
				if(t==NULL) break;
				ttt=malloc(sizeof(struct beditem2));
				ttt->chrom=t->chrom;
				ttt->start=t->start;
				ttt->stop=t->stop;
				ttt->strand=t->strand;
				ttt->rest=t->rest;
				ttt->next=itemPlist[i];
				itemPlist[i]=ttt;
				t=t->next;
				}
			}

		boolean notfinished = TRUE;
		int processRunningNum = 0;
		while(notfinished)
			{
			notfinished = FALSE;
			for(i=0; i<processcount; i++)
				{
				if(pidarr[i] == -1)
					{
					// this set is not processed yet
					if(processRunningNum == MaxProcessNum)
						{
						notfinished = TRUE;
						continue;
						}
					// fork for this set
					pid_t p = fork();
					if(p == 0)
						{
						// inside child process
						FILE *fout=fopen(depositfiles[i],"w");
						assert(fout!=NULL);

						/* oh no...
						tabix handles are opened in child, but not parent
						looks like when opened in parent, it will crash in the case of many processes
						maybe each handle will write to the same temp text file?
						*/
						struct tabixHandle *treatSl=repeatbrowser_parseTrack(treatTkstr,1);
						struct tabixHandle *inputSl=NULL;
						if(inputTkstr!=NULL)
							inputSl=repeatbrowser_parseTrack(inputTkstr,1);
						for(t=itemPlist[i]; t!=NULL; t=t->next)
							{
							fprintf(fout, "%s %d %d %c %s ", t->chrom, t->start, t->stop, t->strand, t->rest);
							// compute score for each treat track, return as a nested array
							for(tt=treatSl; tt!=NULL; tt=tt->next)
								fprintf(fout,"%f,",tabixQuery_bedgraph_singlepoint(tt->fin, t->chrom, t->start, t->stop));
							if(inputSl!=NULL)
								{
								fputs(" ", fout);
								for(tt=inputSl; tt!=NULL; tt=tt->next)
									fprintf(fout,"%f,", tabixQuery_bedgraph_singlepoint(tt->fin, t->chrom, t->start, t->stop));
								}
							fputs("\n", fout);
							}
						fclose(fout);
						_exit(0); // exit without flushing buffer
						}
					else if(p < 0)
						{
						fprintf(stderr, "error spawning child process at processidx %d\n", i);
						exit(0);
						}
					else
						{
						pidarr[i] = p;
						processRunningNum++;
						}
					notfinished = TRUE;
					}
				else if(pidarr[i] > 0)
					{
					// child has been forked
					int status;
					waitpid(pidarr[i], &status, 0);
					if(status == 0)
						{
						// child process has finished successfully
						pidarr[i] = 0;
						FILE *fin=fopen(depositfiles[i],"r");
						assert(fin!=NULL);
						char *line=malloc(1);
						size_t s=1;
						char delim[]="\n";
						while(getline(&line, &s, fin)!=-1)
							{
							printf("'%s',", strtok(line, delim));
							}
						fclose(fin);
						unlink(depositfiles[i]);
						//free(tk->tmpfile);
						}
					else
						{
						fprintf(stderr, "unknown status for child processidx %d %s\n", i, depositfiles[i]);
						exit(0);
						}
					processRunningNum--;
					}
				}
			}
		/** _fork_ ends ***/
		printf("]}");
		done();
		return 1;
		}
	if(cgiVarExists("getconsensuswig"))
		{
		if(cgiVarExists("viewkey")) printf("'key':'%s',", cgiString("viewkey"));
		char *subfam=cgiString("subfam");
		char *query, **row;

		assert(asprintf(&query,"select consensuslength from subfam2id where subfam='%s'",subfam)>0);
		int slen=mysqlGetPositiveInteger(query);
		if(slen==-1)
			{
			fprintf(stderr, "wrong %s\n", query);
			exit(0);
			}
		free(query);
		assert(asprintf(&query,"select treatment,control from geo2info where geo='%s'",cgiString("geo"))>0);
		MYSQL_RES *sr=mysqlGetResult(query);
		free(query);
		row=mysql_fetch_row(sr);
		struct tabixHandle *sl, *t;
		if(row==NULL && row[0]==NULL)
			{
			fprintf(stderr, "geo got no track %s\n", cgiString("geo"));
			exit(0);
			}
		sl=repeatbrowser_parseTrack(strdup(row[0]),2);
		printf("'treatment_all':[");
		double data[slen];
		int i=0;
		for(t=sl; t!=NULL; t=t->next)
			{
			printf("['%s',[", t->name);
			//fprintf(stderr, "treat %s\n", t->name);
			for(i=0; i<slen; i++) data[i]=0;
			tabixQuery_bedgraph(t->fin, subfam, 0, slen, slen, &data[0]);
			ti_close(t->fin);
			for(i=0; i<slen; i++) printf("%f,",data[i]);
			printf("]],");
			}
		slFree(sl);
		sl=repeatbrowser_parseTrack(row[0],3);
		printf("],'treatment_unique':[");
		for(t=sl; t!=NULL; t=t->next)
			{
			printf("['%s',[", t->name);
			for(i=0; i<slen; i++) data[i]=0;
			tabixQuery_bedgraph(t->fin, subfam, 0, slen, slen, &data[0]);
			ti_close(t->fin);
			for(i=0; i<slen; i++) printf("%f,",data[i]);
			printf("]],");
			}
		slFree(sl);

		printf("],");
		if(row[1]!=NULL)
			{
			sl=repeatbrowser_parseTrack(strdup(row[1]),2);
			printf("'input_all':[");
			for(t=sl; t!=NULL; t=t->next)
				{
				printf("['%s',[", t->name);
				//fprintf(stderr, "input %s\n", t->name);
				for(i=0; i<slen; i++) data[i]=0;
				tabixQuery_bedgraph(t->fin, subfam, 0, slen, slen, &data[0]);
				ti_close(t->fin);
				for(i=0; i<slen; i++) printf("%f,",data[i]);
				printf("]],");
				}
			slFree(sl);
			sl=repeatbrowser_parseTrack(row[1],3);
			printf("],'input_unique':[");
			for(t=sl; t!=NULL; t=t->next)
				{
				printf("['%s',[", t->name);
				for(i=0; i<slen; i++) data[i]=0;
				tabixQuery_bedgraph(t->fin, subfam, 0, slen, slen, &data[0]);
				ti_close(t->fin);
				for(i=0; i<slen; i++) printf("%f,",data[i]);
				printf("]],");
				}
			printf("],");
			slFree(sl);
			}
		mysql_free_result(sr);

		// the genome copy density
		sl=repeatbrowser_parseTrack(NULL, 4);
		printf("'density':[");
		for(i=0; i<slen; i++) data[i]=0;
		tabixQuery_bedgraph(sl->fin, subfam, 0, slen, slen, &data[0]);
		ti_close(sl->fin);
		for(i=0; i<slen; i++) printf("%f,", data[i]);
		printf("],");

		// consensus seq if available
		assert(asprintf(&query, "select seq from subfam2seq where name='%s'", subfam)>0);
		char *seq=mysqlGetOnestring(query);
		free(query);
		if(seq!=NULL)
			printf("'consensusseq':'%s',", seq);

		printf("}");
		done();
		return 1;
		}
	if(cgiVarExists("makehub"))
		{
		/* a splinter of ??'s soul drifted into sukn
		hard-coded url, file path for hg19
		*/
		srand(time(0));
		int randnum=rand();
		char *file;
		assert(asprintf(&file, "%s/repeathub_%d", WWWT, randnum)>0);
		FILE *fout=fopen(file, "w");
		if(fout==NULL)
			{
			printf("'error':1}");
			return 1;
			}
		char delim[]=";";
		char *tok;
		if(cgiVarExists("assaytk"))
			{
			// assume all are bedgraph
			tok=strtok(cgiString("assaytk"),delim);
			while(tok!=NULL)
				{
				char *tkname=tok;
				char *label=strtok(NULL,delim);
				if(label==NULL) label="no name";
				char *style=strtok(NULL,delim);
				fprintf(fout, "bedgraph\t%s/genome_bedgraph/%s.gz\t%s\tshow\tn/a", rpbr_file_url, tkname, label);
				if(style!=NULL)
					fprintf(fout, "\t%s", style);
				fputs("\n", fout);
				tok=strtok(NULL,delim);
				}
			}
		if(cgiVarExists("subfambed"))
			{
			fprintf(fout, "bed\t%s/subfam_bed/%s.gz\t%s\tfull\tn/a\n", rpbr_file_url, cgiString("subfambed"), cgiString("subfam"));
			}
		fclose(fout);
		printf("'huburl':'http://epigenomegateway.wustl.edu/browser/t/repeathub_%d'}", randnum);
		done();
		return 1;
		}
	}






/* kegg rest api */
if(cgiVarExists("listkeggpathway"))
	{
	srand(time(0));
	int randnum=rand();
	char *outfile;
	assert(asprintf(&outfile,"%s/%d",trashDir,randnum)>0);
	char *command;
	assert(asprintf(&command,"wget http://rest.kegg.jp/list/pathway/%s -O %s",cgiString("speciescode"),outfile)>0);
	if(system(command)==-1)
		{
		printf("'error':1}");
		return 1;
		}
	char delim[]="\t\n";
	char *tok1,*tok2;
	FILE *fin=fopen(outfile,"r");
	if(fin==NULL)
		{
		printf("'error':1}");
		return 1;
		}
	char *line=malloc(1);
	size_t s=1;
	printf("'lst':[");
	while(getline(&line,&s,fin)!=-1)
		{
		tok1=strtok(line,delim);
		if(tok1==NULL) continue;
		tok2=strtok(NULL,delim);
		if(tok2==NULL) continue;
		printf("['%s',\"%s\"],",tok1,tok2);
		}
	printf("]}");
	fclose(fin);
	unlink(outfile);
	return 1;
	}	




char *dbName=cgiString("dbName");
if(dbName==NULL)
	{
	if(SQUAWK) fputs("No dbName, not connecting db\n",stderr);
	}
else
	{
	if(SQUAWK) fprintf(stderr, "Hooked to %s\n",dbName);
	if(!mysqlConnect(dbName))
		{
		printf("'mysqldberror':\"%s\"}",dbName);
		return 0;
		}
	}


/* validate session from url parameter
exit when done
 */
if(cgiVarExists("validateurlparam_session"))
    {
    char *query;
	char *oldsession=cgiString("session");
    assert(asprintf(&query, "select statusId from session where session='%s'", oldsession)>0);
	if(mysqlResultAffirmative(query))
        {
		sessionstatus2json(oldsession);
		}
	else
		{
		printf("'error':1");
		}
	printf("}");
	done();
    return 1;
    }
if(cgiVarExists("validateurlparam_sessionstatus"))
    {
    char *query;
	char *oldsession=cgiString("session");
	char *oldstatus=cgiString("statusId");
    assert(asprintf(&query, "select statusId from session where session='%s' and statusId=%s", oldsession, oldstatus)>0);
	if(mysqlResultAffirmative(query))
		printf("'good':1}");
	else
		printf("'error':1}");
	done();
    return 1;
    }


if(cgiVarExists("session"))
    session = cgiString("session");
if(cgiVarExists("statusId"))
    statusId = cgiInt("statusId");




if(cgiVarExists("scfdruntimesync"))
	{
	char *query;
	assert(asprintf(&query, "select names from scaffoldRuntime where session='%s' and statusId=%s", cgiString("session"), cgiString("status"))>0);
	char *re=mysqlGetOnestring(query);
	if(re==NULL)
		{
		printf("'error':1}");
		}
	else
		{
		char delim[]=",";
		char *tok=strtok(re,delim);
		printf("'lst':[");
		while(tok!=NULL)
			{
			printf("'%s',", tok);
			strtok(NULL,delim);
			tok=strtok(NULL,delim);
			}
		printf("]}");
		}
	done();
	return 1;
	}


if(cgiVarExists("getgenebykeggpathway"))
	{
	/* need db connection
	use species code (hsa) and pathway id (hsa00010) to query list of kegg genes
	then for each gene convert to refgene name and return
	*/
	srand(time(0));
	char *filename;
	assert(asprintf(&filename, "%s/%d", trashDir, rand())>0);
	char *comm;
	assert(asprintf(&comm, "wget http://rest.kegg.jp/link/%s/%s -O %s", cgiString("speciescode"),cgiString("pathway"),filename)>0);
	if(system(comm)==-1)
		{
		printf("'error':1}");
		return 1;
		}
	char *query;
	char delim[]=":\n";
	char *line=malloc(1);
	size_t s=1;
	FILE *fin=fopen(filename,"r");
	if(fin==NULL)
		{
		printf("'error':1}");
		return 1;
		}
	printf("'lststring':'");
	while(getline(&line,&s,fin)!=-1)
		{
		strtok(line,delim);
		strtok(NULL,delim);
		char *tok=strtok(NULL,delim);
		if(tok==NULL) continue;
		assert(asprintf(&query, "select refgene from kegg2refgene where kegg='%s'", tok)>0);
		char *re=mysqlGetOnestring(query);
		free(query);
		if(re!=NULL) printf("%s\\n", re);
		}
	printf("'}");
	fclose(fin);
	//unlink(filename);
	done();
	return 1;
	}




assert((bbiDir=mysqlGetOnestring("select bbiPath from config"))!=NULL);
seqPath=mysqlGetOnestring("select seqPath from config");


/** load and return info on a genome, so that js will build the object
TODO none-genome related info need to be separated out
exit when done
**/
if(cgiVarExists("loadgenome"))
	{
	if(SQUAWK) fputs("fetching genome info...\n", stderr);
	printf("'dbname':'%s',", dbName);
	if(seqPath==NULL)
		printf("'noblastdb':true,");

    char *query, **row, *re;
	MYSQL_RES *sr=NULL;

	/* timecourse */
	if(mysqlResultAffirmative("show tables like \"yearlyMonthlyLength\""))
		{
		printf("'yearlyMonthlyLength':[");
		sr=mysqlGetResult("select * from yearlyMonthlyLength");
		while((row=mysql_fetch_row(sr))!=NULL)
			printf("[%s,%s,%s],",row[0],row[1],row[2]);
		printf("],");
		mysql_free_result(sr);
		}
	
    // default displayed position
	sr=mysqlGetResult("select defaultTracks,defaultMdcategory,defaultPosition,hasGene,allowJuxtaposition,defaultGenelist,defaultDataset,defaultScaffold,defaultCustomtracks,defaultDecor,runmode,initmatplot from config");
	assert((row=mysql_fetch_row(sr))!=NULL);
    printf(
		"'defaultHmtk':'%s',"
		"'defaultMdcategory':'%s',"
		"'defaultPosition':'%s',"
		"'hasGene':%s,"
		"'allowJuxtaposition':%s,"
		"'defaultGenelist':'%s',"
		"'defaultScaffold':'%s',"
		"'defaultCustomtracks':'%s',"
		"'defaultDecor':'%s',"
		"'runmode':%s,"
		"'initmatplot':%s,",
		row[0]==NULL?"":row[0],
		row[1], row[2],row[3],row[4],row[5],row[7],row[8],
		row[9]==NULL?"":row[9],
		row[10],
		row[11]
	);
	char *defaultDataset=strdup(row[6]);
	mysql_free_result(sr);

	re=mysqlGetOnestring("select keggSpeciesCode from config");
	if(re!=NULL)
		printf("'keggSpeciesCode':'%s',",re);


	// information on all datasets
	// make a list of dataset names
	if(!cgiVarExists("no_dataset_info"))
		{
		sr = mysqlGetResult("select tablename from dataset");
		assert(sr!=NULL);
		struct tnode *sl=NULL, *tt;
		while((row=mysql_fetch_row(sr)) != NULL)
			{
			assert(row[0] != NULL);
			tt = malloc(sizeof(struct tnode));
			tt->t1=strdup(row[0]);
			tt->next = sl;
			sl = tt;
			}
		mysql_free_result(sr);
		slReverse(&sl);
		// get # of tracks for each dataset
		for(tt=sl; tt!=NULL; tt=tt->next)
			{
			if(asprintf(&query, "select count(tkname) from %s", tt->t1)<0) errabort("bae");
			assert((tt->num=mysqlGetPositiveInteger(query))!=-1);
			free(query);
			}
		// return details for each dataset
		printf("'datasetInfo':[");
		for(tt=sl; tt!=NULL; tt=tt->next)
			{
			if(tt->num <= 0) continue;
			if(asprintf(&query, "select * from dataset where tablename='%s'", tt->t1)<0) errabort("bae");
			sr = mysqlGetResult(query);
			free(query);
			if(sr==NULL) continue;
			row = mysql_fetch_row(sr);
			// name, logo, label, url, desc, tk #
			printf("['%s','%s','%s','%s','%s', %d],", tt->t1, row[1], row[2], row[3], row[4], tt->num );
			mysql_free_result(sr);
			}
		printf("],");
		slFree(sl);
		}

    /* which datasets to load
	will only return info on selected datasets
	- if 'hmtk4dataset' is present, user has supplied hmtk tracks from urlparam,
	  and need to fetch data sets spanned by these tracks
	- else, only use first dataset
	*/
	struct slNode *dataset2show=NULL;
	if(cgiVarExists("datasetpreload"))
		{
		// TODO need to detect wrong dataset names
		char delim[]=",";
		char *tok=strtok(cgiString("datasetpreload"),delim);
		while(tok!=NULL)
			{
			struct slNode *t=malloc(sizeof(struct slNode));
			t->name=tok;
			t->next=dataset2show;
			dataset2show=t;
			tok=strtok(NULL,delim);
			}
		}
	else if(cgiVarExists("hmtk4dataset"))
		{
		struct slNode *tksl=NULL, *tk; // list of hmtk
		char delim[]=",";
		char *tok=strtok(cgiString("hmtk4dataset"),delim);
		if(tok!=NULL)
			{
			while(tok!=NULL)
				{
				tk=malloc(sizeof(struct slNode));
				tk->name=tok;
				tk->next=tksl;
				tksl=tk;
				tok=strtok(NULL,delim);
				}
			dataset2show=findDatasetByHmtksl(tksl);
			slFree(tksl);
			}
		}
	if(dataset2show==NULL)
		{
		// use default
		char delim[]=",";
		char *tok=strtok(defaultDataset, delim);
		while(tok!=NULL)
			{
			struct slNode *tt = malloc(sizeof(struct slNode));
			tt->name=tok;
			tt->next=dataset2show;
			dataset2show=tt;
			tok=strtok(NULL,delim);
			}
		}
	printJson_hmtk4datasets(dataset2show);
	
		{
		printf("'dataset2show':[");
		struct slNode *t;
		for(t=dataset2show; t!=NULL; t=t->next)
			printf("'%s',", t->name);
		printf("],");
		}

    // track default parameter
    printf("'trackDefaultparam':{");
    sr = mysqlGetResult("select * from track2Style");
    while((row=mysql_fetch_row(sr)) != NULL)
		printf("'%s':{%s},", row[0], row[1]);
    printf("},");
    mysql_free_result(sr);

	// track 2 regions
	printf("'track2Regions':{");
	sr=mysqlGetResult("select * from track2Regions");
	while((row=mysql_fetch_row(sr))!=NULL)
		printf("'%s':['%s','%s'],",row[0],row[1],row[2]);
	printf("},");
	mysql_free_result(sr);

    // decor tk
    sr = mysqlGetResult("select * from gfGrouping");
    printf("'gfGrouping':[");
    while((row=mysql_fetch_row(sr)) != NULL)
		printf("[%s,\"%s\"],", row[0], row[1]);
    mysql_free_result(sr);
    printf("],");
    sr = mysqlGetResult("select * from decorInfo");
    printf("'decorInfo':{");
	struct slNode *decor2cate=NULL, *tc;
    while((row=mysql_fetch_row(sr)) != NULL)
		{
		// some complex string might have quote so must be in double quote
		printf("\"%s\":{isexpassay:false,label:\"%s\",grp:%s,ft:%s,hasStruct:%s", row[0], row[1], row[3],row[4],row[5]);
		if(row[2] != NULL)
			printf(",parent:\"%s\"", row[2]);
		if(row[6] != NULL)
			printf(",queryUrl:\"%s\"", row[6]);
		printf("},");
		if(strMayPositiveInt(row[4])==FT_cat_n)
			{
			tc=malloc(sizeof(struct slNode));
			tc->name=strdup(row[0]);
			tc->next=decor2cate;
			decor2cate=tc;
			}
		}
    printf("},");
    mysql_free_result(sr);
	/* awkward treatment of cate tk as decors
	*/
	printf("'decor2cateInfo':{");
	for(tc=decor2cate; tc!=NULL; tc=tc->next)
		{
		assert(asprintf(&query, "select info from track2Categorical where name='%s'", tc->name)>0);
		printf("'%s':{%s},", tc->name, mysqlGetOnestring(query));
		free(query);
		free(tc->name);
		}
	slFree(decor2cate);
	printf("},");
	
	

    // metadata vocabulary
    printf("'mdvidx2attrdesc':{");
    sr = mysqlGetResult("select * from trackAttr2idx");
    while((row=mysql_fetch_row(sr)) != NULL)
        printf("%s:[\"%s\",\"%s\"],", row[0], row[1], row[3]);
    printf("},");
    mysql_free_result(sr);
    printf("'mdvc2p':{");
    sr = mysqlGetResult("select * from metadataVocabulary");
    while((row=mysql_fetch_row(sr)) != NULL)
		printf("\"%s\":\"%s\",", row[0], row[1]);
    printf("},");
    mysql_free_result(sr);


    // cytoband (data may be null)
    sr = mysqlGetResult("select chrom,start,stop,colorIdx,name from cytoband");
    printf("'cytoband':[");
    while((row=mysql_fetch_row(sr)) != NULL)
		printf("['%s',%s,%s,%s,'%s'],", row[0], row[1], row[2], row[3], row[4]);
    printf("],");
    mysql_free_result(sr);


    /* retrieve and return ALL scaffold info
	FIXME what about 80,000 contigs???
	*/
    sr = mysqlGetResult("select parent,child,childLength from scaffoldInfo");
	assert(sr!=NULL);
    printf("'scaffoldInfo':[");
    while((row=mysql_fetch_row(sr)) != NULL)
        printf("['%s','%s',%s],",row[0],row[1],row[2]);
    printf("],");
    mysql_free_result(sr);


	if(cgiVarExists("serverload"))
		{
		printf("'trashDir':'%s',",trashDir);

		// get server load
		char *trashfile;
		assert(asprintf(&trashfile, "%s/sysload", trashDir)>0);
		char *comm;
		assert(asprintf(&comm, "uptime > %s", trashfile)>0);
		if(system(comm) == -1)
			errabort("cannot do system call");
		free(comm);
		FILE *fin = fopen(trashfile, "r");
		if(fin != NULL)
			{
			char *line = malloc(1);
			size_t s = 1;
			if(getline(&line, &s, fin))
				{
				// good idea to parse the string on client side
				char delim[] = "\n";
				printf("'serverload':'%s',",strtok(line, delim));
				}
			fclose(fin);
			}
		free(trashfile);

		// host name
		assert(asprintf(&trashfile, "%s/sysload", trashDir)>0);
		assert(asprintf(&comm, "hostname > %s", trashfile)>0);
		if(system(comm) == -1)
			errabort("cannot do system call");
		free(comm);
		fin = fopen(trashfile, "r");
		if(fin != NULL)
			{
			char *line = malloc(1);
			size_t s = 1;
			if(getline(&line, &s, fin))
				{
				// good idea to parse the string on client side
				char delim[] = "\n";
				printf("'hostname':'%s',",strtok(line, delim));
				}
			fclose(fin);
			}
		free(trashfile);
		}

	/* if loading data for repeat browser, don't quit here
	repeatbrowser requests will be dealt with after tempurl assembling
	*/
	if(cgiVarExists("rpbr_init"))
		{
		// retrieve hmtk2geo info
		if(SQUAWK) fputs("rpbr_init...\n",stderr);
		sr=mysqlGetResult("select * from track2GEO");
		printf("'geo2track':{");
		while((row=mysql_fetch_row(sr))!=NULL)
			printf("'%s':'%s',",row[1],row[0]);
		printf("},");
		mysql_free_result(sr);
		mysql_close(conn);
		char *ss= cgiString("rpbrDbname");
		assert(ss!=NULL);
		if(!mysqlConnect(ss))
			{
			fprintf(stderr, "repeatbrowser: cannot connect mysql %s\n", ss);
			return 0;
			}
		sr=mysqlGetResult("select * from subfam2id");
		// info on subfamilies
		printf("'subfam':[");
		while((row=mysql_fetch_row(sr))!=NULL)
			printf("['%s',%s,%s,'%s','%s',%s,%s],", row[0],row[1],row[2],row[3],row[4],row[5],row[6]);
		printf("],");
		mysql_free_result(sr);
		sr=mysqlGetResult("select * from geo2info");
		// info on geo
		printf("'geo':[");
		// TODO input tracks can be null
		while((row=mysql_fetch_row(sr))!=NULL)
			printf("['%s',%s,'%s','%s','%s'],", row[0],row[1],row[2],row[3],row[4]==NULL?"":row[4]);
		printf("],");
		mysql_free_result(sr);
		printf("}");
		done();
		return 1;
		}
	else
		{
		printf("}");
		done();
		return 1;
		}
	}




/*** searching for native tracks by keyword
** gnuhash **
**/
if(cgiVarExists("tkkwsearch"))
	{
	if(hcreate(5000)==0)
		{
		printf("'error':'not enough memory'}");
		return 1;
		}
	char *query;
	/* query label table
	meanwhile output as there's no way of iterating hash after all the queries
	*/
	printf("'lst':[");
	char *_str=cgiString("clauselabel");
	int len=strlen(_str);
	char *str=malloc(sizeof(char)*len);
	strDecode(_str,str,len);
	assert(asprintf(&query, "select name from track2Label where %s",str)>0);
	MYSQL_RES *sr=mysqlGetResult(query);
	free(query);
	if(sr==NULL)
		{
		printf("],'error':'cannot execute mysql query (table of label)'}");
		return 1;
		}
	char **row;
	while((row=mysql_fetch_row(sr))!=NULL)
		{
		ENTRY ent;
		ent.key=strdup(row[0]);
		hsearch(ent,ENTER);
		printf("'%s',",row[0]);
		}
	mysql_free_result(sr);
	// query desc table
	_str=cgiString("clausedesc");
	len=strlen(_str);
	str=malloc(sizeof(char)*len);
	strDecode(_str,str,len);
	assert(asprintf(&query, "select name from track2Detail where %s",str)>0);
	sr=mysqlGetResult(query);
	free(query);
	if(sr==NULL)
		{
		printf("],'error':'cannot execute mysql query (table of detail)'}");
		return 1;
		}
	while((row=mysql_fetch_row(sr))!=NULL)
		{
		ENTRY ent;
		ent.key=strdup(row[0]);
		ENTRY *ent2=hsearch(ent,FIND);
		if(ent2==NULL)
			{
			hsearch(ent,ENTER);
			printf("'%s',",row[0]);
			}
		}
	mysql_free_result(sr);
	// query geo table
	_str=cgiString("clausegeo");
	len=strlen(_str);
	str=malloc(sizeof(char)*len);
	strDecode(_str,str,len);
	assert(asprintf(&query, "select name from track2GEO where %s",str)>0);
	sr=mysqlGetResult(query);
	free(query);
	if(sr==NULL)
		{
		printf("],'error':'cannot execute mysql query (table of geo)'}");
		return 1;
		}
	while((row=mysql_fetch_row(sr))!=NULL)
		{
		ENTRY ent;
		ent.key=strdup(row[0]);
		ENTRY *ent2=hsearch(ent,FIND);
		if(ent2==NULL)
			{
			hsearch(ent,ENTER);
			printf("'%s',",row[0]);
			}
		}
	mysql_free_result(sr);
	printf("]}");
	done();
	return 1;
	}


/*** fetch information on a genome
     exit when done
 ***/
if(cgiVarExists("getgenomeinfo"))
    {
    printf("'info':\"%s\"}", mysqlGetOnestring("select information from config"));
	done();
    return 1;
    }


/*** fetch desc for one repeat
     (for showing decor item info)
     exit when done
	 TODO remove dependency of repeat table, store data in repeat track
 ***/
if(cgiVarExists("getrmskdesc"))
    {
    char *query;
    if(asprintf(&query, "select repName,repClass,repFamily,swScore,milliDiv,milliDel,milliIns from rmsk where genoName=\"%s\" and genoStart=%s and genoEnd=%s", cgiString("chr"), cgiString("start"), cgiString("stop"))<0) errabort("bae");
    MYSQL_RES *sr = mysqlGetResult(query);
	assert(sr!=NULL);
    free(query);
    char **row = mysql_fetch_row(sr);
    if(row!=NULL && row[0]!=NULL && cgiVarExists("repName") && strcmp(row[0], cgiString("repName"))==0)
		{
		printf("'text':'class: %s<br>family: %s<br>SW score: %s<br>divergence: %.1f%%<br>deletions: %.1f%%<br>insertions: %.1f%%'}", row[1], row[2], row[3], strtod(row[4],NULL)/10, strtod(row[5],NULL)/10, strtod(row[6],NULL)/10);
		}
    mysql_free_result(sr);
	done();
    return 1;
    }



/*** get sam read info
	exit when done
 ***/
if(cgiVarExists("getsamreadinfo"))
    {
	char *read=cgiString("read1");
	char *tok;
	char delim[]=",";
	char *chrom=strtok(read,delim);
	if(chrom==NULL) { fputs("sam read1 invalid chrom\n", stderr); return 1; }
	if((tok=strtok(NULL,delim))==NULL) { fputs("sam read1 no start coord\n", stderr); return 1;}
	int start=strMayPositiveInt(tok);
	if(start==-1) {fputs("sam read1 invalid start\n", stderr);return 1;}
	if((tok=strtok(NULL,delim))==NULL) { fputs("sam read1 no stop coord\n", stderr); return 1;}
	int stop=strMayPositiveInt(tok);
	if(stop==-1) {fputs("sam read1 invalid stop\n", stderr);return 1;}
	//char *strand=strtok(NULL,delim);

	tabix_t *fin=tabixOpen(seqPath);
	if(fin==NULL)
		{
		printf("'abort':'Cannot open fa/tabix file'}");
		return 1;
		}

	char *seq=getSeq(fin,chrom, start, stop);
	printf("'read1':");
	if(seq==NULL)
		printf("'ERROR'");
	else
		{
		printf("'%s'",seq);
		free(seq);
		}
	
	if(cgiVarExists("read2"))
		{
		printf(",");
		read=cgiString("read2");
		chrom=strtok(read,delim);
		if(chrom==NULL) { fputs("sam read2 invalid chrom\n", stderr); return 1; }
		if((tok=strtok(NULL,delim))==NULL) { fputs("sam read2 no start coord\n", stderr); return 1;}
		start=strMayPositiveInt(tok);
		if(start==-1) {fputs("sam read2 invalid start\n", stderr);return 1;}
		if((tok=strtok(NULL,delim))==NULL) { fputs("sam read2 no stop coord\n", stderr); return 1;}
		stop=strMayPositiveInt(tok);
		if(stop==-1) {fputs("sam read2 invalid stop\n", stderr);return 1;}
		//strand=strtok(NULL,delim);
		seq=getSeq(fin,chrom, start, stop);
		printf("'read2':");
		if(seq==NULL)
			printf("'ERROR'");
		else
			{
			printf("'%s'",seq);
			free(seq);
			}
		}
	printf("}");
	done();
	ti_close(fin);
    return 1;
    }



/*** save pieces of long URL
     exit when done
 ***/
if(cgiVarExists("saveURLpiece"))
    {
	if(0) fputs("saving URL piece...\n", stderr);
    char *query;
    if(asprintf(&query, "insert into tempURL values('%s', %s, '%s')", session, cgiString("offset"), cgiString("saveURLpiece"))<0) errabort("bae");
    mysqlUpdate(query);
	printf("}");
    if(0) fputs("url piece saved\n", stderr);
	done();
    return 1;
    }




/*******************************************************
   GENE related short queries
   need to prepare list of gene tracks and "normal chrs"
*/

	{
	// gene track list, global variable
	genetrack_sl=NULL;
	char *query = "select hasGene from config";
	char *re = mysqlGetOnestring(query);
	if(re[0]!='0')
		{
		int genegrpid = mysqlGetGenesGroupId();
		if(genegrpid != -1)
			{
			struct slNode *tt;
			char **row;
			assert(asprintf(&query, "select name from decorInfo where parent is NULL and grp=%d", genegrpid)>0);
			MYSQL_RES *sr = mysqlGetResult(query);
			assert(sr!=NULL);
			free(query);
			while((row=mysql_fetch_row(sr)) != NULL)
				{
				tt = malloc(sizeof(struct slNode));
				tt->name = strdup(row[0]);
				tt->next = genetrack_sl;
				genetrack_sl = tt;
				}
			mysql_free_result(sr);
			}
		}
	}

	{
	// normal chr list, global variable
	normalchr_sl=NULL;
	char *re=mysqlGetOnestring("select defaultScaffold from config");
	if(re!=NULL)
		{
		char delim[]=",";
		struct slNode *t;
		char *tok=strtok(re,delim);
		while(tok!=NULL)
			{
			t=malloc(sizeof(struct slNode));
			t->name=tok;
			t->next=normalchr_sl;
			normalchr_sl=t;
			tok=strtok(NULL,delim);
			}
		}
	}


if(cgiVarExists("querygene"))
	{
	/* to query for alternative gene models given a valid gene name

	used in cases including:
	- gsv changing gene model
	- manual splintering, see all gene models

	TODO will be changed in future, gsv function will be singled out as gsv items will remember their gene type and item id
	*/
	assert(genetrack_sl!=NULL);
	struct geneParam *param=initGeneParam();
	param->query_str=cgiString("name");
	param->use_chromid=FALSE;
	struct gene *re=queryGene(param);
	if(re==NULL)
		{
		printf("'lst':[]}");
		done();
		return 1;
		}
	printf("'lst':[");
	struct gene *g;
	for(g=re; g!=NULL; g=g->next)
		{
		printf("{'type':'%s','name':'%s','chrom':'%s','start':%d,'stop':%d,'strand':'%c','desc':\"%s\",'id':%d,'struct':",
			g->type,
			g->name,
			g->chrom,
			g->txstart,
			g->txstop,
			g->strand,
			g->text,
			g->id);
		genestruct2json(g);
		printf("},");
		}
	printf("]}");
	done();
	return 1;
	}



/* find genes by partial name
*/
if(cgiVarExists("findgenebypartialname"))
	{
	char *query=cgiString("query");
	struct geneParam *param=initGeneParam();
	param->query_str=query;
	param->use_chromid=FALSE;
	param->use_exactmatch=FALSE;
	struct gene *re=queryGene(param);
	printf("'query':'%s','lst':[",query);
	struct gene *g;
	int count=0;
	for(g=re; g!=NULL; g=g->next)
		{
		if(count++==100) break;
		printf("'%s',", g->name);
		}
	printf("]}");
	done();
	return 1;
	}




if(cgiVarExists("geofindhmtk"))
	{
	struct slNode *nsl=NULL, *t; // for dataset querying
	char delim[]=",";
	char *tok=strtok(cgiString("lst"),delim);
	char *query;
	char **row;
	printf("'result':[");
	while(tok!=NULL)
		{
		assert(asprintf(&query, "select name from track2GEO where geo like '%%%s%%'", tok)>0);
		MYSQL_RES *sr=mysqlGetResult(query);
		free(query);
		while((row=mysql_fetch_row(sr))!=NULL)
			{
			printf("'%s','%s',",tok,row[0]);

			t=malloc(sizeof(struct slNode));
			t->name=strdup(row[0]);
			t->next=nsl;
			nsl=t;
			}
		mysql_free_result(sr);
		tok=strtok(NULL,delim);
		}
	printf("],");

	struct slNode *datasetsl=findDatasetByHmtksl(nsl);
	printf("'datasets':[");
	for(t=datasetsl; t!=NULL; t=t->next)
		printf("'%s',", t->name);
	printf("]}");

	slFree(datasetsl);
	slFree(nsl);
	}


/* in case of having session and statusId, retrieve scaffolds in use,
   and initiate lastChrIdx, chrInfo
   the code will have to be ahead of any other code that uses lastChrIdx and chrInfo
   TODO make sure following "exit when done" branches need it
   if they don't need it, move them upward
 */
if(session!=NULL && statusId!=-1)
    {
    /* but some cases there's no corresponding entry in runtime table
    isfirst, gene set view, restore status about gene set view...
	*/
    char *query;
    if(asprintf(&query, "select count from scaffoldRuntime where session='%s' and statusId=%d", session, statusId)<0) errabort("bae");
    char *re = mysqlGetOnestring(query);
    free(query);
    if(re != NULL)
		{
		reviveScaffold();
		}
	else
		{
		defaultScaffold();
		}
    }



/*** fetch detail for one track
     this never gonna make long url
     exit when done
 ***/
if(cgiVarExists("gettrackdetails"))
    {
    if(!cgiVarExists("tkname"))
        {
	printf("'abort':'no track name given'}");
	return 1;
	}
    char *query;
    char *result;
    char *tkname = cgiString("tkname");

    // general
    printf("'general':\"");
    if(asprintf(&query, "select detail from track2Detail where name=\"%s\"", tkname)<0) errabort("bae");
    result = mysqlGetOnestring(query);
    free(query);
    if(result == NULL)
	printf("NULL");
    else
        printf("%s",result);
    printf("\",");

    // processing
    printf("'processing':\"");
    if(asprintf(&query, "select detail from track2ProcessInfo where name='%s'", tkname)<0) errabort("bae");
    result = mysqlGetOnestring(query);
    free(query);
    if(result == NULL)
        printf("NULL");
    else
        printf("%s",result);
    printf("\",");

    // geo
    char *geo;
    printf("'geo':\"");
    if(asprintf(&query, "select geo from track2GEO where name='%s'", tkname)<0) errabort("bae");
    geo = mysqlGetOnestring(query);
    free(query);
    if(geo == NULL)
        printf("NULL");
    else
        printf("%s", geo);
    printf("\",");

    // other versions
    printf("'other':[");
    if(geo != NULL)
        {
        if(asprintf(&query, "select b.name,b.info,c.label from track2GEO as a,track2VersionInfo as b,track2Label as c where a.geo='%s' and a.name=b.name and a.name=c.name", geo)<0) errabort("bae");
		MYSQL_RES *sr = mysqlGetResult(query);
		free(query);
		if(sr!=NULL)
			{
			char **row;
			while((row=mysql_fetch_row(sr)) != NULL)
				printf("[\"%s\",\"%s\", \"%s\"],", row[0], row[1], row[2]);
			mysql_free_result(sr);
			}
		}
    printf("],");

    // bam
    printf("'bam':[");
    if(asprintf(&query, "select bamfile,bamfilelabel from track2BamInfo where name='%s'", tkname)<0) errabort("bae");
    MYSQL_RES *sr = mysqlGetResult(query);
    free(query);
    if(sr!=NULL)
        {
		char **row;
		while((row=mysql_fetch_row(sr)) != NULL)
			printf("[\"%s\",\"%s\"],", row[0], row[1]);
		mysql_free_result(sr);
		}
    printf("],");

    printf("}");
	done();
    return 1;
    }


/*** revive URL after it has been completely saved in pieces
 ***/
if(cgiVarExists("reviveURL"))
	{
	// read everything from mysql, write into temp file, revive cgi param enviroment from file, clean up...
	char *query, **row;

	// count how many pieces are in db, to allocate memory for array...
	if(asprintf(&query, "select count(session) from tempURL where session=\"%s\"", session)<0) errabort("bae");
	int number = mysqlGetPositiveInteger(query);
	if(number == -1)
		{
		printf("\"abort\":\"cannot run query (supposedly cached URL missing)\"}");
		return 1;
		}

	struct urlpiece urp[number];
	int i=0;

	// having array, store url and offset into it
	if(asprintf(&query, "select offset,urlpiece from tempURL where session=\"%s\"", session)<0) errabort("bae");
	MYSQL_RES *sr = mysqlGetResult(query);
	while((row=mysql_fetch_row(sr)) != NULL)
		{
		assert((urp[i].offset=strMayPositiveInt(row[0]))!=-1);
		urp[i].text=strdup(row[1]);
		i++;
		}
	free(query);
	mysql_free_result(sr);

	// arrange pieces in correct order
	qsort(&urp, (size_t)number, sizeof(struct urlpiece), urlpiece_cmp);

	char *urlstring = NULL; // concatenate all of them into one string
	for(i=0; i<number; i++)
		{
		if(urlstring == NULL)
			{
			urlstring=strdup(urp[i].text);
			}
		else
			{
			char *tmp;
			if(asprintf(&tmp, "%s%s", urlstring, urp[i].text)<0) errabort("bae");
			free(urlstring);
			urlstring = tmp;
			}
		}
	if(0) fprintf(stderr,"assembled urlstring:\n%s\n\n", urlstring);

	//if(cgiVarExists("decodeURL"))
		{
		/* always decode URL */
		int len=strlen(urlstring);
		char *out = malloc(sizeof(char) * len);
		strDecode(urlstring, out, len);
		free(urlstring);
		urlstring = out;
		if(0) fprintf(stderr, "decoded urlstring:\n%s\n\n", urlstring);
		}


	/* dissect original url, use cgiVarSet on each param...
	must not free tsl->name
	as its values will be used in cgi enviroment
	*/
	struct slNode *tsl = NULL, *t;
	char delim[] = "&";
	// first param
	char *tok = strtok(urlstring, delim); assert(tok!=NULL);
	while(tok != NULL)
		{
		t = malloc(sizeof(struct slNode));
		t->name = tok;
		t->next = tsl;
		tsl = t;
		tok = strtok(NULL, delim);
		}
	char delim2[] = "=";
	char *tok1, *tok2;
	for(t=tsl; t!=NULL; t=t->next)
		{
		tok1 = strtok(t->name, delim2);
		tok2 = strtok(NULL, delim2);
		cgiVarSet(tok1, tok2);
		}


	// pieces in db must be cleaned up, or they will mess up new queries under same session
	if(asprintf(&query, "delete from tempURL where session=\"%s\"", session)<0) errabort("bae");
	mysqlUpdate(query);
	free(query);

	if(SQUAWK) fputs("url revived...\n", Squawk);
	}






/*** fetch desc for one gene for showing decor item info
     providing gene track name and item id for querying
     gene description must not contain double quote
     TODO geneontology
     exit when done
 ***/
if(cgiVarExists("getgenedesc"))
    {
    char *query;
    assert(asprintf(&query, "select description from %ssymbol where id=%s", cgiString("genetype"), cgiString("itemid"))>0);
    char *desc = mysqlGetOnestring(query);
    free(query);
    printf("'desc':\"%s\"}", desc==NULL?"No description":desc);
	done();
    return 1;
    }



/****************************
  SESS save one status under session
      exit when done

when saving status, the status ID will be re-generated
so need to update genesetRuntime table when doing gene set view

special case for rpbr beaming, told by RM_rpbr_beam
****************************/
if(cgiVarExists("saveStatus"))
    {
	if(SQUAWK)
		fprintf(stderr, "saving status for %s %d...\n", session, statusId);

    // fields to be inserted
    // TODO decor4 is not saved
	char *clientCtrl,
		 *note,
	*customTrackAttr,
	*juxWhat,
	*juxNote,
	*dspStat,
	*corrStat,
	*custmd,
	*mdconshow,
	*dataset,
	*pairwisecomp,
	*bed_n, *bed_c,
	*bedgraph_n, *bedgraph_c,
	*sam_n, *sam_c,
	*qdecor_n,
	*lr_n, *lr_c,
	*cat_n, *cat_c,
	*bigwighmtk_n, *bigwighmtk_c,
	*bam_n, *bam_c,
	*decorRenderingParams,
	*qtc,
	*gssparam,
	*stickynote,
	*itemlist = NULL, // in case of running genesetview
	*scaffoldNames = NULL; // in case of using scaffold

    int scaffoldCount = 0;
    char *query;

    // make new status id
    int newstatusid;
    srand(time(0));
    do {
        newstatusid = rand();
       }
    while(sessionStatusExists(session, newstatusid));

    int runmode = cgiInt("runmode");
	if(runmode==RM_rpbr_beam)
		{
		// rpbr won't use scaffold runtime, so must insert it
		defaultScaffold();
		printf("'key':%s,",cgiString("viewkey"));
		}

    // always need to save scaffold, and update status id
    assert(asprintf(&query, "select count,names from scaffoldRuntime where session='%s' and statusId=%d", session, statusId)>0);
    MYSQL_RES *sr = mysqlGetResult(query);
	assert(sr!=NULL);
    free(query);
    char **row = mysql_fetch_row(sr);
    assert(row[0] != NULL);
    assert(row[1] != NULL);
    assert((scaffoldCount=strMayPositiveInt(row[0]))!=-1);
    assert(asprintf(&scaffoldNames, "'%s'", row[1])>0);
    mysql_free_result(sr);

    // need to update status in scaffoldRuntime table
    assert(asprintf(&query, "update scaffoldRuntime set statusId=%d where session='%s' and statusId=%d", newstatusid, session, statusId)>0);
    mysqlUpdate(query);
    free(query);

	if(runmode==RM_rpbr_beam)
		{
		/* ranked repeats in a subfam
		as from rpbr
		rmsk repeats have strand info!!
		*/
		struct beditem2 *rsl=getRepeats4subfam(), *t;
		char *old="", *new, *tok;
		boolean atfirst=TRUE;
		char delim[]=",";
		int flank=cgiInt("flankbp");
		tok=strtok(cgiString("rankid"),delim);
		// make a string for repeats, by the ranked order
		while(tok!=NULL)
			{
			for(t=rsl; t!=NULL; t=t->next)
				{
				if(strcmp(t->rest, tok)==0)
					{
					int chrid=strMayChrID(t->chrom);
					if(chrid==-1) continue;
					int bstart,bstop, f5start, f5stop, f3start, f3stop;
					if(t->strand=='+')
						{
						f5start=max(t->start-flank,0);
						f5stop=t->start;
						f3start=t->stop;
						f3stop=min(chrInfo[chrid]->length, t->stop+flank);
						bstart=f5start;
						bstop=f3stop;
						}
					else
						{
						f5start=t->stop;
						f5stop=min(chrInfo[chrid]->length, t->stop+flank);
						f3start=max(t->start-flank,0);
						f3stop=t->start;
						bstart=f3start;
						bstop=f5stop;
						}
					assert(asprintf(&new, "%s%s:%d-%d,%d,%d,%d,%d,%d,%d,%d,%c,0,",
						old,
						t->chrom, t->start, t->stop,
						chrid, bstart, bstop,
						f5start, f5stop,
						f3start, f3stop,
						t->strand)>0);
					if(atfirst) atfirst=FALSE;
					else free(old);
					old=new;
					}
				}
			tok=strtok(NULL,delim);
			}
		assert(asprintf(&itemlist, "'%s'", old)>0);
		free(old);
		// run mode must be changed to "custom gene set"
		runmode=RM_gsv_c;
		// compose gssparam
		struct tnode *_t=malloc(sizeof(struct tnode));
		_t->t1="gssparam";
		assert(asprintf(&(_t->t2), "{\"gss_says\":\"%d bp upstream and %d bp downstream surrounding gene body or entire interval\",\"gss_opt\":\"custom\",\"gss_origin\":\"genebody\",\"gss_up\":\"%d\",\"gss_down\":\"%d\"}", flank,flank,flank,flank)>0);
		_t->next=cgiparamsl;
		cgiparamsl=_t;
		}
    else if(runmode==RM_gsv_c || runmode==RM_gsv_kegg)
        {
		// gene set view string
        if(asprintf(&query, "select itemlist from genesetRuntime where session='%s' and statusId=%d", session, statusId)<0) errabort("bae");
		char *tmp = mysqlGetOnestring(query);
		assert(tmp != NULL);
        if(asprintf(&itemlist, "'%s'", tmp)<0) errabort("bae");
        free(query);
		// update status ID in collateitemlist
		if(asprintf(&query, "update genesetRuntime set statusId=%d where session='%s' and statusId=%d", newstatusid, session, statusId)<0) errabort("bae");
		mysqlUpdate(query);
		free(query);
		}

    // have note?
    if(cgiVarExists("statusNote"))
        note = cgiString("statusNote");
    else
        {
        char *cast[] = {"Shaun the sheep", "Wallace the British folk", "Gromit the clay-dog", "Bunny the were-rabbit", "Tachikoma the taranchula", "Bato the veteran", "Popeye the sailor", "Ginger the chicken", "Rocky the rooster", "Tweedy the rancher", "Emily the bride", "Remy the chef", "Roddy the pet", "Rita the stealth", "Shrek the ogre", "Lyra's Golden Compass", "Will's Subtle Knife", "A computerless wire"};
		note = cast[(int)(18*((float)rand()/RAND_MAX))];
		}

    // prepare the rest of insert strings
    if(cgiVarExists("clientCtrl"))
		assert(asprintf(&clientCtrl, "'%s'", cgiString("clientCtrl"))>0);
    else
        clientCtrl = "\\N";
    if(cgiVarExists("customTrackAttr"))
		assert(asprintf(&customTrackAttr, "'%s'", cgiString("customTrackAttr"))>0);
    else
        customTrackAttr = "\\N";
    if(cgiVarExists("juxWhat"))
		assert(asprintf(&juxWhat, "'%s'", cgiString("juxWhat"))>0);
    else
        juxWhat = "\\N";
    if(cgiVarExists("juxNote"))
		assert(asprintf(&juxNote, "'%s'", cgiString("juxNote"))>0);
    else
        juxNote= "\\N";
    if(cgiVarExists("dspStat"))
		assert(asprintf(&dspStat, "'%s'", cgiString("dspStat"))>0);
    else
        dspStat = "\\N";
    if(cgiVarExists("corrStat"))
		assert(asprintf(&corrStat, "'%s'", cgiString("corrStat"))>0);
    else
        corrStat = "\\N";
    if(cgiVarExists("custmd"))
		assert(asprintf(&custmd, "'%s'", cgiString("custmd"))>0);
    else
        custmd = "\\N";
    if(cgiVarExists("mdconshow"))
		assert(asprintf(&mdconshow, "'%s'", cgiString("mdconshow"))>0);
    else
        mdconshow = "\\N";
    if(cgiVarExists("dataset"))
		assert(asprintf(&dataset, "'%s'", cgiString("dataset"))>0);
    else
        dataset = "\\N";
    if(cgiVarExists("pairwisecomp"))
		assert(asprintf(&pairwisecomp, "'%s'", cgiString("pairwisecomp"))>0);
    else
        pairwisecomp = "\\N";
	if(cgiVarExists("decor0"))
		assert(asprintf(&bed_n, "'%s'", cgiString("decor0"))>0);
	else
		bed_n="\\N";
	if(cgiVarExists("decor1"))
		assert(asprintf(&bed_c, "'%s'", cgiString("decor1"))>0);
	else
		bed_c="\\N";
	if(cgiVarExists("hmtk2"))
		assert(asprintf(&bedgraph_n, "'%s'", cgiString("hmtk2"))>0);
	else
		bedgraph_n="\\N";
	if(cgiVarExists("hmtk3"))
		assert(asprintf(&bedgraph_c, "'%s'", cgiString("hmtk3"))>0);
	else
		bedgraph_c="\\N";
	if(cgiVarExists("decor4")) // sam/tabix not in use
		assert(asprintf(&sam_n, "'%s'", cgiString("decor4"))>0);
	else
		sam_n="\\N";
	if(cgiVarExists("decor5"))
		assert(asprintf(&sam_c, "'%s'", cgiString("decor5"))>0);
	else
		sam_c="\\N";
	if(cgiVarExists("decor8"))
		assert(asprintf(&qdecor_n, "'%s'", cgiString("decor8"))>0);
	else
		qdecor_n="\\N";
	if(cgiVarExists("decor9"))
		assert(asprintf(&lr_n, "'%s'", cgiString("decor9"))>0);
	else
		lr_n="\\N";
	if(cgiVarExists("decor10"))
		assert(asprintf(&lr_c, "'%s'", cgiString("decor10"))>0);
	else
		lr_c="\\N";
	if(cgiVarExists("hmtk12"))
		assert(asprintf(&cat_n, "'%s'", cgiString("hmtk12"))>0);
	else
		cat_n="\\N";
	if(cgiVarExists("hmtk13"))
		assert(asprintf(&cat_c, "'%s'", cgiString("hmtk13"))>0);
	else
		cat_c="\\N";
	if(cgiVarExists("hmtk14"))
		assert(asprintf(&bigwighmtk_n, "'%s'", cgiString("hmtk14"))>0);
	else
		bigwighmtk_n="\\N";
	if(cgiVarExists("hmtk15"))
		assert(asprintf(&bigwighmtk_c, "'%s'", cgiString("hmtk15"))>0);
	else
		bigwighmtk_c="\\N";
	if(cgiVarExists("decor17"))
		assert(asprintf(&bam_n,"'%s'",cgiString("decor17"))>0);
	else
		bam_n="\\N";
	if(cgiVarExists("decor18"))
		assert(asprintf(&bam_c,"'%s'",cgiString("decor18"))>0);
	else
		bam_c="\\N";
	

    if(cgiVarExists("decorRenderingParams"))
		assert(asprintf(&decorRenderingParams, "'%s'", cgiString("decorRenderingParams"))>0);
    else
        decorRenderingParams = "\\N";
    if(cgiVarExists("qtc"))
		assert(asprintf(&qtc, "'%s'", cgiString("qtc"))>0);
    else
        qtc = "\\N";
    if(cgiVarExists("gssparam"))
		assert(asprintf(&gssparam, "'%s'", cgiString("gssparam"))>0);
    else
        gssparam = "\\N";
    if(cgiVarExists("stickynote"))
		assert(asprintf(&stickynote, "'%s'", cgiString("stickynote"))>0);
    else
        stickynote = "\\N";

    // save new status under this session, return status ID
	/*** status note must be double quoted, as some words contain single quote
	***/
    assert(asprintf(&query, "insert into session values('%s',%d,\"%s\",%s,%s,"
    	"%d,%s,%s,%s,%s,"
		"%s,%s,%s,%s,%s,"
		"%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,%s,"
		"%s,%s,%s,%s,"
		"%d,%s,"
		"%s,%s,"
		"\\N,\\N,"
		"\\N,\\N,"
		"\\N,\\N,"
		"\\N,\\N"
		")",
    	session, newstatusid, note, clientCtrl, customTrackAttr, 
		runmode, juxWhat, juxNote, dspStat, corrStat,
		itemlist==NULL?"\\N":itemlist, custmd, mdconshow, dataset, pairwisecomp,
		bed_n,bed_c,bedgraph_n,bedgraph_c,sam_n,sam_c,qdecor_n,lr_n,lr_c,cat_n,cat_c,bigwighmtk_n,bigwighmtk_c,
		decorRenderingParams, qtc, gssparam, stickynote, 
		scaffoldCount, scaffoldNames==NULL?"\\N":scaffoldNames,
		bam_n,bam_c
		)>0);
    mysqlUpdate(query);
    free(query);
    printf("'statusTicket':[%d,\"%s\",'%s']}",
		newstatusid,
		note,
		dspStat2readable(cgiString("dspStat"), cgiString("runmode"))
		);
	done();
    return 1;
    }

if(cgiVarExists("pastSession"))
    {
    /* SESS retrieve saved status for a past session, 
       display on client side for user to choose which one to restore
     */
    char *session2use = cgiString("pastSession"); // session on which info will be retrieved
	sessionstatus2json(session2use);
	printf("}");
    // current session is still in global variable

    if(cgiVarExists("updateRuntimetable"))
        {
		/* update run time tables as session ID has changed!! */
		char *query;
		if(asprintf(&query, "update scaffoldRuntime set session='%s' where session='%s' and statusId=%d", session2use, session, statusId)<0) errabort("bae");
		mysqlUpdate(query);
		free(query);
		int runmode = cgiInt("runmode");
		if(runmode==RM_gsv_c || runmode==RM_gsv_kegg)
			{
			// running gene set view, update that runtime table
			assert(asprintf(&query, "update genesetRuntime set session='%s' where session='%s' and statusId=%d", session2use, session, statusId)>0);
			mysqlUpdate(query);
			free(query);
			}
		}
	done();
    return 1;
    }

if(cgiVarExists("deleteStatus"))
    {
    char *statusId = cgiString("deleteStatus");
    char *session = cgiString("session");
    char *query;
    if(asprintf(&query, "delete from session where session=\"%s\" and statusId=%s", session, statusId)<0) errabort("bae");
    mysqlUpdate(query);
    free(query);
    printf("}");
	done();
    return 1;
    }



/*** longrange wreath track
     exit when done
 ***/
if(cgiVarExists("longrangewreath"))
	{
	if(SQUAWK) fputs("longrange wreath track\n", stderr);
	// regionlst
	struct displayedRegion dsp;
	dsp.runmode = RM_genome;
	dsp.head = NULL;
	dsp.tail = NULL;
	dsp.atbplevel = FALSE;
	dsp.usedSummaryNumber = 0;
	struct region *r;
	char delim[]=",";
	char *tok=strtok(cgiString("regionlst"), delim);
	while(tok!=NULL)
		{
		r=malloc(sizeof(struct region));
		r->chromIdx=strMayChrID(tok);
		if(r->chromIdx == -1)
			{
			scaffoldAddnew(tok);
			r->chromIdx = lastChrIdx;
			}
		assert((tok=strtok(NULL,delim))!=NULL);
		assert((r->dstart=strMayPositiveInt(tok))!=-1);
		assert((tok=strtok(NULL,delim))!=NULL);
		assert((r->dstop=strMayPositiveInt(tok))!=-1);
		assert((tok=strtok(NULL,delim))!=NULL);
		assert((r->summarySize=strMayPositiveInt(tok))!=-1);
		dsp.usedSummaryNumber+=r->summarySize;
		dspAddRegionTail(&dsp, r);
		tok=strtok(NULL, delim);
		}
	// dsp is ready
	if(0)
		{
		for(r=dsp.head; r!=NULL; r=r->next)
			fprintf(stderr, "%s %d %d %d\n", chrInfo[r->chromIdx]->name, r->dstart, r->dstop, r->summarySize);
		}
	/* for each track, print its data
	must not interlace strtok with tabix query */
	struct track *sl=NULL, *t;
	if(cgiVarExists("hmtk2")) parseTrackParam(cgiString("hmtk2"), FT_bedgraph_n, &sl);
	if(cgiVarExists("hmtk3")) parseTrackParam(cgiString("hmtk3"), FT_bedgraph_c, &sl);
	if(cgiVarExists("hmtk14")) parseTrackParam(cgiString("hmtk14"), FT_bigwighmtk_n, &sl);
	if(cgiVarExists("hmtk15")) parseTrackParam(cgiString("hmtk15"), FT_bigwighmtk_c, &sl);
	if(cgiVarExists("hmtk12")) parseTrackParam(cgiString("hmtk12"), FT_cat_n, &sl);
	if(cgiVarExists("hmtk13")) parseTrackParam(cgiString("hmtk13"), FT_cat_c, &sl);
	if(cgiVarExists("decor0")) parseTrackParam(cgiString("decor0"), FT_bed_n, &sl);
	if(cgiVarExists("decor1")) parseTrackParam(cgiString("decor1"), FT_bed_c, &sl);
	if(cgiVarExists("decor8")) parseTrackParam(cgiString("decor8"), FT_qdecor_n, &sl);

	printf("'data':[");
	for(t=sl; t!=NULL; t=t->next)
		{
		switch(t->ft)
			{
			case FT_bed_n:
			case FT_bed_c:
				printf("{'name':'%s','data':", t->name);
				printJson_itemDensity(&dsp, t->urlpath, t->ft);
				printf("},");
				break;
			case FT_bedgraph_n:
			case FT_bedgraph_c:
			case FT_bigwighmtk_n:
			case FT_bigwighmtk_c:
			case FT_qdecor_n:
			case FT_cat_c:
			case FT_cat_n:
				printf("{'name':'%s','data':", t->name);
				printJson_dsp_spnum(&dsp, t->urlpath, t->ft);
				printf("},");
				break;
			}
		}
	slFree(sl);
	printf("]}");
	done();
	return 1;
	}


/*** BEV bird's eye view
TODO get integrated with dsp
     exit when done
 ***/
if(cgiVarExists("birdseyeview"))
	{
	if(SQUAWK) fputs("running bev...\n", Squawk);
	int runtype = cgiInt("runtype");

	struct track *tracksl=NULL, *tk;
	char *tok;
	char delim[] = ",";
	if(cgiVarExists("hmtk2")) parseTrackParam(cgiString("hmtk2"), FT_bedgraph_n, &tracksl);
	if(cgiVarExists("hmtk3")) parseTrackParam(cgiString("hmtk3"), FT_bedgraph_c, &tracksl);
	if(cgiVarExists("hmtk14")) parseTrackParam(cgiString("hmtk14"), FT_bigwighmtk_n, &tracksl);
	if(cgiVarExists("hmtk15")) parseTrackParam(cgiString("hmtk15"), FT_bigwighmtk_c, &tracksl);
	if(cgiVarExists("hmtk12")) parseTrackParam(cgiString("hmtk12"), FT_cat_n, &tracksl);
	if(cgiVarExists("hmtk13")) parseTrackParam(cgiString("hmtk13"), FT_cat_c, &tracksl);
	if(cgiVarExists("decor0")) parseTrackParam(cgiString("decor0"), FT_bed_n, &tracksl);
	if(cgiVarExists("decor1")) parseTrackParam(cgiString("decor1"), FT_bed_c, &tracksl);
	if(cgiVarExists("decor8")) parseTrackParam(cgiString("decor8"), FT_qdecor_n, &tracksl);
	if(0)
		{
		fputs("all tracks:\n", stderr);
		for(tk=tracksl; tk!=NULL; tk=tk->next)
			fprintf(stderr, "\t%s\n", tk->name);
		}

	/* make a tnode sl for searching
	t1: scaffold name
	num: plot width (number of summary points)
	*/
	lastChrIdx = -1;
	int sptotal=0;
	struct tnode *sl=NULL, *tt;
	char *tmp = cgiString("plotwidth");
	tok = strtok(strdup(tmp), delim);
	while(tok != NULL)
		{
		lastChrIdx++;
		char *seqname = tok;
		// get seq summary size
		tok = strtok(NULL, delim);
		assert(tok!=NULL);
		int sp = strMayPositiveInt(tok);
		assert(sp!=-1);
		tt=malloc(sizeof(struct tnode));
		tt->t1=seqname;
		tt->num=sp;
		tt->next=sl;
		sl=tt;
		sptotal += sp;
		strtok(NULL, delim);
		tok = strtok(NULL, delim);
		}
	// fill up chrInfo
	chrInfo = malloc(sizeof(struct chrInfoNode *)*(lastChrIdx+1));
	tok = strtok(strdup(tmp), delim);
	int i=0;
	while(tok != NULL)
		{
		chrInfo[i] = malloc(sizeof(struct chrInfoNode));
		chrInfo[i]->name = tok;
		strtok(NULL, delim);
		tok = strtok(NULL, delim);
		assert(tok != NULL);
		assert((chrInfo[i++]->length=strMayPositiveInt(tok))!=-1);
		tok = strtok(NULL, delim);
		}
	assert(i==lastChrIdx+1);

	printf("'tkdata':{");
	for(tk=tracksl; tk!=NULL; tk=tk->next)
		{
		tk->data = NULL;
		if(runtype == 3)
			{
			tk->data = malloc(sizeof(double) * sptotal);
			if(tk->data == NULL)
				{
				printf("\"abort\":\"Out of memory! Reduce chr plot width and try again\"}");
				return 1;
				}
			}
		int j;
		if(tk->ft==FT_bedgraph_c || tk->ft==FT_bedgraph_n || tk->ft==FT_qdecor_n)
			{
			printf("'%s':{", tk->name);
			tabix_t *fin=tabixOpen(tk->urlpath);
			int k=0;
			for(i=0; i<=lastChrIdx; i++)
				{
				int sSize = tnodesl_search4num(sl, chrInfo[i]->name);
				assert(sSize!=-1);
				double values[sSize];
				for(j=0; j<sSize; j++)
					values[j] = .0;
				tabixQuery_bedgraph(fin, chrInfo[i]->name, 0, chrInfo[i]->length, sSize, &values[0]);
				printf("\"%s\":[", chrInfo[i]->name);
				for(j=0; j<sSize; j++)
					printf("%.2f,", values[j]);
				printf("],");
				if(runtype == 3)
					{
					for(j=0; j<sSize; j++)
						(tk->data)[k++] = values[j];
					}
				}
			ti_close(fin);
			printf("},");
			}
		else if(tk->ft==FT_bigwighmtk_n || tk->ft==FT_bigwighmtk_c)
			{
			printf("'%s':{", tk->name);
			int k=0;
			for(i=0; i<=lastChrIdx; i++)
				{
				int sSize = tnodesl_search4num(sl, chrInfo[i]->name);
				assert(sSize!=-1);
				double *data=malloc(sizeof(double)*sSize);
				int x;
				for(x=0; x<sSize; x++) data[x]=0;
				bigwigQuery(tk->urlpath, chrInfo[i]->name, 1, chrInfo[i]->length-1, sSize, data);
				printf("\"%s\":[", chrInfo[i]->name);
				for(j=0; j<sSize; j++)
					printf("%.2f,", data[j]);
				printf("],");
				if(runtype == 3)
					{
					for(j=0; j<sSize; j++)
						(tk->data)[k++] = data[j];
					}
				free(data);
				}
			printf("},");
			}
		else if(tk->ft==FT_bed_c || tk->ft==FT_bed_n || tk->ft==FT_lr_n || tk->ft==FT_lr_c)
			{
			printf("'%s':{", tk->name);
			tabix_t *fin=tabixOpen(tk->urlpath);
			int k=0;
			for(i=0; i<=lastChrIdx; i++)
				{
				int sSize = tnodesl_search4num(sl, chrInfo[i]->name);
				assert(sSize!=-1);
				int sLen = chrInfo[i]->length;
				double values[sSize];
				for(j=0; j<sSize; j++)
					values[j] = .0;
				struct region r;
				r.dstart = 0;
				r.dstop = sLen;
				r.chromIdx = i;
				r.summarySize = sSize;
				tabixQuery_regiondensity(fin, NULL, NULL, tk->ft, &r, &values[0]);
				printf("\"%s\":[", chrInfo[i]->name);
				for(j=0; j<sSize; j++)
					printf("%.2f,", values[j]);
				printf("],");
				if(runtype == 3)
					{
					for(j=0; j<sSize; j++)
						(tk->data)[k++] = values[j];
					}
				}
			ti_close(fin);
			printf("},");
			}
		else if(tk->ft==FT_cat_c || tk->ft==FT_cat_n)
			{
			printf("'%s':{", tk->name);
			tabix_t *fin=tabixOpen(tk->urlpath);
			int k=0;
			for(i=0; i<=lastChrIdx; i++)
				{
				int sSize = tnodesl_search4num(sl, chrInfo[i]->name);
				assert(sSize!=-1);
				int values[sSize];
				for(j=0; j<sSize; j++)
					values[j] = -1;
				tabixQuery_categorical(fin, chrInfo[i]->name, 0, chrInfo[i]->length, sSize, &values[0]);
				printf("'%s':[", chrInfo[i]->name);
				for(j=0; j<sSize; j++)
					printf("%d,", values[j]);
				printf("],");
				if(runtype == 3)
					{
					for(j=0; j<sSize; j++)
						(tk->data)[k++] = values[j];
					}
				}
			ti_close(fin);
			printf("},");
			}
		//if(fout != NULL) fclose(fout);
		}
	printf("},");

	if(runtype == 3)
		{
		// data are ready now, run htest, compute and return pvalues
		parseParamHtest(tracksl);

		// horrible duplication of code below...
		if(SQUAWK) fputs("Running bev htest...\n", Squawk);
		char *rsourcefile;
		if(asprintf(&rsourcefile, "%s/%s.R", trashDir, session)<0) errabort("bae");
		FILE *fout = fopen(rsourcefile, "w");
		if(fout == NULL) errabort(rsourcefile);
		char *routfile;
		if(asprintf(&routfile, "%s/%s.Rout", trashDir, session)<0) errabort("bae");
		char *pvfile;
		if(asprintf(&pvfile, "%s/%s.pvalues", trashDir, session)<0) errabort("bae");
		fputs("options(echo=FALSE)\n", fout);
		fputs("grp=c(", fout);
		for(tk=tracksl; tk!=NULL; tk=tk->next)
			{
			if(tk->gid != 0)
			fprintf(fout, "%d,", tk->gid);
			}
		fseek(fout, -1, SEEK_CUR);
		fputs(")\npv=numeric()\n", fout);
		// write track data, one vector for each summary point over participating tracks
		int j;
		int k=0; // cumulative over all regions, index of tk->data
		for(i=0; i<=lastChrIdx; i++)
			{
			int sSize=tnodesl_search4num(sl, chrInfo[i]->name);
			assert(sSize!=-1);
			for(j=0; j<sSize; j++)
				{
				fputs("data = c(", fout);
				for(tk=tracksl; tk!=NULL; tk=tk->next)
					{
					if(tk->gid != 0)
					fprintf(fout, "%f,", (float)(tk->data)[k]);
					}
				fseek(fout, -1, SEEK_CUR);
				fprintf(fout, ")\npv[%d]=kruskal.test(data, grp)$p.value\n", k+1);
				k++;
				}
			}
		if(cgiVarExists("htestc"))
			fprintf(fout, "pv=p.adjust(pv, method=\"%s\")\n", cgiString("htestc"));
		fprintf(fout, "write(pv, \"%s\", sep=\"\\n\")\n", pvfile);
		fclose(fout);
		// run test
		char *command;
		if(asprintf(&command, "R CMD BATCH --no-save %s %s", rsourcefile, routfile)<0) errabort("BAE");
		if(system(command) == -1)
			errabort("cannot spawn process");
		free(command);
		// parse pvalue file, return pvalues
		printf("\"%s\":{", cgiString("htestvn"));
		FILE *fin = fopen(pvfile,"r");
		char *line = malloc(1);
		size_t s = 1;
		for(i=0; i<=lastChrIdx; i++)
			{
			printf("\"%s\":[", chrInfo[i]->name);
			int sSize=tnodesl_search4num(sl, chrInfo[i]->name);
			for(j=0; j<sSize; j++)
				{
				if(getline(&line, &s, fin) == -1)
					errabort("expecting more pvalues but reaches EOF...");
				if(line[0] == 'N') // NaN
					printf("1,");
				else if(line[0] == '1')
					printf("1,");
				else
					printf("%.4f,", strtod(line, NULL));
				}
			printf("],");
			}
		fclose(fin);
		printf("}");
		}

	slFree(sl);

	printf("}");
	done();
	return 1;
	}





/*** load track annotation in datasets 
     exit when done
 ***/
if(cgiVarExists("importdataset"))
    {
	// argument is ds name joined by comma
    char *string = cgiString("importdataset");
    char delim[] = ",";
	char *tok=strtok(string, delim);
	assert(tok!=NULL);
	struct slNode *sl=NULL;
	printf("'dsnames':[");
	while(tok!=NULL)
		{
		struct slNode *t=malloc(sizeof(struct slNode));
		t->name=tok;
		printf("'%s',", tok);
		t->next=sl;
		sl=t;
		tok=strtok(NULL,delim);
		}
	printf("],");
	printJson_hmtk4datasets(sl);
	printf("}");
	done();
    return 1;
    }


/*** load track annotation in datasets 
     exit when done
 ***/
if(cgiVarExists("loaddataset_tkname"))
    {
    char *string = cgiString("loaddataset_tkname");
    char delim[] = ",";
	char *tok=strtok(string, delim);
	assert(tok!=NULL);
	struct slNode *sl=NULL, *t;
	printf("'dsnames':[");
	while(tok!=NULL)
		{
		t=malloc(sizeof(struct slNode));
		t->name=tok;
		printf("'%s',",tok);
		t->next=sl;
		sl=t;
		tok=strtok(NULL,delim);
		}
	printf("],'tknames':[");

	char **row, *query;
	for(t=sl; t!=NULL; t=t->next)
		{
		assert(asprintf(&query, "select tkname from %s", t->name)>0);
		MYSQL_RES *sr=mysqlGetResult(query);
		while((row=mysql_fetch_row(sr))!=NULL)
			printf("'%s',", row[0]);
		mysql_free_result(sr);
		}
	printf("]}");
	done();
    return 1;
    }


/*** fetch chromosomal sequence
     exit when done
 ***/
if(cgiVarExists("getChromseq"))
    {
	if(SQUAWK) fputs("getChromseq...\n", stderr);
	if(cgiVarExists("rid"))
		printf("'rid':%s,", cgiString("rid"));
    // return one seq fragment for each region
    char *string = cgiString("regionlst");
    char delim[] = ",";
    char *tok = strtok(string, delim);

	tabix_t *fin=tabixOpen(seqPath);
	if(fin==NULL)
		{
		printf("'abort':'Cannot open seq/tabix file'}");
		return 1;
		}

    printf("'lst':[");
	char *chr, *s;
	int start,stop;

    while(tok != NULL)
        {
		chr = tok;
		assert((tok=strtok(NULL, delim))!=NULL);
		assert((start=strMayPositiveInt(tok))!=-1);
		assert((tok=strtok(NULL, delim))!=NULL);
		assert((stop=strMayPositiveInt(tok))!=-1);
		s=getSeq(fin,chr, start, stop);
		if(s==NULL)
			{
			printf("'ERROR',");
			}
		else
			{
			printf("'%s',", s);
			free(s);
			}
		tok = strtok(NULL, delim);
		}
    printf("]}");
	done();
	ti_close(fin);
    return 1;
    }


/* for a list of genes
genetrack list must be ready
*/
if(cgiVarExists("getdesc4genelst"))
	{
	char delim[]=",";
	char *tok= strtok(cgiString("lst"), delim);
	char *query,*def;
	struct slNode *table;
	printf("'desc':{");
    while(tok!= NULL)
        {
		def=NULL;
		for(table=genetrack_sl; table!=NULL; table=table->next)
			{
			assert(asprintf(&query, "select description from %ssymbol where name=\"%s\" or symbol=\"%s\"", table->name, tok, tok)>0);
			def=mysqlGetOnestring(query);
			free(query);
			if(def != NULL)
				break;
			}
		if(def!=NULL)
			printf("'%s':[\"%s\",'%s'],", tok, def, table->name);
		tok=strtok(NULL, delim);
		}
    printf("}}");
	done();
    return 1;
	}








/* GP fetch data for gene plot
   exit when done
 */
if(cgiVarExists("makegeneplot"))
    {
	// 0: gsv, 2: standalone
	char gpcontext=cgiString("context")[0];

    /***************************
       Step 1
       parse items from a number of possible sources
       any valid input will be taken
     ***************************/
    struct nnode *itemsl = NULL, *item;

	if(gpcontext=='0')
		{
		struct displayedRegion dsp;
		dsp.head=dsp.tail=NULL;
		if(!dspRestoreByCleanItemlist(&dsp, cgiString("session"), cgiInt("statusid")))
			{
			printf("'gpabort':1,'alert':'cannot recover gene set view item list'}");
			return 1;
			}
		struct region *r;
		for(r=dsp.head; r!=NULL; r=r->next)
			{
			item=malloc(sizeof(struct nnode));
			item->name=r->name;
			item->chrIdx=r->chromIdx;
			item->start=r->bstart;
			item->stop=r->bstop;
			item->strand=r->strand;
			item->next=itemsl;
			itemsl=item;
			}
		}
    else if(cgiVarExists("lst"))
		{
		// positional data: itemname, chr, start, stop, joined by comma
		char delim[] = ",";
		char *tok = strtok(cgiString("lst"), delim);
		while(tok != NULL)
			{
			item = malloc(sizeof(struct nnode));
			item->name = tok;
			assert((tok=strtok(NULL, delim))!=NULL);
			assert((item->chrIdx=strMayChrID(tok))!=-1);
			assert((tok=strtok(NULL, delim))!=NULL);
			assert((item->start=strMayPositiveInt(tok))!=-1);
			assert((tok=strtok(NULL, delim))!=NULL);
			assert((item->stop=strMayPositiveInt(tok))!=-1);
			item->strand = strMayGetgeneStrand(genetrack_sl, item->name);
			if(item->strand == '.')
				item->strand = '+';
			item->next = itemsl;
			itemsl = item;
			tok = strtok(NULL, delim);
			}
		}
    else if(cgiVarExists("tainput"))
        {
		// input from textarea, standalone mode
		char *genestruct = cgiString("gss_opt");
		char *origintype = cgiString("gss_origin");
		int leftlength = cgiInt("gss_up");
		int rightlength = cgiInt("gss_down");
		// dissect long string into list of items using comma
		struct slNode *tmpsl=NULL, *tmptt=NULL;
		char delim[] = ",";
		char *tok = strtok(cgiString("tainput"), delim);
		while(tok != NULL)
			{
			tmptt = malloc(sizeof(struct slNode));
			tmptt->name = tok;
			tmptt->next = tmpsl;
			tmpsl = tmptt;
			tok = strtok(NULL, delim);
			}
		if(tmpsl == NULL)
			{
			printf("'abort':'no recognizable items'}");
			fputs("makegeneplot: got nothing from dissecting tainput", stderr);
			exit(0);
			}

		if(0)
			{
			for(tmptt=tmpsl; tmptt!=NULL; tmptt=tmptt->next)
				fprintf(stderr, "raw item: %s\n", tmptt->name);
			}

		// make regions from each input item, slightly duplicating code as genesetview
		struct geneParam *prm=initGeneParam();
		prm->whichgenepart=genestruct;
		prm->origintype=origintype;
		prm->flank5=leftlength;
		prm->flank3=rightlength;
		boolean intervalonly=cgiVarExists("intervalonly");
		for(tmptt=tmpsl; tmptt!=NULL; tmptt=tmptt->next)
			{
			prm->query_str=tmptt->name;
			if(intervalonly)
				{
				int *arr = dissectCoordString(tmptt->name);
				if(arr==NULL || arr[0]==-1 || arr[1]==-1 || arr[2]==-1) 
					continue;
				item = malloc(sizeof(struct nnode));
				item->chrIdx = arr[0];
				if(asprintf(&item->name, "%s:%d-%d", chrInfo[arr[0]]->name, arr[1], arr[2])<0) errabort("bae");
				if(strcmp(genestruct, "custom")==0)
					{
					// add flanking
					if(arr[3] == 1)
						{
						item->start = arr[1]>=leftlength ? arr[1]-leftlength : 0;
						item->stop = arr[2]+rightlength>=chrInfo[arr[0]]->length ? chrInfo[arr[0]]->length : arr[2]+rightlength;
						}
					else
						{
						item->start = arr[1]>=rightlength ? arr[1]-rightlength : 0;
						item->stop = arr[2]+leftlength>=chrInfo[arr[0]]->length ? chrInfo[arr[0]]->length : arr[2]+leftlength;
						}
					}
				else
					{
					item->start = arr[1];
					item->stop = arr[2];
					}
				item->strand = arr[3]==1 ? '+' : '-';
				item->next = itemsl;
				itemsl = item;
				}
			else
				{
				struct gene *sl = queryGene(prm);
				if(sl != NULL)
					{
					/* find one or many genes
					automatically choose one that's on 'normal' chrs but not on a contig
					*/
					struct gene *_g;
					for(_g=sl; _g!=NULL; _g=_g->next)
						{
						if(_g->on_normalchr) break;
						}
					if(_g==NULL) _g=sl;
					item = malloc(sizeof(struct nnode));
					item->chrIdx = _g->chromIdx;
					item->name = tmptt->name;
					item->start = _g->report_start;
					item->stop = _g->report_stop;
					item->strand = _g->strand;
					item->next = itemsl;
					itemsl = item;
					slFree(sl);
					}
				else
					{
					// not a gene
					int *arr = dissectCoordString(tmptt->name);
					if(arr==NULL || arr[0]==-1 || arr[1]==-1 || arr[2]==-1) 
						continue;
					item = malloc(sizeof(struct nnode));
					item->chrIdx = arr[0];
					if(asprintf(&item->name, "%s:%d-%d", chrInfo[arr[0]]->name, arr[1], arr[2])<0) errabort("bae");
					if(strcmp(genestruct, "custom")==0)
						{
						// add flanking
						if(arr[3] == 1)
							{
							item->start = arr[1]>=leftlength ? arr[1]-leftlength : 0;
							item->stop = arr[2]+rightlength>=chrInfo[arr[0]]->length ? chrInfo[arr[0]]->length : arr[2]+rightlength;
							}
						else
							{
							item->start = arr[1]>=rightlength ? arr[1]-rightlength : 0;
							item->stop = arr[2]+leftlength>=chrInfo[arr[0]]->length ? chrInfo[arr[0]]->length : arr[2]+leftlength;
							}
						}
					else
						{
						item->start = arr[1];
						item->stop = arr[2];
						}
					item->strand = arr[3]==1 ? '+' : '-';
					item->next = itemsl;
					itemsl = item;
					}
				}
			}
		}
    else if(cgiVarExists("forbedtk"))
		{
		// items from tabix bed file
		// limit items to be used by this Max number
		int maxitemnum = MaxBigbeditem4geneplot;
		char *tmp=cgiString("forbedtk");
		int ft=cgiInt("bedtkft");
		char *urlpath=makefilepath(tmp,ft);
		tabix_t *fin=tabixOpen(urlpath);
		if(fin==NULL)
			{
			printf("'abort':'invalid tabix file'}");
			return 1;
			}
		int chrIdx;
		struct beditem *btmsl, *btm;
		char delim[]="\t";
		char *tok;
		for(chrIdx=0; chrIdx<lastChrIdx; chrIdx++)
			{
			if(maxitemnum <= 0)
				break;
			btmsl=tabixQuery_regioncoord(fin, FT_bed_c, chrInfo[chrIdx]->name, 0, chrInfo[chrIdx]->length);
			if(btmsl == NULL)
				continue;
			// no need to sort btmsl, no need to cache, exit when done
			for(btm=btmsl; btm!=NULL; btm=btm->next)
				{
				item = malloc(sizeof(struct nnode));
				if(item==NULL)
					{
					printf("'abort':'Out of memory'}");
					return 1;
					}
				item->chrIdx = chrIdx;
				item->start = btm->start;
				item->stop = btm->stop;
				if(btm->rest == NULL)
					{
					assert(asprintf(&item->name, "%s:%d-%d", chrInfo[chrIdx]->name, btm->start, btm->stop)>0);
					item->strand='+';
					}
				else
					{
					tok = strtok(btm->rest, delim); // name
					if(tok == NULL)
						{
						assert(asprintf(&item->name, "%s:%d-%d", chrInfo[chrIdx]->name, btm->start, btm->stop)>0);
						item->strand='+';
						}
					else
						{
						item->name = tok;
						strtok(NULL, delim); // id
						tok = strtok(NULL, delim); // strand
						if(tok != NULL)
							{
							if(tok[0]!='+' && tok[0]!='-')
								item->strand='+';
							else
								item->strand=tok[0];
							}
						else
							item->strand='+';
						}
					}
				item->next = itemsl;
				itemsl = item;
				maxitemnum--;
				}
			}
		ti_close(fin);
		}

    if(itemsl == NULL)
        {
		printf("'abort':'Gene Plot got no valid items'}");
		return 1;
		}

    if(0)
        {
        fputs("testing>>>\n", stderr);
        for(item=itemsl; item!=NULL; item=item->next)
            fprintf(stderr, "\t%s %d %d %d %c\n", item->name, item->chrIdx, item->start, item->stop, item->strand);
        fputs("done<<<\n", stderr);
		}


    /******************************
       Step 2
       retrieve data and store in memory
     ******************************/
	char *tmp=cgiString("datatkft");
	int ft=strMayPositiveInt(tmp);
	if(ft==-1)
		{
		printf("'alert':'wrong data track filetype','gpabort':1}");
		return 1;
		}
    char *urlpath=makefilepath(cgiString("datatk"),ft);
	tabix_t *fin=NULL;
	// different querying methods for bedgraph/tabix, categorical/tabix, and bigwig
	boolean isTabix=(ft==FT_bedgraph_c||ft==FT_bedgraph_n||ft==FT_qdecor_n);
	if(isTabix)
		fin=tabixOpen(urlpath);
    int spnum = cgiInt("spnum");
    int i;
    char *plottype = cgiString("plottype");
    boolean is_s1 = strcmp(plottype,"s1")==0;
    boolean is_s2 = strcmp(plottype,"s2")==0;
    boolean is_s3 = strcmp(plottype,"s3")==0;
    boolean is_s4 = strcmp(plottype,"s4")==0;
    if(is_s1 || is_s2 || is_s4)
        {
		/* these types will take the coordinates as provided, everything, no matter gene or coord
		   make single vector for each item
		 */
		for(item=itemsl; item!=NULL; item=item->next)
			{
			double *data = malloc(sizeof(double) * spnum);
			for(i=0; i<spnum; i++) data[i] = 0;
			if(isTabix)
				tabixQuery_bedgraph(fin, chrInfo[item->chrIdx]->name, item->start, item->stop, spnum, data);
			else
				bigwigQuery(urlpath,chrInfo[item->chrIdx]->name, item->start, item->stop, spnum, data);
			item->data = data;
			}
		}
    else if(is_s3)
        {
		/* 5 vectors for the 5 segment types, only process genes
		   5 vectors concatenated into one, follow the order:
			  promoter, utr5, exons, introns, utr3

		   for genes missing certain parts, the data value corresponding to that part
		   is indicated by LONG_MAX, and later during R code output,
		   this value will be detected and replaced by NA
		   so the summary value won't be affected
		   I just assume using LONG_MAX should be secure....

		   revise this part of code as gene querying is called twice
		 */
		double tmpdata[spnum], // store single vector
			   sumdata[spnum]; // store sum for exons/introns
		int validgenenum = 0;
		struct geneParam *geneparam=initGeneParam();
		for(item=itemsl; item!=NULL; item=item->next)
			{
			geneparam->query_str=item->name;
			struct gene *_sl = queryGene(geneparam);
			if(_sl== NULL)
				{
				item->data = NULL;
				continue;
				}
			struct gene *thisgene;
			for(thisgene=_sl; thisgene!=NULL; thisgene=thisgene->next)
				{
				if(thisgene->on_normalchr) break;
				}
			if(thisgene==NULL) thisgene=_sl;

			validgenenum++;
			double *data = malloc(sizeof(double) * spnum * 5); // entire data vector containing sub vectors
			unsigned int regionstart, regionstop;
			int dataIdx=0;

			/*** promoter ***/
			for(i=0; i<spnum; i++)
				tmpdata[i] = 0;
			if(isTabix)
				tabixQuery_bedgraph(fin, chrInfo[item->chrIdx]->name, thisgene->report_start, thisgene->report_stop, spnum, &tmpdata[0]);
			else
				bigwigQuery(urlpath,chrInfo[item->chrIdx]->name, thisgene->report_start, thisgene->report_stop, spnum, &tmpdata[0]);
			if(item->strand == '-')
				{
				for(i=0; i<spnum; i++)
					data[dataIdx+i] = tmpdata[spnum-1-i];
				}
			else
				{
				for(i=0; i<spnum; i++)
					data[dataIdx+i] = tmpdata[i];
				}
			dataIdx += spnum;

			/*** 5' utr ***/
			if(thisgene->utr5width<=0)
				{
				// no 5' utr, use special value to indicate
				for(i=0; i<spnum; i++)
					data[dataIdx+i] = LONG_MAX;
				}
			else
				{
				regionstart=thisgene->utr5start;
				regionstop=thisgene->utr5start+thisgene->utr5width;
				for(i=0; i<spnum; i++)
					tmpdata[i] = 0;
				if(isTabix)
					tabixQuery_bedgraph(fin, chrInfo[item->chrIdx]->name, regionstart, regionstop, spnum, &tmpdata[0]);
				else
					bigwigQuery(urlpath, chrInfo[item->chrIdx]->name, regionstart, regionstop, spnum, &tmpdata[0]);
				if(item->strand == '-')
					{
					for(i=0; i<spnum; i++)
						data[dataIdx+i] = tmpdata[spnum-1-i];
					}
				else
					{
					for(i=0; i<spnum; i++)
						data[dataIdx+i] = tmpdata[i];
					}
				}
			dataIdx += spnum;

			/*** exons (average vector is computed for each gene) ***/
			if(thisgene->exoncount<=0)
				{
				// no exon
				for(i=0; i<spnum; i++)
					data[dataIdx+i] = LONG_MAX;
				}
			else
				{
				for(i=0; i<spnum; i++) 
					sumdata[i]=0;
				int exon_use_count= 0;
				int exonidx=0;
				for(; exonidx<thisgene->exoncount; exonidx++) {
					regionstart=(thisgene->exons)[exonidx][0];
					regionstop=(thisgene->exons)[exonidx][1];
					if(regionstart>=regionstop)
						continue;
					for(i=0; i<spnum; i++) 
						tmpdata[i]=0;
					if(isTabix)
						tabixQuery_bedgraph(fin, chrInfo[item->chrIdx]->name, regionstart, regionstop, spnum, &tmpdata[0]);
					else
						bigwigQuery(urlpath, chrInfo[item->chrIdx]->name, regionstart, regionstop, spnum, &tmpdata[0]);
					for(i=0; i<spnum; i++)
						sumdata[i] += tmpdata[i];
					exon_use_count++;
					}
				if(item->strand == '-')
					{
					for(i=0; i<spnum; i++)
						data[dataIdx+i] = sumdata[spnum-1-i] / exon_use_count;
					}
				else
					{
					for(i=0; i<spnum; i++)
						data[dataIdx+i] = sumdata[i] / exon_use_count;
					}
				}
			dataIdx += spnum;
			/*** introns (average vector is computed) ***/
			if(thisgene->exoncount<=1)
				{
				// no intron
				for(i=0; i<spnum; i++)
					data[dataIdx+i] = LONG_MAX;
				}
			else
				{
				for(i=0; i<spnum; i++) 
					sumdata[i]=0;
				int intronidx=0;
				int intron_use_count=0;
				for(; intronidx<thisgene->exoncount-1; intronidx++)
					{
					regionstart=(thisgene->exons)[intronidx][1];
					regionstop=(thisgene->exons)[intronidx+1][0];
					if(regionstart>=regionstop)
						continue;
					for(i=0; i<spnum; i++) 
						tmpdata[i] = 0;
					if(isTabix)
						{
						tabixQuery_bedgraph(fin, chrInfo[item->chrIdx]->name, regionstart, regionstop, spnum, &tmpdata[0]);
						}
					else
						{
						bigwigQuery(urlpath, chrInfo[item->chrIdx]->name, regionstart, regionstop, spnum, &tmpdata[0]);
						}
					for(i=0; i<spnum; i++)
						sumdata[i] += tmpdata[i];
					intron_use_count++;
					}
				if(item->strand == '-')
					{
					for(i=0; i<spnum; i++)
						data[dataIdx+i] = sumdata[spnum-1-i] / intron_use_count;
					}
				else
					{
					for(i=0; i<spnum; i++)
						data[dataIdx+i] = sumdata[i] / intron_use_count;
					}
				}
			dataIdx += spnum;
			/*** 3' utr ***/
			if(thisgene->utr3width<=0)
				{
				// no 3' utr
				for(i=0; i<spnum; i++)
					data[dataIdx+i] = LONG_MAX;
				}
			else
				{
				for(i=0; i<spnum; i++) 
					tmpdata[i] = 0;
				regionstart=thisgene->utr3start;
				regionstop=thisgene->utr3start+thisgene->utr3width;
				if(isTabix)
					tabixQuery_bedgraph(fin, chrInfo[item->chrIdx]->name, regionstart,regionstop, spnum, &tmpdata[0]);
				else
					bigwigQuery(urlpath, chrInfo[item->chrIdx]->name, regionstart,regionstop, spnum, &tmpdata[0]);
				if(item->strand == '-')
					{
					for(i=0; i<spnum; i++)
						data[dataIdx+i] = tmpdata[spnum-1-i];
					}
				else
					{
					for(i=0; i<spnum; i++)
						data[dataIdx+i] = tmpdata[i];
					}
				}
			item->data = data;
			}
		if(validgenenum == 0)
			{
			// no valid gene to make s3 plot, need to abort
			if(fin!=NULL) ti_close(fin);
			printf("'alert':'no valid gene, please check your input','gpabort':1}");
			exit(0);
			}
		}
	if(fin!=NULL) ti_close(fin);

    /* Step 2.1
       make safe name for clustering
       this will be used in returning json data
     */
    struct slNode *clustersafenamesl = NULL, *namenode;
    if(is_s4)
        {
		i = 0;
		for(item=itemsl; item!=NULL; item=item->next)
			{
			namenode = malloc(sizeof(struct slNode));
			assert(asprintf(&(namenode->name), "g%d", i++)>0);
			namenode->next = clustersafenamesl;
			clustersafenamesl = namenode;
			}
		slReverse(&clustersafenamesl);
		}

    /* Step 2.2
       prepare to write data to temperary files
       - R rendering
       - clustering
     */
    boolean usingR = cgiVarExists("usingR"); // now this only means using R for rendering
    if(is_s4)
        usingR = FALSE;
    char *rfile; // a single file to hold R command
    char *pngfile;
    char *routfile; // dummy out file when calling R in batch mode
    char *rtmpfile; // temp file to store some R output (e.g. summary values)
    FILE *fout = NULL;
    if(is_s1 || is_s3 || is_s4 || usingR)
        {
		if(asprintf(&rfile, "%s/%s.geneplot", trashDir, session)<0) errabort("bae");
		fout = fopen(rfile, "w");
		if(fout == NULL)
			errabort("cannot open file for writing geneplot code");
		}
    if(asprintf(&routfile, "%s/%s.rout", trashDir, session)<0) errabort("bae");
    if(is_s1 || is_s3 || usingR)
        {
		// R code will be writen when using R for rendering or computing summary for s1/s3
		srand(time(0));
		if(asprintf(&pngfile, "%s/%d.png", WWWT, rand())<0) errabort("bae");
		if(asprintf(&rtmpfile, "%s/%s.rtmp", trashDir, session)<0) errabort("bae");
		if(usingR)
			fprintf(fout, "png(\"%s\", %s, %s)\n", pngfile, cgiString("width"), cgiString("height"));
		}


    /*********************************
       Step 3
       s2 json output of unprocessed data
       when doing R rendering, write to R source file
       finishes s2
     *********************************/
    double value;
    if(is_s2)
        {
		/* spaghetti plot, a list of vectors
		   [name string, v1, v2, v3, ...]
		 */
		printf("\"data\":[");
		if(usingR)
			fputs("mat = matrix(c(", fout);
		for(item=itemsl; item!=NULL; item=item->next)
			{
			printf("[\"%s\",", item->name);
			if(item->strand == '-')
				{ // reverse
				for(i=0; i<spnum; i++)
					{
					value = (item->data)[spnum-1-i];
					printf("%.2f,", value);
					if(usingR)
						fprintf(fout, "%.2f,", value);
					}
				}
			else
				{ // forward
				for(i=0; i<spnum; i++)
					{
					value = (item->data)[i];
					printf("%.2f,", value);
					if(usingR)
						fprintf(fout, "%.2f,", value);
					}
				}
			printf("],");
			}
		printf("],");
		if(usingR)
			{
			fseek(fout, -1, SEEK_CUR);
			fprintf(fout, "), nrow=%d)\n", spnum);
			fprintf(fout, "matplot(mat, type='l', lwd=%s, xlab=\"Genomic positions from 5' to 3'\", ylab='Data point values', xaxt='n')\ndev.off()\n", cgiString("lw"));
			fclose(fout);
			char *command;
			if(asprintf(&command, "R CMD BATCH --no-save %s %s", rfile, routfile)<0) errabort("bae");
			if(system(command) == -1)
			errabort("cannot spawn process to do clustering");
			printf("\"image\":\"%s\"", pngfile);
			}
		/*** s2 finishes ***/
		}

    /*********************************
       Step 4
       data processing/summarization for s1 and s3
       the summarization (calling summary function) has to be done in R
       summary numbers are writen to a text file
       parse it and return as json
       R rendering will be run in mean time if applicable
       this finishes up s1 and s3
     *********************************/
    if(is_s1)
        {
		// use R to summarize data
		fputs("mat = t(matrix(c(", fout);
		for(item=itemsl; item!=NULL; item=item->next)
			{
			if(item->strand == '-')
				{
				for(i=0; i<spnum; i++)
					fprintf(fout, "%.2f,", (item->data)[spnum-1-i]);
				}
			else
				{
				for(i=0; i<spnum; i++)
					fprintf(fout, "%.2f,", (item->data)[i]);
				}
			}
		fseek(fout, -1, SEEK_CUR);
		fprintf(fout, "), nrow=%d))\n", spnum);
		// no matter whether to use R to render or not, compute summary, save result in a file
		for(i=1; i<=spnum; i++)
			fprintf(fout, "write(summary(mat[,%d]), \"%s\", ncolumns=6, append=T, sep=\",\")\n", i, rtmpfile);
		if(usingR)
			{
			fprintf(fout, "boxplot(mat,outline=%s,range=%s,col=%s", cgiString("outlier"), cgiString("range"), cgiString("lc"));
			fseek(fout, -1, SEEK_CUR);
			fputs(",max=255),xlab=\"Genomic positions from 5' to 3'\",ylab=\"Data value distribution\")\n", fout);
			if(cgiVarExists("average"))
				{
				// draw average line
				fputs("lines(c(", fout);
				for(i=1; i<=spnum; i++)
					fprintf(fout, "%d,", i);
				fseek(fout, -1, SEEK_CUR);
				fputs("),c(", fout);
				for(i=1; i<=spnum; i++)
					fprintf(fout, "mean(mat[,%d],na.rm=T),", i);
				fseek(fout, -1, SEEK_CUR);
				fprintf(fout, "), lw=%s, col=%s", cgiString("lw"), cgiString("averagelc"));
				fseek(fout, -1, SEEK_CUR);
				fputs(",max=255))\n", fout);
				}
			fputs("dev.off()\n", fout);
			}
		fclose(fout);
		}
    else if(is_s3)
        {
		// won't reverse vector for genes on minus strand, this has already been done dude!
		fputs("mat = t(matrix(c(", fout);
		int all = spnum*5;
		for(item=itemsl; item!=NULL; item=item->next)
			{
			if(item->data == NULL)
				continue;
			for(i=0; i<all; i++)
				{
				float x = (item->data)[i];
				// FIXME why some values are printed out as "-nan" in maize??
				if(x==LONG_MAX || isnan(x))
					fputs("NA,", fout);
				else
					fprintf(fout, "%.2f,", (item->data)[i]);
				}
			}
		fseek(fout, -1, SEEK_CUR);
		fprintf(fout, "), nrow=%d))\n", all);
		// no matter whether to use R to render or not, compute summary, save result in a file
		for(i=1; i<=all; i++)
			fprintf(fout, "write(summary(mat[,%d])[1:6], \"%s\", ncolumns=6, append=T, sep=\",\")\n", i, rtmpfile);
		if(usingR)
			{
			fprintf(fout, "boxplot(mat,outline=%s,range=%s,xaxt='n',xlab=\"Genomic positions from 5' to 3'\",ylab=\"Data value distribution\",col=c(", cgiString("outlier"), cgiString("range"));
			char *col = cgiString("promoterc");
			for(i=0; i<spnum; i++)
				{
				fprintf(fout, "%s", col);
				fseek(fout, -1, SEEK_CUR);
				fputs(",max=255),", fout);
				}
			col = cgiString("utr5c");
			for(i=0; i<spnum; i++)
				{
				fprintf(fout, "%s", col);
				fseek(fout, -1, SEEK_CUR);
				fputs(",max=255),", fout);
				}
			col = cgiString("exonsc");
			for(i=0; i<spnum; i++)
				{
				fprintf(fout, "%s", col);
				fseek(fout, -1, SEEK_CUR);
				fputs(",max=255),", fout);
				}
			col = cgiString("intronsc");
			for(i=0; i<spnum; i++)
				{
				fprintf(fout, "%s", col);
				fseek(fout, -1, SEEK_CUR);
				fputs(",max=255),", fout);
				}
			col = cgiString("utr3c");
			for(i=0; i<spnum; i++)
				{
				fprintf(fout, "%s", col);
				fseek(fout, -1, SEEK_CUR);
				fputs(",max=255),", fout);
				}
			fseek(fout, -1, SEEK_CUR);
			fputs("))\n", fout);
			if(cgiVarExists("average"))
				{
				// draw average curve
				int j = 0;
				// promoter
				fputs("lines(c(", fout);
				for(i=0; i<spnum; i++)
					fprintf(fout, "%d,", j+i+1);
				fseek(fout, -1, SEEK_CUR);
				fputs("),c(",fout);
				for(i=0; i<spnum; i++)
					fprintf(fout, "mean(mat[,%d],na.rm=T),", j+i+1);
				fseek(fout, -1, SEEK_CUR);
				fprintf(fout, "),lw=%s,col=%s", cgiString("lw"), cgiString("average_promoterc"));
				fseek(fout, -1, SEEK_CUR);
				fputs(",max=255))\n", fout);
				j += spnum;
				// 5' utr
				fputs("lines(c(", fout);
				for(i=0; i<spnum; i++)
					fprintf(fout, "%d,", j+i+1);
				fseek(fout, -1, SEEK_CUR);
				fputs("),c(",fout);
				for(i=0; i<spnum; i++)
					fprintf(fout, "mean(mat[,%d],na.rm=T),", j+i+1);
				fseek(fout, -1, SEEK_CUR);
				fprintf(fout, "),lw=%s,col=%s", cgiString("lw"), cgiString("average_utr5c"));
				fseek(fout, -1, SEEK_CUR);
				fputs(",max=255))\n", fout);
				j += spnum;
				// exons
				fputs("lines(c(", fout);
				for(i=0; i<spnum; i++)
					fprintf(fout, "%d,", j+i+1);
				fseek(fout, -1, SEEK_CUR);
				fputs("),c(",fout);
				for(i=0; i<spnum; i++)
					fprintf(fout, "mean(mat[,%d],na.rm=T),", j+i+1);
				fseek(fout, -1, SEEK_CUR);
				fprintf(fout, "),lw=%s,col=%s", cgiString("lw"), cgiString("average_exonsc"));
				fseek(fout, -1, SEEK_CUR);
				fputs(",max=255))\n", fout);
				j += spnum;
				// introns
				fputs("lines(c(", fout);
				for(i=0; i<spnum; i++)
					fprintf(fout, "%d,", j+i+1);
				fseek(fout, -1, SEEK_CUR);
				fputs("),c(",fout);
				for(i=0; i<spnum; i++)
					fprintf(fout, "mean(mat[,%d],na.rm=T),", j+i+1);
				fseek(fout, -1, SEEK_CUR);
				fprintf(fout, "),lw=%s,col=%s", cgiString("lw"), cgiString("average_intronsc"));
				fseek(fout, -1, SEEK_CUR);
				fputs(",max=255))\n", fout);
				j += spnum;
				// 3' utr
				fputs("lines(c(", fout);
				for(i=0; i<spnum; i++)
					fprintf(fout, "%d,", j+i+1);
				fseek(fout, -1, SEEK_CUR);
				fputs("),c(",fout);
				for(i=0; i<spnum; i++)
					fprintf(fout, "mean(mat[,%d],na.rm=T),", j+i+1);
				fseek(fout, -1, SEEK_CUR);
				fprintf(fout, "),lw=%s,col=%s", cgiString("lw"), cgiString("average_utr3c"));
				fseek(fout, -1, SEEK_CUR);
				fputs(",max=255))\n", fout);
				}
			fprintf(fout, "axis(side=1, at=c(%d,%d,%d,%d,%d), labels=c(\"promoter\",\"5' UTR\",\"exons\",\"introns\",\"3' UTR\"))\n",spnum/2,spnum+spnum/2,spnum*2+spnum/2,spnum*3+spnum/2,spnum*4+spnum/2);
			fputs("dev.off()\n", fout);
			}
		fclose(fout);
		}
    // common step to run R code for s1 and s3, do json return
    if(is_s1 || is_s3)
        {
		/* the summarized data will be writen to rtmpfile
		   R "write" function is appending stuff to it
		   so have to remove this file if it exists (previous geneplot in same session)
		 */
		unlink(rtmpfile);
		char *command;
		if(asprintf(&command, "R CMD BATCH --no-save %s %s", rfile, routfile)<0) errabort("bae");
		if(system(command) == -1)
			errabort("cannot spawn process to do clustering");
		/* parse rtmpfile to read summary values and return as json

		   json: list of vectors, by number of spnum
		   each vector: [min, 1 qu, median, mean, 3 qu, max]
		   there's no item name!!
		 */
		char *line = malloc(1);
		size_t s = 1;
		char delim[] = "\n";
		FILE *fin = fopen(rtmpfile, "r");
		if(fin == NULL) errabort("s1/s3 rtmpfile not found");
		printf("\"data\":[");
		while(getline(&line, &s, fin) != -1)
			{
			printf("[%s],", strtok(line, delim));
			}
		printf("],");
		fclose(fin);
		if(usingR)
			printf("\"image\":\"%s\"", pngfile);
		/*** s1 and s3 finishes ***/
		}
    

    /*********************************
       Step 5
       clustering
       in certain clustering methods:
         - vectors with zero std need to be discarded in correlation-based distance metric
	 - should not transpose matrix in correlation-based distance metric
     *********************************/
    if(is_s4)
        {
		// make a logical vector in same length as itemsl
		int totalNum=0;
		for(item=itemsl; item!=NULL; item=item->next)
			totalNum++;
		int std0flag[totalNum]; // set to 1 for zero sd
		for(i=0; i<totalNum; i++)
			std0flag[i] = 0;
		int std0num = 0; // number of vectors with 0 std
		boolean transpose = TRUE; // whether to transpose matrix at creation
		if(cgiString("clustmethod")[0] == 'h')
			{
			if(cgiString("clustdist")[0] != 'e')
				{
				/* hierarchical, not using euclidean distance but using correlation based
				   need to escape 0-std vectors
				   need not to transpose matrix
				 */
				transpose = FALSE;
				i = 0;
				for(item=itemsl; item!=NULL; item=item->next)
					{
					if(standardDeviation(item->data, spnum) == 0)
					{
					std0num++;
					std0flag[i] = 1;
					}
					i++;
					}
				}
			}
		if(std0num > 0)
			{
			printf("\"zerosdgene\":[");
			i = 0;
			for(item=itemsl; item!=NULL; item=item->next)
				{
				if(std0flag[i++])
					{
					printf("\"%s\",", item->name);
					}
				}
			printf("],");
			}

		/* return data as json hash, key: safe name, value: [real name, v1, v2, ...]
		   also write data portion of R code
		   omit 0-std ones
		 */
		printf("\"data\":{");
		fprintf(fout, "mat = %smatrix(c(", transpose?"t(":"");
		namenode=clustersafenamesl;
		int k = 0;
		for(item=itemsl; item!=NULL; item=item->next)
			{
			if(!std0flag[k])
				{
				printf("\"%s\":[\"%s\",", namenode->name, item->name);
				if(item->strand == '-')
					{
					for(i=0; i<spnum; i++)
						{
						value = (item->data)[spnum-1-i];
						printf("%.2f,", value);
						fprintf(fout, "%f,", value);
						}
					}
				else
					{
					for(i=0; i<spnum; i++)
						{
						value = (item->data)[i];
						printf("%.2f,", value);
						fprintf(fout, "%f,", value);
						}
					}
				printf("],");
				}
			k++;
			namenode = namenode->next;
			}
		printf("},");
		fseek(fout, -1, SEEK_CUR);
		fprintf(fout, "), nrow=%d)", spnum);
		if(transpose)
			fputs(")\nrownames(mat)=c(", fout);
		else
			fputs("\ncolnames(mat)=c(", fout);
		i = 0;
		for(namenode=clustersafenamesl; namenode!=NULL; namenode=namenode->next)
			{
			if(!std0flag[i++])
				fprintf(fout, "\"%s\",", namenode->name);
			}
		fseek(fout, -1, SEEK_CUR);
		fputs(")\n", fout);

		/* clustering method specific processing */
		if(cgiString("clustmethod")[0] == 'k')
			{
			/* kmeans clustering */
			char *kmnamefile;
			char *kmclusterfile;
			if(asprintf(&kmnamefile, "%s/%s.kmname", trashDir, session)<0) errabort("bae");
			if(asprintf(&kmclusterfile, "%s/%s.kmcluster", trashDir, session)<0) errabort("bae");
			fprintf(fout, "k=kmeans(mat, %s, nstart=10)\n", cgiString("kmeanscnum"));
			fprintf(fout, "write(names(k$cluster), ncol=1, '%s')\n", kmnamefile);
			fprintf(fout, "write(k$cluster, ncol=1, '%s')\n", kmclusterfile);
			fclose(fout);
			char *command;
			if(asprintf(&command, "R CMD BATCH --no-save %s %s", rfile, routfile)<0) errabort("bae");
			if(system(command) == -1)
				errabort("cannot spawn process to do clustering");
			// return clustering result
			char *line = malloc(1); size_t s = 1;
			char delim[] = "\n";

			FILE *fin = fopen(kmnamefile, "r");
			if(fin == NULL) errabort("kmnamefile not found");
				printf("\"kmnames\":[");
			while(getline(&line, &s, fin) != -1)
				printf("\"%s\",", strtok(line, delim));
			printf("],");
			fclose(fin);
			fin = fopen(kmclusterfile, "r");
			if(fin == NULL) errabort("kmclusterfile not found");
				printf("\"kmnamecluster\":[");
			while(getline(&line, &s, fin) != -1)
				printf("%s,", strtok(line, delim));
			printf("]");
			fclose(fin);
			/*** kmeans done ***/
			}
		else
			{
			/* hclust: hierarchical clustering */
			char *hclustmergefile; // store hc$merge
			char *hclustlabelsfile; // store hc$labels
			char *hclustoutfile; // store hc$label[hc$order]
			char *hclustheightfile; // store hc$height
			if(asprintf(&hclustmergefile, "%s/%s.hclustmerge", trashDir, session)<0) errabort("bae");
			if(asprintf(&hclustoutfile, "%s/%s.hclustout", trashDir, session)<0) errabort("bae");
			if(asprintf(&hclustlabelsfile, "%s/%s.hclustlabels", trashDir, session)<0) errabort("bae");
			if(asprintf(&hclustheightfile, "%s/%s.hclustheight", trashDir, session)<0) errabort("bae");
			char dist = cgiString("clustdist")[0];
			char *agglomeration = cgiString("clustaglm");
			if(dist == 'e')
				{
				/* using euclidean distance, no filtering on vectors */
				fprintf(fout, "hc = hclust(dist(mat, method='e'), '%s')\n", agglomeration);
				}
			else
				{
				/* using spearman or pearson as distance metric, excluding 0-std vectors */
				fprintf(fout, "hc = hclust(as.dist(1-abs(cor(mat, method='%c'))), '%s')\n", dist, agglomeration);
				}
			fprintf(fout, "write(hc$labels[hc$order], '%s')\n", hclustoutfile);
			// write out hclust$merge, two children for each cluster, which number is validnum-1
			fprintf(fout, "write(hc$merge[1,], ncolumns=2, sep=',', '%s')\n", hclustmergefile);
			for(i=2; i<totalNum-std0num; i++)
				fprintf(fout, "write(hc$merge[%d,],ncolumns=2, append=T, sep=',', '%s')\n", i, hclustmergefile);
			fprintf(fout, "write(hc$labels, ncolumns=1, '%s')\n", hclustlabelsfile);
			fprintf(fout, "write(hc$height, ncolumns=1, '%s')\n", hclustheightfile);
			fclose(fout);

			char *command;
			if(asprintf(&command, "R CMD BATCH --no-save %s %s", rfile, routfile)<0) errabort("bae");
			if(system(command) == -1)
				errabort("cannot spawn process to do clustering");

			char *line = malloc(1); size_t s = 1;
			char delim[] = "\n";

			// return hc$labels, R's stubborn list of items
			FILE *fin = fopen(hclustlabelsfile, "r");
			if(fin == NULL) errabort("hclustlabelsfile not found");
			printf("\"hclustlabels\":[");
			while(getline(&line, &s, fin) != -1)
				printf("\"%s\",", strtok(line, delim));
			printf("],");
			fclose(fin);
			// return hc$labels[hc$order] the order of appearance of gene safe names
			fin = fopen(hclustoutfile, "r");
			if(fin == NULL) errabort("hclustoutfile not found");
				printf("\"hclustgenelst\":[");
			while(getline(&line, &s, fin) != -1)
				printf("\"%s\",", strtok(line, delim));
			printf("],");
			fclose(fin);
			// return hc$merge
			fin = fopen(hclustmergefile, "r");
			if(fin == NULL) errabort("hclustmergefile not found");
			printf("\"hclustmerge\":[");
			while(getline(&line, &s, fin) != -1)
				printf("[%s],", strtok(line, delim));
			printf("],");
			fclose(fin);

			// return hc$height
			fin = fopen(hclustheightfile, "r");
			if(fin == NULL) errabort("hclustheightfile not found");
			printf("\"hclustheight\":[");
			while(getline(&line, &s, fin) != -1)
				printf("%s,", strtok(line, delim));
			printf("],");
			fclose(fin);
			/*** hierarchical clustering done ***/
			}
		}

    printf("}");
	done();
    return 1;
    }


// TRI triggers
boolean trigger_changeGF = cgiVarExists("changeGF");
boolean trigger_zoom = cgiVarExists("zoom");
boolean trigger_move = cgiVarExists("move");
boolean trigger_imgAreaSelect = cgiVarExists("imgAreaSelect"); // defunc, relic...
boolean trigger_jump = cgiVarExists("jump");
boolean trigger_addtracks= cgiVarExists("addtracks");
//boolean trigger_correlation = cgiVarExists("correlation");
boolean trigger_htest = cgiVarExists("htest");
trigger_getdata = cgiVarExists("getdata"); // this is global variable...
boolean trigger_restoreStatus = cgiVarExists("restoreStatus");
boolean trigger_genesetview = cgiVarExists("itemlist"); // running in gene set view mode
boolean trigger_newitemlist = FALSE;
boolean trigger_updateitemlist = FALSE;
boolean trigger_scaffoldUpdate = cgiVarExists("scaffoldUpdate");
/* adding arbitrary types of custom tracks
so that it will fetch hmtk data */



struct displayedRegion dsp;
dsp.head = NULL;
dsp.tail = NULL;
dsp.chritemsl=NULL;
dsp.atbplevel = FALSE;
dsp.usedSummaryNumber = 0;
dsp.runmode = RM_genome;
dsp.juxtkft=-1;
dsp.juxtkurlpath=NULL;
dsp.tabix=NULL;

struct heatmap hm;
hm.dsp = &dsp;
hm.trackSl = NULL;
hm.datasetSl = NULL;
hm.decor0 = NULL;
hm.decor1 = NULL;
hm.decor8 = NULL;
hm.decor4 = NULL;
hm.decor5 = NULL;
hm.decor9 = NULL;
hm.decor10 = NULL;
hm.decor17=NULL;
hm.decor18=NULL;
hm.genetrack_sl = genetrack_sl;

hmSpan = cgiInt("hmspan");

    /* greets user
    char *greetings[] = {"Howdy!", "Alo...ha!", "How are you doing today?", 
        "Lieutenant Kerrigan reporting.", "Let's roll!", "Greetings, honored traveler!",
	"@_@ get some sleep...",
	"<img src=images/facai.png>","<img src=images/ja1.png>",
	"<img src=images/ja2.png>",
	"<img src=images/hindi1.png>", "<img src=images/korean1.png>"};
    printf("\"greeting\":\"%s\",", greetings[(int)floor(12*(float)rand()/RAND_MAX)]);
	*/


/***************************************
dsp can be determined in a number of ways, exclusive of each other:
1. gene set view
2. juxtaposition
3. restore status
4. jump
5. normal (boundaries given via CGI)

based on "normal", dsp might be altered by following triggers:
1. image area select (defunc)
2. zoom out
3. move

Operations that will change zoom level will only be provided with dsp view boundaries,
no number of summary point will be given, 
an extension from one hmSpan to 3*hmSpan will be attempted.
Such includes: 
  - zoom in/out
  - change GF
  - jump
All the other operations will be provided the entire boundary,
and total number of summary points.

TODO better arrange code for:
new submission
adding new items
sorting
changing part
lstholder updating
***********************************/

if(trigger_genesetview)
	{
	if(SQUAWK) fputs("gene set view mode...\n", stderr);
	/*********************
	GSV gene set view

	case 1: raw user input
	case 2: update existing, in case of add/remove, change part, rearrange, sort
	case 3: revive from db and read-only, either session restore, or move/zoom/...
	but not rpbr beaming

	chrInfo array has to be made!
	but genes might appear in new scaffold and they will be added to chrInfo array
	**********************/
	char *session = cgiString("session");
	int statusId = cgiInt("statusId"); // won't change status id

	// custom geneset and kegg pathway are not discerned, treated the same...
	dsp.runmode = RM_gsv_c;

	trigger_newitemlist = cgiVarExists("new");
	trigger_updateitemlist = cgiVarExists("update");

	if(trigger_newitemlist || trigger_updateitemlist)
		{
		/* new-submit and update shares code in deciding exact coord of each item
		   - store in genesetRuntime as new entry
		   - generate and return new statusId
		   - store "session,statusId,itemlistId" in session (insert or update, depends on if session exists)
		   - return new border
		 */
		boolean use_custom_flanking=FALSE;

		struct geneParam *geneparam=initGeneParam();
		// query name or id will fill before each querying

		if(cgiVarExists("gss_opt"))
			{
			geneparam->whichgenepart=cgiString("gss_opt");
			//fprintf(stderr, "whichgenepart: %s\n", geneparam->whichgenepart);
			geneparam->origintype=cgiString("gss_origin");
			geneparam->flank5=cgiInt("gss_up");
			geneparam->flank3=cgiInt("gss_down");
			printf("'gssparam':{'gss_says':\"%s\",'gss_opt':\"%s\",'gss_origin':\"%s\",'gss_up':%d,'gss_down':%d},",
				cgiString("gss_says"),
				geneparam->whichgenepart,
				geneparam->origintype,
				geneparam->flank5,
				geneparam->flank3);
			use_custom_flanking=strcmp(geneparam->whichgenepart, "custom")==0;
			}

		if(trigger_newitemlist)
			{
			// numbers to report
			int redundantItemNum = 0,
				genenum=0,
				coordnum=0,
				invalidnum=0,
				validnum=0;
			// remove redundancy
			char *rawinput = cgiString("itemlist");
			//fprintf(stderr,"rawinput: %s\n",rawinput);
			char delim[] = ",\n";
			char *tok = strtok(rawinput, delim);
			struct slNode *_sl=NULL, *_t;
			while(tok != NULL)
				{
				if(slLookup(_sl, tok, 0))
					{
					redundantItemNum++; 
					}
				else
					{
					_t = malloc(sizeof(struct slNode));
					_t->name=tok;
					_t->next=_sl;
					_sl=_t;
					}
				tok = strtok(NULL, delim);
				}
			if(_sl==NULL)
				{
				printf("'abort':'no recognizable items'}");
				return 1;
				}

			slReverse(&_sl); // this is must

			// user tells if her inputs are all intervals
			boolean intervalOnly=cgiVarExists("intervalonly");

			// make one region for each valid item, skip invalid items
			for(_t=_sl; _t!=NULL; _t=_t->next)
				{
				char *cc = strdup(_t->name);
				if(!intervalOnly)
					{
					// test to see if it is gene
					geneparam->query_str=_t->name;
					struct gene *sl = queryGene(geneparam);
					if(sl != NULL)
						{
						/* now that it might match multiple genes,
						use the first one that's from a default chr but not on a contig
						*/
						struct gene *t;
						for(t=sl; t!=NULL; t=t->next)
							{
							if(t->on_normalchr) break;
							}
						if(t==NULL) t=sl;
						struct region *r = malloc(sizeof(struct region));
						r->chromIdx = t->chromIdx;
						r->bstart = r->dstart = t->report_start;
						r->bstop = r->dstop = t->report_stop;
						r->flank5start = t->flank5start;
						r->flank5stop = t->flank5stop;
						r->flank3start = t->flank3start;
						r->flank3stop = t->flank3stop;
						r->strand=t->strand;
						r->name = cc;
						r->isViewStart = FALSE;
						r->flag=TRUE;
						dspAddRegionTail(hm.dsp, r);
						genenum++;
						slFree(sl);
						validnum++;
						continue;
						}
					}
				// now treat it as coord
				int *arr = dissectCoordString(_t->name);
				if(arr==NULL || arr[0]==-1 || arr[1]==-1 || arr[2]==-1) 
					{
					invalidnum++;
					continue;
					}
				// a valid coord
				struct region *r = malloc(sizeof(struct region));
				r->chromIdx = arr[0];
				r->bstart = r->dstart = arr[1];
				r->bstop = r->dstop = arr[2];
				r->strand=arr[3]==2?'.':(arr[3]==1?'+':'-');
				// see if to add flanking
				if(use_custom_flanking)
					{
					if(r->strand=='+' || r->strand=='.')
						{
						// this item is on forward strand, or strand info is not available
						r->flank5start = max(0, r->bstart-geneparam->flank5);
						r->flank5stop = r->bstart;
						r->flank3start = r->bstop;
						r->flank3stop = min(r->bstop+geneparam->flank3,chrInfo[r->chromIdx]->length);
						r->bstart = r->dstart = r->flank5start;
						r->bstop = r->dstop = r->flank3stop;
						}
					else
						{
						// on reverse strand
						r->flank3start = max(r->bstart-geneparam->flank3,0);
						r->flank3stop = r->bstart;
						r->flank5start = r->bstop;
						r->flank5stop = min(r->bstop+geneparam->flank5,chrInfo[r->chromIdx]->length);
						r->bstart = r->dstart = r->flank3start;
						r->bstop = r->dstop = r->flank5stop;
						}
					}
				else
					{
					// no flanking
					r->flank5start=r->flank5stop=r->flank3start=r->flank3stop=-1;
					}
				// don't use original coord as name as it might contain space and that breaks bedToBigBed
				assert(asprintf(&r->name, "%s:%d-%d", chrInfo[r->chromIdx]->name, r->bstart, r->bstop)>0);

				r->isViewStart = FALSE;
				r->flag=FALSE;
				dspAddRegionTail(hm.dsp, r);
				free(arr);
				coordnum++;
				validnum++;
				}
			slFree(_sl);

			if(validnum == 0)
				{
				printf("'abort':'No recognizable items'}");
				return 0;
				}

			/*********** flow diverges here *******
			fetch data for scatter plot
			for sukn only, not for repeat browser...
			exit when done
			*/
			if(cgiVarExists("scatterplot"))
				{
				/* 1 data point for each item
				*/
				dsp.usedSummaryNumber=0;
				struct region *r;
				for(r=dsp.head; r!=NULL; r=r->next)
					{
					r->summarySize=1;
					dsp.usedSummaryNumber++;
					}
				char delim[]=",";
				char *str,*nameurl, *ftstr, *urlpath;
				int ft;
				// tracks
				str=cgiString("tk1");
				assert((nameurl=strtok(str,delim))!=NULL);
				assert((ftstr=strtok(NULL,delim))!=NULL);
				assert((ft=strMayPositiveInt(ftstr))!=-1);
				urlpath=makefilepath(nameurl,ft);
				double *data1=tabixQuery_bedgraph_dsp(&dsp,urlpath,ft);
				str=cgiString("tk2");
				assert((nameurl=strtok(str,delim))!=NULL);
				assert((ftstr=strtok(NULL,delim))!=NULL);
				assert((ft=strMayPositiveInt(ftstr))!=-1);
				urlpath=makefilepath(nameurl,ft);
				double *data2=tabixQuery_bedgraph_dsp(&dsp,urlpath,ft);
				int i=0;
				printf("'data':[");
				for(r=dsp.head; r!=NULL; r=r->next)
					{
					printf("{'name':'%s','coord':'%s:%d-%d',", r->name,chrInfo[r->chromIdx]->name,r->dstart,r->dstop);
					if(isnan(data1[i]))
						printf("'x':NaN,");
					else
						printf("'x':%f,",data1[i]);
					if(isnan(data2[i]))
						printf("'y':NaN,");
					else
						printf("'y':%f,",data2[i]);
					printf("},");
					i++;
					}
				printf("]}");
				done();
				return 1;
				}

			if(cgiVarExists("add2existing"))
				{
				/** above processed are new items
				existing items are in runtime table
				**/
				// recover old list
				struct displayedRegion olddsp;
				olddsp.head=olddsp.tail=NULL;
				if(!dspRestoreByCleanItemlist(&olddsp, session, statusId))
					{
					printf("'abort':'Failed to restore existing list, mysql corrupted?'}");
					return 1;
					}
				// find out those truly new items
				struct region *r1, *r2;
				struct displayedRegion newdsp;
				newdsp.head=newdsp.tail=NULL;
				for(r1=dsp.head; r1!=NULL; r1=r1->next)
					{
					boolean notsame=TRUE;
					for(r2=olddsp.head; r2!=NULL; r2=r2->next)
						{
						if(strcasecmp(r1->name,r2->name)==0)
							{
							notsame=FALSE;
							break;
							}
						}
					if(notsame)
						{
						dspAddRegionTail(&newdsp, gsv_duplicateregion(r1));
						}
					}
				if(newdsp.head==NULL)
					{
					printf("'abort':'Input items all exist in the current list'}");
					return 1;
					}
				// append new items to olddsp
				for(r1=newdsp.head; r1!=NULL; r1=r1->next)
					dspAddRegionTail(&olddsp, gsv_duplicateregion(r1));
				dsp.head=olddsp.head;
				dsp.tail=olddsp.tail;
				/* done adding new items */
				}

			printf("'alert':'");
			if(coordnum == 0)
				printf("%d genes added", genenum);
			else
				printf("%d genes and %d coordinates added", genenum, coordnum);
			if(invalidnum > 0)
				printf(", %d unrecognized items discarded", invalidnum);
			if(redundantItemNum > 0)
				printf(", %d redundant items skipped", redundantItemNum);
			printf("',");
			}
		else
			{
			/* update existing itemlist
			recover regionLst from db record
			TODO sorting should go here?
			*/
			if(!dspRestoreByCleanItemlist(hm.dsp, session, statusId))
				{
				printf("'abort':'Failed to query current item list'}");
				return 0;
				}
			if(cgiVarExists("lstholderupdate"))
				{
				/* updating from lstholder operation: manual reordering, removal
				*/
				struct slNode *_sl=NULL, *t;
				char delim[]=",";
				char *tok=strtok(cgiString("lstholderupdate"),delim);
				while(tok!=NULL)
					{
					t=malloc(sizeof(struct slNode));
					t->name=tok;
					t->next=_sl;
					_sl=t;
					tok=strtok(NULL,delim);
					}
				slReverse(&_sl);
				struct displayedRegion dsp2;
				struct region *r;
				dsp2.head=dsp2.tail=NULL;
				for(t=_sl; t!=NULL; t=t->next)
					{
					for(r=dsp.head; r!=NULL; r=r->next)
						{
						if(strcmp(r->name, t->name)==0)
							{
							dspAddRegionTail(&dsp2, gsv_duplicateregion(r));
							break;
							}
						}
					}
				if(dsp2.head==NULL)
					{
					printf("'abort':'Error making regions'}");
					return 1;
					}
				dsp.head=dsp2.head;
				dsp.tail=dsp2.tail;
				/* done lstholder updating */
				}
			else if(cgiVarExists("switchgenemodel"))
				{
				geneparam->query_id=cgiInt("targetid");
				geneparam->query_table=cgiString("genetype");
				char *queryname=cgiString("queryname");
				struct gene *t = queryGene(geneparam);
				assert(t!=NULL);
				struct region *r;
				for(r=dsp.head; r!=NULL; r=r->next)
					{
					if(strcasecmp(r->name,queryname)==0)
						{
						r->chromIdx = t->chromIdx;
						r->bstart = r->dstart = t->report_start;
						r->bstop = r->dstop = t->report_stop;
						r->flank5start = t->flank5start;
						r->flank5stop = t->flank5stop;
						r->flank3start = t->flank3start;
						r->flank3stop = t->flank3stop;
						break;
						}
					}
				}
			}


		/*************************************
		   flow diverges here
		   generating bigBed file request is handled here
		   exit when done
		if(cgiVarExists("generatebigbed"))
			{
			int bid = 1;
			srand(time(0));
			int rn=rand();
			char *bedfile, *sortfile;
			if(asprintf(&bedfile, "%s/%s.%d", WWWT, session, rn)<0) errabort("bae");
			if(asprintf(&sortfile, "%s/%s.%d.s", WWWT, session, rn)<0) errabort("bae");
			FILE *fout = fopen(bedfile, "w");
			if(fout == NULL)
				errabort("cannot open temp bed file");
			struct region *r;
			for(r=dsp.head; r!=NULL; r=r->next)
				fprintf(fout, "%s\t%d\t%d\t%s\t%d\t%c\n", chrInfo[r->chromIdx]->name, r->bstart, r->bstop, r->name, bid++, strMayGetgeneStrand(genetrack_sl, r->name));
			fclose(fout);

			char *command;
			if(asprintf(&command, "sort -k1,1 -k2,2n %s > %s", bedfile, sortfile)<0) errabort("bae");
			if(system(command) == -1)
				errabort("cannot do unix sort");
			unlink(bedfile);
			free(command);

			if(asprintf(&command, "%s/bgzip %s", BINdir, sortfile)<0) errabort("bae");
			if(system(command) == -1)
				errabort("cannot run bgzip");
			free(command);
			if(asprintf(&command, "%s/tabix -p bed %s.gz", BINdir, sortfile)<0) errabort("bae");
			if(system(command) == -1)
				errabort("cannot run tabix");
			free(command);

			printf("'rand':%d,'itemnum':%d}", rn,bid-1);
			done();
			return 1;
			}
		*************************************/


		/* itemlist sorting functions goes here 
		TODO revise the sorting code with nsortItem
		but it won't work with coordinate sort
		remove coordinate sort?
		*/
		if(cgiVarExists("sortby"))
			{
			int itemtotalcount=0;
			struct region *r;
			for(r=dsp.head; r!=NULL; r=r->next) itemtotalcount++;
			char *sortby = cgiString("sortby");
			int i;
			// array of pointer doesn't work for qsort???
			struct region itemarr[itemtotalcount];
			r=dsp.head;
			for(i=0; i<itemtotalcount; i++)
				{
				itemarr[i].chromIdx = r->chromIdx;
				itemarr[i].bstart = r->bstart;
				itemarr[i].bstop = r->bstop;
				itemarr[i].dstart = r->dstart;
				itemarr[i].dstop = r->dstop;
				itemarr[i].name = strdup(r->name);
				itemarr[i].isViewStart = FALSE;
				itemarr[i].flag=r->flag;
				itemarr[i].flank5start=r->flank5start;
				itemarr[i].flank5stop=r->flank5stop;
				itemarr[i].flank3start=r->flank3start;
				itemarr[i].flank3stop=r->flank3stop;
				r = r->next;
				}

			// average score from a numerical track
			if(strcmp(sortby, "tkmeanAsc")==0 || strcmp(sortby, "tkmeanDesc")==0)
				{
				// each item got one summary point, as average signal over that region...
				dsp.usedSummaryNumber = itemtotalcount;
				r = dsp.head;
				for(i=0; i<itemtotalcount; i++)
					{
					r->summarySize = 1;
					r = r->next;
					}
				// if is custom, will be url, else, need to compose file path
				char *tmp1=cgiString("datatk");
				char *tmp2=cgiString("datatkft");
				int ft=strMayPositiveInt(tmp2);
				if(ft==-1)
					{
					printf("'abort':'wrong filetype for numerical track'}");
					fprintf(stderr, "gene set sort: wrong file type (%s)\n", tmp2);
					return 1;
					}
				if(ft!=FT_bedgraph_c && ft!=FT_bedgraph_n && ft!=FT_bigwighmtk_n && ft!=FT_bigwighmtk_c && ft!=FT_qdecor_n)
					{
					printf("'alert':'data track not of quantitative','gpabort':1}");
					return 1;
					}
				char *urlpath=makefilepath(tmp1, ft);
				double *data = tabixQuery_bedgraph_dsp(hm.dsp, urlpath, ft);
				for(i=0; i<itemtotalcount; i++)
					{
					itemarr[i].fvalue = data[i];
					}
				free(data);
				if(strcmp(sortby, "tkmeanAsc") == 0)
					qsort(itemarr, (size_t)itemtotalcount, sizeof(struct region), itemlistsort_tkmeanAsc);
				else
					qsort(itemarr, (size_t)itemtotalcount, sizeof(struct region), itemlistsort_tkmeanDesc);
				}
			else if(strcmp(sortby, "lenAsc") == 0)
				{
				qsort(itemarr, (size_t)itemtotalcount, sizeof(struct region), itemlistsort_lenAsc);
				}
			else if(strcmp(sortby, "lenDesc") == 0)
				{
				qsort(itemarr, (size_t)itemtotalcount, sizeof(struct region), itemlistsort_lenDesc);
				}
			else if(strcmp(sortby, "coord") == 0)
				{
				qsort(itemarr, (size_t)itemtotalcount, sizeof(struct region), itemlistsort_coord);
				}
			else
				{
				fputs("unrecognized itemlist sortby criteria\n", stderr);
				}
			if(0)
				{
				fputs("check sorted itemlist:\n", stderr);
				for(i=0; i<itemtotalcount; i++)
					{
					int start = itemarr[i].bstart;
					int stop = itemarr[i].bstop;
					fprintf(stderr, "%s %s %d %d, len %d\n", itemarr[i].name, chrInfo[itemarr[i].chromIdx]->name, start, stop, stop-start);
					}
				}
			r = dsp.head;
			for(i=0; i<itemtotalcount; i++)
				{
				// why so much trouble
				r->chromIdx = itemarr[i].chromIdx;
				r->dstart = itemarr[i].dstart;
				r->dstop = itemarr[i].dstop;
				r->bstart = itemarr[i].bstart;
				r->bstop = itemarr[i].bstop;
				r->name = strdup(itemarr[i].name);
				r->isViewStart = FALSE;
				r->flag=itemarr[i].flag;
				r->flank5start=itemarr[i].flank5start;
				r->flank5stop=itemarr[i].flank5stop;
				r->flank3start=itemarr[i].flank3start;
				r->flank3stop=itemarr[i].flank3stop;
				r = r->next;
				}
			// done sorting, put a trigger and inform ajaxX()
			}
		else if(cgiVarExists("changepart"))
			{
			/* changing part
			make a new dsp, new regions are added to this
			TODO currently exons/intron are disabled
			if enabled, a gene name list must be made from previous regionLst
			FIXME need to remember genetype and id of original gene model
			*/
			struct displayedRegion dsp2;
			dsp2.head=NULL;
			dsp2.tail=NULL;
			struct region *r2;
			for(r2=dsp.head; r2!=NULL; r2=r2->next)
				{
				if(r2->flag)
					{
					// is a gene
					geneparam->query_str=r2->name;
					struct gene *sl = queryGene(geneparam);
					if(sl==NULL) continue;
					// tolerate null case as some genes might miss certain parts
					struct gene *t;
					for(t=sl; t!=NULL; t=t->next)
						{
						if(t->on_normalchr) break;
						}
					if(t==NULL) t=sl;
					struct region *r = malloc(sizeof(struct region));
					r->chromIdx = t->chromIdx;
					r->bstart = r->dstart = t->report_start;
					r->bstop = r->dstop = t->report_stop;
					r->flank5start = t->flank5start;
					r->flank5stop = t->flank5stop;
					r->flank3start = t->flank3start;
					r->flank3stop = t->flank3stop;
					r->strand=t->strand;
					r->name = r2->name;
					r->isViewStart = FALSE;
					r->flag=TRUE;
					dspAddRegionTail(&dsp2, r);
					slFree(sl);
					continue;
					}
				/* is a coord
				must not use r2->bstart/bstop
				the original coord is in r2->name
				in case of previous flanking, r2->bstart is not original coord
				*/
				struct region *r = malloc(sizeof(struct region));
				r->chromIdx=r2->chromIdx;
				r->name=strdup(r2->name);
				r->strand=r2->strand;
				int *arr = dissectCoordString(r2->name);
				assert(arr!=NULL);
				assert(arr[0]!=-1);
				assert(arr[1]!=-1);
				assert(arr[2]!=-1);
				r->bstart = r->dstart = arr[1];
				r->bstop = r->dstop = arr[2];
				// see if to add flanking
				if(use_custom_flanking)
					{
					if(r->strand=='+' || r->strand=='.')
						{
						// this item is on forward strand or no strand
						r->flank5start = max(0, r->bstart-geneparam->flank5);
						r->flank5stop = r->bstart;
						r->flank3start = r->bstop;
						r->flank3stop = min(r->bstop+geneparam->flank3,chrInfo[r->chromIdx]->length);
						r->bstart = r->dstart = r->flank5start;
						r->bstop = r->dstop = r->flank3stop;
						}
					else
						{
						// on reverse strand
						r->flank3start = max(r->bstart-geneparam->flank3,0);
						r->flank3stop = r->bstart;
						r->flank5start = r->bstop;
						r->flank5stop = min(r->bstop+geneparam->flank5,chrInfo[r->chromIdx]->length);
						r->bstart = r->dstart = r->flank3start;
						r->bstop = r->dstop = r->flank5stop;
						}
					}
				else
					{
					// no flanking
					r->flank5start=r->flank5stop=r->flank3start=r->flank3stop=-1;
					}
				r->isViewStart = FALSE;
				r->flag=FALSE;
				dspAddRegionTail(&dsp2, r);
				}
			dsp.head=dsp2.head;
			dsp.tail=dsp2.tail;
			// done changing part
			}


		/* border */
		printf("'border':['%s',%d,'%s',%d],", dsp.head->name,dsp.head->bstart,dsp.tail->name,dsp.tail->bstop);
		printf("'ajaxXtrigger_gsvupdate':1,");
		struct region *r;
		printf("'entirelst':[");
		for(r=dsp.head; r!=NULL; r=r->next)
			printf("{'chrom':'%s','name':'%s','start':%d,'stop':%d,'isgene':%s,'strand':'%c'},", chrInfo[r->chromIdx]->name,r->name,r->bstart,r->bstop,r->flag?"true":"false",r->strand);
		printf("],");

		/* determine view start
		   all the rest regions has been set to FALSE
		 */
		dsp.head->isViewStart = TRUE;
		dsp.head->viewStartCoord = dsp.head->dstart;

			
		// make long string for itemlist to be stored in genesetRuntime table, very inefficient
		char *longstr=gsv_dsp_makestring(hm.dsp);
		genesetRuntime_insert(session, statusId, longstr);
		free(longstr);


		if(SQUAWK) fputs("itemlist stored into db\n", Squawk);

		computeEntireLength_itemlist(hm.dsp);
		computeSummarySize_itemlist(hm.dsp);

		printf("'gsvflankinglst':[");
		for(r=dsp.head; r!=NULL; r=r->next)
			printf("[%d,%d,%d,%d],", r->flank5start, r->flank5stop, r->flank3start, r->flank3stop);
		printf("],");

		}
	else
		{
		/*************************
		   fetch neat item list string from db using session
		   load all items as a big regionSl by order
		   fix dsp.head/tail on two ends of the SL
		   then try to determine real displayed region by sliding on this SL

		   instead of chromsome name, item names are passed as dsp args,
		   (because item names are not redundant)

		   in case of using displayedRegionParamPrecise, summary size will be given for each item
		   special args:
			- allrss
		 *************************/
		if(!dspRestoreByCleanItemlist(hm.dsp, session, statusId))
			{
			printf("\"abort\":\"sorry, gene set information could not be retrieved from database\"}");
			return 1;
			}

		if(SQUAWK) fputs("itemlist fetched from db, regionSl made...\n", Squawk);

		// so all items have been loaded, find previous displayed boundary from CGI param
		char *startItem = cgiString(StartChrom_cgi);
		int startCoord = cgiInt(StartCoord_cgi);
		char *stopItem = cgiString(StopChrom_cgi);
		int stopCoord = cgiInt(StopCoord_cgi);
		if(SQUAWK) fprintf(Squawk, "itemlist=on coord param: %s %d %s %d\n", startItem, startCoord, stopItem, stopCoord);
		trimRegionSl_itemlist(hm.dsp, startItem, startCoord, stopItem, stopCoord);

		// dsp adjustments
		if(cgiVarExists("allrss"))
			{
			if(SQUAWK) fputs("using precomputed summary size for all regions...\n", Squawk);
			/* forgot to compute dsp.usedSummaryNumber
			   2011/5/12 CSHL
			 */
			dsp.usedSummaryNumber = 0;
			dsp.tail->next = NULL;
			// assign given summary size to each item
			char delim[] = ",";
			char *allrss = cgiString("allrss");
			char *tok;
			tok = strtok(allrss, delim); assert(tok != NULL);
			// first item
			struct region *r = dsp.head;
			assert((r->summarySize=strMayPositiveInt(tok))!=-1);
			dsp.usedSummaryNumber += r->summarySize;
			for(r=r->next; r!=NULL; r=r->next) // rest of items
				{
			tok = strtok(NULL, delim); assert(tok != NULL);
			assert((r->summarySize=strMayPositiveInt(tok))!=-1);
			dsp.usedSummaryNumber += r->summarySize;
			}
			}
		else if(trigger_move)
			{
			// TODO don't need to tell from param if in basepair level??
			char direction = cgiString("move")[0];
			int mDistance = cgiInt("distance");
			int mSummarySize = cgiInt("summarySize");
			if(SQUAWK) fprintf(Squawk, "for moving (direction: %c mDist: %d mSumm: %d)...\n", direction, mDistance, mSummarySize);
			long remainingDist = 0;
			if(direction == 'l')
				{
				// newly exposed on left
				dsp.head->dstop = dsp.head->dstart;
				dsp.tail = dsp.head;
						remainingDist = moveBoundary_list_which(hm.dsp, 'h', 'l', mDistance);
				if(remainingDist != 0)
					{
					mSummarySize = (int)(mSummarySize * (1-(float)remainingDist/mDistance));
					}
				}
			else if(direction == 'r')
				{
				// newly exposed on right
				dsp.tail->dstart = dsp.tail->dstop;
				dsp.head = dsp.tail;
				remainingDist = moveBoundary_list_which(hm.dsp, 't', 'r', mDistance);
				if(remainingDist != 0)
					{
					mSummarySize = (int)(mSummarySize * (1-(float)remainingDist/mDistance));
					}
				}
			else
				{
				errabort("unknown moving direction");
				}
			dsp.head->prev = NULL;
			dsp.tail->next = NULL;

			computeEntireLength(hm.dsp);
			// not calling computeEntireLength_itemlist as need to do for entire list
			computeSummarySize(hm.dsp, mSummarySize);

			}
		else
			{
			computeEntireLength_itemlist(hm.dsp);
			if(trigger_imgAreaSelect)
				{
				// isViewStart is set in makeDspWing_itemlist()...
				makeDspWing_itemlist(hm.dsp);
				}
			else if(trigger_zoom)
				{
				if(SQUAWK) fputs("zoom out:\n", Squawk);
			double f = cgiDouble("zoom");
					moveBoundary_list_which(hm.dsp, 'h', 'l', (long)(dsp.entireLength*f));
					moveBoundary_list_which(hm.dsp, 't', 'r', (long)(dsp.entireLength*f));
				makeDspWing_itemlist(hm.dsp);
				}
			else
				{
				computeSummarySize(hm.dsp, cgiInt("sptotalnum"));
				}
			}
		if(0) 
			{
			fprintf(stderr, "usedSummaryNumber: %d\n", dsp.usedSummaryNumber);
			showRegionSl(hm.dsp);
			}
		}
	}
else if(trigger_restoreStatus)
    {
    /**********************************
             SESS restore status
     **********************************/
	if(SQUAWK) fprintf(stderr, "restoring status (%s %d)...\n", session, statusId);

    struct tnode *voidcustomtrack = NULL; // t1: track name, t2: url

    /* run mode other than genome:
    if juxtaposing geneset/pathway,
    extract itemlist by running additional sql query,
	flip trigger_newitemlist
    */
    char *query, **row;
	MYSQL_RES *sr;

    /* restore scaffold runtime before doing anything else */
    assert(asprintf(&query, "select scaffoldCount,scaffoldNames from session where session='%s' and statusId=%d", session, statusId)>0);
    sr = mysqlGetResult(query);
	assert(sr!=NULL);
    free(query);
    row = mysql_fetch_row(sr);
    assert(row[0] != NULL);
    assert(row[1] != NULL);
    char *tmp1 = strdup(row[0]); // saved for reviving scaffoldruntime later (calling mysql)
    char *tmp2 = strdup(row[1]);
    mysql_free_result(sr);

    assert(asprintf(&query, "delete from scaffoldRuntime where session='%s' and statusId=%d", session, statusId)>0);
    mysqlUpdate(query);
    free(query);
    assert(asprintf(&query, "insert into scaffoldRuntime values('%s',%d,%s,'%s')", session, statusId, tmp1, tmp2)>0);
    mysqlUpdate(query);
    free(query);
    reviveScaffold();
    free(tmp1);
	free(tmp2);

	int i;
	printf("'scaffoldOrder':[");
	for(i=0; i<=lastChrIdx; i++)
		printf("'%s',", chrInfo[i]->name);
	printf("],");

    assert(asprintf(&query, "select juxType from session where session='%s' and statusId=%d", session, statusId)>0);
	dsp.runmode=mysqlGetPositiveInteger(query);
    free(query);


	/*********** 
	all contents of saved status
	************/
    assert(asprintf(&query, "select * from session where session='%s' and statusId=%d", session, statusId)>0);
    sr = mysqlGetResult(query);
	assert(sr!=NULL);
    free(query);
    if(sr==NULL || (row=mysql_fetch_row(sr))==NULL)
        {
		printf("'abort':'Cannot restore status: session or status not found'}");
		return 0;
		}

	if(row[sid_clientctrl]!=NULL)
		printf("%s,",row[sid_clientctrl]);

    char delim[] = ",";
    char *tok;

    /* dataset */
	if(row[sid_dataset]!=NULL)
		{
    	printf("'dataset':[");
		tok = strtok(row[sid_dataset], delim);
		struct slNode *ttt;
		while(tok != NULL)
			{
			ttt = malloc(sizeof(struct slNode));
			ttt->name=strdup(tok);
			ttt->next = hm.datasetSl;
			hm.datasetSl = ttt;
			printf("'%s',", tok);
			tok = strtok(NULL, delim);
			}
		printf("],");
		}

    /* tracks on display */
	if(row[sid_bedgraph_n]!=NULL)
		parseTrackParam(row[sid_bedgraph_n], FT_bedgraph_n, &(hm.trackSl));
	if(row[sid_bedgraph_c]!=NULL)
		parseTrackParam(row[sid_bedgraph_c], FT_bedgraph_c, &(hm.trackSl));
	if(row[sid_bigwighmtk_n]!=NULL)
		parseTrackParam(row[sid_bigwighmtk_n], FT_bigwighmtk_n, &(hm.trackSl));
	if(row[sid_bigwighmtk_c]!=NULL)
		parseTrackParam(row[sid_bigwighmtk_c], FT_bigwighmtk_c, &(hm.trackSl));
	if(row[sid_cat_n]!=NULL)
		parseTrackParam(row[sid_cat_n], FT_cat_n, &(hm.trackSl));
	if(row[sid_cat_c]!=NULL)
		parseTrackParam(row[sid_cat_c], FT_cat_c, &(hm.trackSl));
	if(row[sid_bed_n]!=NULL)
		parseTrackParam(row[sid_bed_n], FT_bed_n, &(hm.decor0));
	if(row[sid_bed_c]!=NULL)
		parseTrackParam(row[sid_bed_c], FT_bed_c, &(hm.decor1));
	if(row[sid_qdecor_n]!=NULL)
		parseTrackParam(row[sid_qdecor_n], FT_qdecor_n, &(hm.decor8));
	if(row[sid_sam_n]!=NULL)
		parseTrackParam(row[sid_sam_n], FT_sam_n, &(hm.decor4));
	if(row[sid_sam_c]!=NULL)
		parseTrackParam(row[sid_sam_c], FT_sam_c, &(hm.decor5));
	if(row[sid_lr_n]!=NULL)
		parseTrackParam(row[sid_lr_n], FT_lr_n, &(hm.decor9));
	if(row[sid_lr_c]!=NULL)
		parseTrackParam(row[sid_lr_c],FT_lr_c, &(hm.decor10));
	if(row[sid_bam_n]!=NULL)
		parseTrackParam(row[sid_bam_n],FT_bam_n, &(hm.decor17));
	if(row[sid_bam_c]!=NULL)
		parseTrackParam(row[sid_bam_c],FT_bam_c, &(hm.decor18));

    /* custom track attribute 
       will be printed out later, after track data has been made & sent to client
     */
    if(row[sid_customTrackAttr] != NULL)
        {
        printf("'customTrackAttr':{%s},", row[sid_customTrackAttr]);
		}

    /* juxtaposition type */
    printf("'juxtaposition':{'type':%d,'what':\"%s\",'note':\"%s\"},", 
    	dsp.runmode, 
		row[sid_juxWhat], 
		row[sid_juxNote]);
	if(dsp.runmode==RM_jux_n)
		{
		dsp.juxtkft=FT_bed_n;
		dsp.juxtkurlpath=makefilepath(row[sid_juxWhat], dsp.juxtkft);
		assert((dsp.tabix=tabixOpen(dsp.juxtkurlpath))!=NULL);
		}
	else if(dsp.runmode==RM_jux_c)
		{
		dsp.juxtkurlpath = strdup(row[sid_juxWhat]);
		dsp.juxtkft=FT_bed_c;
		assert((dsp.tabix=tabixOpen(dsp.juxtkurlpath))!=NULL);
		}
	else
		{
		// do nothing for gsv
		}

    /* make DSP
     */
    assert((tok=strtok(row[sid_dspStat], delim))!=NULL);
    char *startChrstr = strdup(tok); 
    assert((tok=strtok(NULL, delim))!=NULL);
    int startCoord = strMayPositiveInt(tok);
	assert(startCoord!=-1);
    assert((tok=strtok(NULL, delim))!=NULL);
    char *stopChrstr = strdup(tok);
    assert((tok=strtok(NULL, delim))!=NULL);
    int stopCoord = strMayPositiveInt(tok);
	assert(stopCoord!=-1);

    /* corr
    if(row[sid_correlation] != NULL)
		{
		trigger_correlation = TRUE;
		tok = strtok(row[sid_correlation], delim); assert(tok!=NULL); // file type
		printf("\"correlation\":[%s,", tok);
		tok = strtok(NULL, delim); assert(tok!=NULL); // where
		printf("%s,", tok);
		tok = strtok(NULL, delim); assert(tok!=NULL); // name
		printf("\"%s\"],", tok);
		}
		*/

    /* custom metadata */
    if(row[sid_custmd] != NULL)
		printf("'custmd':{%s},", row[sid_custmd]);
    /* metadata category onshow */
    if(row[sid_mdconshow] != NULL)
		printf("\"mdconshow\":[%s],", row[sid_mdconshow]);
    /* pairwise comparison */
    if(row[sid_pairwisecomp] != NULL)
        printf("\"pairwisecomp\":%s,", row[sid_pairwisecomp]);
    /* decor rendering param */
    if(row[sid_decorRenderingParams] != NULL)
        printf("'decorRenderingParams':{%s},", row[sid_decorRenderingParams]);
    /* qtc, insanely long json string */
    if(row[sid_qtc] != NULL)
        printf("\"qtc\":%s,", row[sid_qtc]);
    /* gssparam, short json string */
    if(row[sid_gssparam] != NULL)
        printf("\"gssparam\":%s,", row[sid_gssparam]);
    /* sticky note */
    if(row[sid_stickynote] != NULL)
        {
		printf("'stickynote':[");
		char delim2[] = "|";
		tok = strtok(row[sid_stickynote], delim2);
		while(tok != NULL)
			{
			printf("[%s,", tok);
			tok = strtok(NULL, delim2); assert(tok!=NULL);
			printf("%s,", tok);
			tok = strtok(NULL, delim2); assert(tok!=NULL);
			printf("\"%s\"],", tok);
			tok = strtok(NULL, delim2);
			}
		printf("],");
		}

    /* gsv */
    char *itemliststring = NULL;
    trigger_newitemlist = (dsp.runmode==RM_gsv_c || dsp.runmode==RM_gsv_kegg);
    if(trigger_newitemlist)
		itemliststring = strdup(row[sid_itemlist]); // to be inserted to genesetRuntime table

    mysql_free_result(sr);


	/* can only call this outside previous mysql call */
	printJson_hmtk4datasets(hm.datasetSl);

    if(trigger_newitemlist)
        {
		trigger_genesetview = TRUE; // not used?
		genesetRuntime_insert(session, statusId, itemliststring);

		if(!dspRestoreByCleanItemlist(hm.dsp, session, statusId))
			{
			printf("'abort':'Error parsing itemlist string, please send the session ID to developer'}");
			return 0;
			}

		/* before trimming dsp
		   - print out border
		   - print out entire list for entire.itemlist (drawing minichrcanvas)
		 */
		printf("'border':['%s',%d,'%s',%d],", dsp.head->name,dsp.head->bstart,dsp.tail->name,dsp.tail->bstop);
		struct region *r;
		printf("'entireItemlist':[");
		for(r=dsp.head; r!=NULL; r=r->next)
			{
			printf("{'name':\"%s\",'chrom':'%s','start':%d,'stop':%d,'isgene':%s,'strand':'%c'},",
				r->name, chrInfo[r->chromIdx]->name,r->bstart, r->bstop,r->flag?"true":"false", r->strand);
			}
		printf("],");
		printf("'gsvflankinghash':{");
		for(r=dsp.head; r!=NULL; r=r->next)
			{
			printf("'%s':[%d,%d,%d,%d],",r->name,r->flank5start,r->flank5stop,r->flank3start,r->flank3stop);
			}
		printf("},");
		// trim region list, startChrstr/stopChrstr are item names
		trimRegionSl_itemlist(hm.dsp, startChrstr, startCoord, stopChrstr, stopCoord);
		makeDspWing_itemlist(hm.dsp);
		}
    else
        {
		/* not in gsv mode */
        int startChr = strMayChrID(startChrstr);
		assert(startChr!=-1);
        int stopChr = strMayChrID(stopChrstr);
		assert(stopChr!=-1);
        makeBorder(hm.dsp);
        printf("\"border\":[\"%s\",%d,\"%s\",%d],", chrInfo[leftBorder.chromIdx]->name, leftBorder.coord, chrInfo[rightBorder.chromIdx]->name, rightBorder.coord);
        setDspBoundary(hm.dsp, startChr, startCoord, stopChr, stopCoord);
		computeEntireLength(hm.dsp);
		if(dsp.runmode == RM_genome)
			finalizeDsp_genome(hm.dsp);
		else 
			finalizeDsp_gf(hm.dsp);
		}

    if(voidcustomtrack != NULL)
        {
		struct tnode *tmp;
		printf("\"voidcustomtrack\":[");
		for(tmp=voidcustomtrack; tmp!=NULL; tmp=tmp->next)
			printf("[\"%s\",\"%s\"],", tmp->t1, tmp->t2);
		printf("],");
		}

    if(SQUAWK) fputs("semi-restored from past status\n", Squawk);
    }
else
    {
    /*****************************************

                NORmal dsp setup

     *****************************************/
    if(SQUAWK) fputs("normal dsp...\n", stderr);
	dsp.runmode=cgiInt("runmode");
	/* FIXME a bug to be fixed
	in case of bad juxtapose tk info browser backs to default runmode from "config" table
	but not RM_genome
	*/
	if(dsp.runmode==RM_genome)
		{
		// nothing
		}
	else if(dsp.runmode==RM_jux_n)
		{
		char *tk=cgiString("juxtaposeTk");
		if(tk==NULL)
			{
			dsp.runmode=RM_genome;
			}
		else
			{
			dsp.juxtkft=FT_bed_n;
			dsp.juxtkurlpath=makefilepath(tk,dsp.juxtkft);
			dsp.tabix=tabixOpen(dsp.juxtkurlpath);
			if(dsp.tabix==NULL)
				{
				dsp.runmode=RM_genome;
				}
			}
		}
	else if(dsp.runmode==RM_jux_c)
		{
		dsp.juxtkft=FT_bed_c;
		dsp.juxtkurlpath=cgiString("juxtaposeTk"); // url
		dsp.tabix=tabixOpen(dsp.juxtkurlpath);
		if(dsp.tabix==NULL)
			{
			dsp.runmode=RM_genome;
			}
		}
	else if(dsp.runmode==RM_yearmonthday)
		{
		dsp.runmode=RM_jux_n;
		dsp.juxtkft=FT_bed_n;
		dsp.juxtkurlpath=makefilepath(ymdayfile,dsp.juxtkft);
		dsp.tabix=tabixOpen(dsp.juxtkurlpath);
		assert(dsp.tabix!=NULL);
		}
	else
		{
		fputs("Unknown runmode parameter\n", stderr);
		return 1;
		}

	if(SQUAWK) fprintf(stderr, "runmode: %d, jux tk: %s\n", dsp.runmode, dsp.juxtkurlpath==NULL?"not available":dsp.juxtkurlpath);

	if(lastChrIdx==-1)
		{
		/* scenarios
		- sukn creating browser object with default content
		*/
		defaultScaffold();
		}

	makeBorder(hm.dsp);
	/* in case of juxtaposition,
	itemsl data for entire chromosomes are fetched and cached in dsp.chritemsl for the border chrs
	*/

    /* ordinary procedure to determine DSP
       first check whether to jump
       if not, get parameter from CGI
       if not, set boundary to default location
     */

    boolean notJumped = TRUE;
    int startChr=-1, stopChr=-1, startCoord=0, stopCoord=0;
    if(trigger_jump)
		{
		// JUMP
		if(cgiVarExists("jumppos"))
			{
			/* input can be "chr:start-stop [strand]"
			*/
			//fprintf(stderr, "%s\n", cgiString("jumppos"));
			char *raw=cgiString("jumppos");
			if(SQUAWK) fprintf(stderr, "relocate to %s\n", raw);
			// no strand info
			while(TRUE)
				{
				char *raw1=strdup(raw);
				// oh dear
				int *arr = dissectCoordString(raw1);
				if(arr == NULL)
					{
					printf("\"abort\":\"invalid genomic coordinate\"}");
					return 0;
					}
				if(arr[0] == -1)
					{
					// see if it is new chr
					char *raw2=strdup(raw);
					char delim[] = " \t-:";
					char *tok=strtok(raw2,delim);
					int len=getScaffoldLength(tok);
					if(len==-1)
						{
						printf("\"abort\":\"wrong chromosome name\"}");
						return 0;
						}
					// add new chrom!!
					scaffoldAddnew(tok);
					printf("'newscaffold':['%s',%u],", tok, len);
					continue;
					}
				if(arr[1] == -1)
					{
					printf("\"abort\":\"invalid start coordinate\"}");
					return 0;
					}
				if(arr[2] == -1)
					{
					printf("\"abort\":\"invalid stop coordinate\"}");
					return 0;
					}
				if(arr[1] >= arr[2])
					{
					printf("\"abort\":\"start coordinate must precede stop coordinate\"}");
					return 0;
					}
				startChr = arr[0];
				startCoord = arr[1];
				stopCoord = arr[2];
				break;
				}
			/* at finest zoomin level, each basepair spans MAXbpwidth basepairs,
			so hmSpan will take hmSpan/MAXbpwidth basepairs */
			int L = stopCoord-startCoord;
			int maxbpnum=hmSpan/MAXbpwidth;
			if(L < maxbpnum)
				{
				if(startCoord+maxbpnum < chrInfo[startChr]->length)
					{
					stopCoord = startCoord+maxbpnum-1;
					}
				else
					{
					stopCoord = chrInfo[startChr]->length-1;
					startCoord = stopCoord-maxbpnum+1;
					}
				}
			}
		else if(cgiVarExists("jumppos2"))
			{
			// now it is in form of chr1,start1,chr2,start2
			if(SQUAWK) fprintf(stderr, "relocate to %s\n", cgiString("jumppos2"));
			char delim[] = ",";
			char *tok;
			assert((tok=strtok(cgiString("jumppos2"), delim)) != NULL);
			assert(tok != NULL);
			startChr = strMayChrID(tok);
			if(startChr == -1)
				{
				/* new scaffold sequence, insert into chrInfo */
				scaffoldAddnew(tok);
				startChr = lastChrIdx;
				}
			assert((tok=strtok(NULL, delim)) != NULL);
			assert((startCoord=strMayPositiveInt(tok))!=-1);
			assert((tok=strtok(NULL, delim)) != NULL);
			stopChr = strMayChrID(tok);
			if(stopChr == -1)
				{
				scaffoldAddnew(tok);
				stopChr = lastChrIdx;
				}
			assert((tok=strtok(NULL, delim)) != NULL);
			assert((stopCoord=strMayPositiveInt(tok))!=-1);
			}
		else if (cgiVarExists("jumpgene"))
			{
			/* query for only one gene struct
			- multiple hits
			  report to js and quit
			- novel scaffold sequence that the hit gene is on:
			  queryGene will append new scaffold at the end of chrInfo
			  so lastChrIdx will update
			  this change shall be reported to JS
			 */
			if(SQUAWK) fputs("relocate to a gene\n", stderr);
			int lastChrIdx_old = lastChrIdx;
			struct geneParam *param=initGeneParam();
			param->query_str=cgiString("jumpgene");
			struct gene *re=queryGene(param);
			if(re == NULL)
				{
				printf("'abort':'No genes found matching your input'}");
				return 0;
				}
			else
				{
				if(re->next==NULL)
					{
					// only one hit
					startChr = re->chromIdx;
					startCoord = re->txstart;
					stopCoord = re->txstop;
					}
				else
					{
					/* multiple hit
					flow exists here!!
					*/
					printf("'jumpgenehitlst':[");
					struct gene *g;
					for(g=re; g!=NULL; g=g->next)
						{
						printf("{'type':'%s','name':'%s','chrom':'%s','start':%d,'stop':%d,'strand':'%c','desc':\"%s\",'id':%d,'struct':",
							g->type,
							g->name,
							chrInfo[g->chromIdx]->name,
							g->txstart,
							g->txstop,
							g->strand,
							g->text,
							g->id);
						genestruct2json(g);
						printf("},");
						}
					printf("],");
					if(lastChrIdx_old != lastChrIdx)
						printf("'newscaffold':1,");
					printf("}");
					done();
					return 1;
					}
				}
			if(lastChrIdx_old != lastChrIdx)
				{
				assert(lastChrIdx == lastChrIdx_old+1);
				printf("'newscaffold':1,");
				}
			}
		notJumped = FALSE;
		setDspBoundary(hm.dsp, startChr, startCoord, stopChr==-1 ? startChr : stopChr, stopCoord);
		if(dsp.runmode == RM_genome)
			finalizeDsp_genome(hm.dsp);
		else
			finalizeDsp_gf(hm.dsp);
		}
	if(notJumped)
		{
		if(SQUAWK) fputs("not jumping...\n", Squawk);

		if(trigger_move)
			{
			if(SQUAWK) fputs("Moving...\n", Squawk);

			makeDspBoundaryFromCgi(hm.dsp);

			/******************************
			   MOVE
			   only fetch data for newly exposed region
			   make a region to start with
			   the region will covers the point where new region will be made on one of its side

			   |<<<<<<left new<<<<<<|======old======|>>>>>>right new>>>>>>|

			   if atbplevel, only use mDistance, mSummarySize is of no use then
			 ******************************/
			dsp.atbplevel = cgiVarExists("atbplevel");
			char direction = cgiString("move")[0];
			int mDistance = cgiInt("distance");
			int mSummarySize = cgiInt("summarySize");
			if(SQUAWK) fprintf(Squawk, "need to move %d (%d)\n", mDistance, mSummarySize);

			struct region *r = malloc(sizeof(struct region)); // dsp.head and dsp.tail will be set to this region
			if(dsp.runmode == RM_genome)
				{
				if(direction == 'l')
					{
					r->chromIdx = dsp.start->chromIdx;
					r->dstart = r->dstop = dsp.start->coord;
					}
				else
					{
					r->chromIdx = dsp.stop->chromIdx;
					r->dstart = r->dstop = dsp.stop->coord;
					}
				r->bstart = 0;
				r->bstop = chrInfo[r->chromIdx]->length;
				}
			else
				{
				/* juxtaposition
				form a region from left/right of initial point
				chromosome might not contain genomic feature of this kind, so use loop

				the operations dealing with left/right moving is symmetrical
				but it seems impossible to be integrated
				 */
				struct beditem *itemsl=NULL;
				struct chritemsl *ci;
				if(direction == 'l')
					{
					/* look to left, for each chr, fetch bed items from an interval,
					   interval start is always 0, stop is variable
					 */
					int lookChrIdx = dsp.start->chromIdx;
					while(itemsl == NULL)
						{
						if((ci=chritemsl_searchidx(hm.dsp, lookChrIdx))==NULL)
							{
							struct beditem *sl2=tabixQuery_regioncoord(dsp.tabix, dsp.juxtkft, chrInfo[lookChrIdx]->name, 0, chrInfo[lookChrIdx]->length);
							if(sl2!=NULL)
								{
								itemsl=(struct beditem *)beditemsort_stopDsc((struct genericItem *)sl2);
								chritemslAppend(hm.dsp, lookChrIdx, itemsl);
								}
							}
						else
							{
							assert(ci->itemsl!=NULL);
							itemsl=(struct beditem *)beditemsort_stopDsc((struct genericItem *)ci->itemsl);
							ci->itemsl=itemsl;
							}
						if(itemsl == NULL)
							{
							if(lookChrIdx==0) break;
							lookChrIdx--;
							}
						else
							{
							/* in case the itemsl is from same chr as dsp.start,
							need to check if there's any item exists on the upstream of dsp.start
							those will be used to create the region
							*/
							if(lookChrIdx==dsp.start->chromIdx)
								{
								for(; itemsl!=NULL; itemsl=itemsl->next)
									{
									if(itemsl->start < dsp.start->coord)
										break;
									}
								if(itemsl==NULL)
									{
									if(lookChrIdx==0) break;
									lookChrIdx--;
									}
								}
							}
						}
					if(itemsl==NULL)
						{
						/* cannot find anything on the upstream of dsp.start
						including all chrs on dsp.start chr's "upstream"
						must reset dsp.start at right-most item from left border chr
						and generate a region
						*/
						if(SQUAWK)
							fprintf(stderr, "! juxtaposition trying to set first region for moving to left, no items could be found from chrs before %s (dsp.start), resetting dsp.start to left border...\n", chrInfo[dsp.start->chromIdx]->name);
						ci=chritemsl_searchidx(hm.dsp, leftBorder.chromIdx);
						assert(ci!=NULL);
						assert(ci->itemsl!=NULL);
						itemsl=(struct beditem *)beditemsort_stopDsc((struct genericItem *)ci->itemsl);
						ci->itemsl=itemsl;
						dsp.start->chromIdx=dsp.stop->chromIdx=leftBorder.chromIdx;
						dsp.start->coord=dsp.stop->coord=itemsl->stop;
						lookChrIdx=leftBorder.chromIdx;
						}
					// use the first item in itemsl to form the region
					r->chromIdx = lookChrIdx;
					r->bstart = itemsl->start;
					r->bstop = itemsl->stop;
					r->dstart = r->dstop = (itemsl->stop < dsp.start->coord) ? itemsl->stop : dsp.start->coord;
					// extend this region using overlapping bed items in itemsl
					for(itemsl=itemsl->next; itemsl!=NULL; itemsl=itemsl->next)
						{
						if(itemsl->stop >= r->bstart && itemsl->start < r->bstart)
							r->bstart = itemsl->start;
						else
							break;
						}
					}
				else
					{
					/* look to right, for each chr, fetch bed items from an interval,
					   interval stop is always chromosome end, start is variable
					 */
					int lookChrIdx = dsp.stop->chromIdx;
					while(itemsl == NULL)
						{
						if((ci=chritemsl_searchidx(hm.dsp, lookChrIdx))==NULL)
							{
							struct beditem *sl2=tabixQuery_regioncoord(dsp.tabix, dsp.juxtkft, chrInfo[lookChrIdx]->name, 0, chrInfo[lookChrIdx]->length);
							if(sl2!=NULL)
								{
								itemsl=(struct beditem *)beditemsort_startAsc((struct genericItem *)sl2);
								chritemslAppend(hm.dsp, lookChrIdx, itemsl);
								}
							}
						else
							{
							assert(ci->itemsl!=NULL);
							itemsl=(struct beditem *)beditemsort_startAsc((struct genericItem *)ci->itemsl);
							ci->itemsl=itemsl;
							}
						if(itemsl==NULL)
							{
							if(lookChrIdx == lastChrIdx) 
								break;
							lookChrIdx++;
							}
						else
							{
							/* in case the itemsl is from same chr as dsp.stop,
							need to check if there's any item exists on the downstream of dsp.stop
							those will be used to create the region
							*/
							if(lookChrIdx==dsp.stop->chromIdx)
								{
								for(; itemsl!=NULL; itemsl=itemsl->next)
									{
									if(itemsl->stop > dsp.stop->coord)
										break;
									}
								if(itemsl==NULL)
									{
									if(lookChrIdx==lastChrIdx)
										break;
									lookChrIdx++;
									}
								}
							}
						}
					if(itemsl == NULL)
						{
						/* cannot find anything on the downstream of dsp.stop
						including all chrs on dsp.stop chr's "downstream"
						must reset dsp.stop at left-most item from right border chr
						and generate a region
						*/
						if(SQUAWK)
							fprintf(stderr, "! juxtaposition trying to set first region for moving to right, no items could be found from chrs after %s (dsp.stop), resetting dsp.stop to right border...\n", chrInfo[dsp.stop->chromIdx]->name);
						ci=chritemsl_searchidx(hm.dsp, rightBorder.chromIdx);
						assert(ci!=NULL);
						assert(ci->itemsl!=NULL);
						itemsl=(struct beditem *)beditemsort_startAsc((struct genericItem *)ci->itemsl);
						ci->itemsl=itemsl;
						dsp.start->chromIdx=dsp.stop->chromIdx=rightBorder.chromIdx;
						dsp.start->coord=dsp.stop->coord=itemsl->start;
						lookChrIdx=rightBorder.chromIdx;
						}
					r->chromIdx = lookChrIdx;
					r->bstart = itemsl->start;
					r->bstop = itemsl->stop;
					r->dstart = r->dstop = (itemsl->start > dsp.stop->coord) ? itemsl->start : dsp.stop->coord;
					for(itemsl=itemsl->next; itemsl!=NULL; itemsl=itemsl->next)
						{
						if(itemsl->start <= r->bstop && itemsl->stop > r->bstop)
							r->bstop = itemsl->stop;
						else 
							break;
						}
					}
				}
			if(SQUAWK) fprintf(Squawk, "start region determined for move: %s bstart %d bstop %d\n", chrInfo[r->chromIdx]->name, r->bstart, r->bstop);
			r->summarySize = 0;
			dspAddRegionHead(hm.dsp, r);

			double sf = (double)mDistance / mSummarySize;
			if(dsp.runmode == RM_genome)
				{
				moveBoundary_genome(hm.dsp, direction, mDistance, &mSummarySize, sf);
				}
			else
				{
				if(direction == 'l')
					moveBoundary_gf_left(hm.dsp, mDistance, mSummarySize);
				else
					moveBoundary_gf_right(hm.dsp, mDistance, mSummarySize);
				}
			if(SQUAWK) fprintf(Squawk, "dsp.head: %s:%d-%d   dsp.tail: %s:%d-%d\n\n", 
				chrInfo[dsp.head->chromIdx]->name, dsp.head->dstart, dsp.head->dstop, 
				chrInfo[dsp.tail->chromIdx]->name, dsp.tail->dstart, dsp.tail->dstop);

			computeEntireLength(hm.dsp);


			if(dsp.atbplevel)
				{
				computeSummarySize_bplevel(hm.dsp);
				}
			computeUsedSummaryNumber(hm.dsp);
			}
		else 
			{
			if(SQUAWK) fputs("not moving...\n", Squawk);
			/****************************
				not moving, possibilities:
				* zoom in (used to be image area select)
				* zoom out
				* change mode (genome <=> juxtaposition)
				* not changing dsp at all, just rebuild from CGI parameters
				* update scaffold
			 ****************************/
			if(trigger_scaffoldUpdate)
				{
				char *query;
				if(asprintf(&query, "update scaffoldRuntime set count=%s, names='%s' where session='%s' and statusId=%d", cgiString("scaffoldCount"), cgiString("scaffoldNames"), session, statusId)<0) errabort("bae");
				mysqlUpdate(query);
				free(query);
				reviveScaffold();
				trigger_imgAreaSelect = TRUE; // patch?
				}
			if(trigger_zoom || trigger_imgAreaSelect || trigger_changeGF)
				{
				makeDspBoundaryFromCgi(hm.dsp);
				if(dsp.runmode == RM_genome)
					makeRegionSl_genome(hm.dsp);
				else
					makeRegionSl_gf(hm.dsp);
				computeEntireLength(hm.dsp);
				if(trigger_zoom) 
					{
					double zoomoutPerc = cgiDouble("zoom"); // >0
					if(SQUAWK) fprintf(Squawk, "zoom out by %.2f...\n", zoomoutPerc);
					// distance to expand to each side
					long dist = (long)((dsp.entireLength * zoomoutPerc /2));
					// region after zoom out should take hmSpan summary points
					double sf = dsp.entireLength * (1+zoomoutPerc) / hmSpan;
					computeSummarySize(hm.dsp, ceil(hmSpan / (1+zoomoutPerc)));
					int spnum = hmSpan*zoomoutPerc / (1+zoomoutPerc);
					if(dsp.runmode == RM_genome)
						{
						moveBoundary_genome(hm.dsp, 'l', dist, &spnum, sf);
						spnum = hmSpan*zoomoutPerc / (1+zoomoutPerc);
						moveBoundary_genome(hm.dsp, 'r', dist, &spnum, sf);
						}
					else
						{
						moveBoundary_gf_left(hm.dsp, dist, spnum);
						moveBoundary_gf_right(hm.dsp, dist, spnum);
						}
					}
				if(dsp.runmode == RM_genome)
					finalizeDsp_genome(hm.dsp);
				else
					finalizeDsp_gf(hm.dsp);
				}
			else
				{
				/* recover dsp using params:
				   - regionLst: chr,bstart,bstop,summarysize all joined by comma
				   - startCoord: dstart of head
				   - stopCoord: dstop of tail

				   NEVER forget to compute entire length and usedSummaryNumber
				 */
				recoverDsp_cgiParam(hm.dsp);

				computeEntireLength(hm.dsp);
				if(dsp.usedSummaryNumber == dsp.entireLength)
					dsp.atbplevel = TRUE;
				}
			if(trigger_scaffoldUpdate)
				makeBorder(hm.dsp);
			}
		}
    }




/* CGI tells JS that they are at basepair level */
if(dsp.atbplevel) printf("'atbplevel':1,");

/****************
   border
   border info is also printed elsewhere...
   * first submission of itemlist
   * restoring saved status
 ****************/
if(cgiVarExists("isfirst") || trigger_changeGF || trigger_scaffoldUpdate)
    printf("'border':['%s',%d,'%s',%d],", chrInfo[leftBorder.chromIdx]->name, leftBorder.coord, chrInfo[rightBorder.chromIdx]->name, rightBorder.coord);



if(SQUAWK) showRegionSl(hm.dsp);

/* where does the view start?
   this piece of info is only needed when dsp got changed ON SERVER SIDE
   such as zoom, image area select, jump, ...
   in those cases, view start region should have been flagged out

   operations that change dsp on client side (e.g. move) will keep an updated dsp view start
 */
struct region *r;
int i = 0;
for(r=dsp.head; r!=NULL; r=r->next)
    {
    if(r->isViewStart)
        {
		printf("\"viewStart\":[%d,%d],", i, r->viewStartCoord);
		if(SQUAWK) fprintf(stderr, "viewStart: %s %d\n", chrInfo[r->chromIdx]->name, r->viewStartCoord);
		break;
		}
    i++;
    }

/* which datasets are being used?
 */
if(cgiVarExists("dataset"))
	{
	char *string = cgiString("dataset");
	// comma separated table names
	char delim[] = ",";
	char *tok = strtok(string, delim);
	while(tok != NULL)
		{
		struct slNode *tt = malloc(sizeof(struct slNode));
		tt->name=strdup(tok);
		tt->next = hm.datasetSl;
		hm.datasetSl = tt;
		}
	}


if(trigger_changeGF || trigger_zoom || trigger_move || trigger_imgAreaSelect || trigger_jump || trigger_restoreStatus || trigger_newitemlist || trigger_updateitemlist)
	{
	// return regionlst when there's dsp change
	printf("'regionLst':[");
	struct region *r;
	for(r=dsp.head; r!=NULL; r=r->next)
		{
		// skip region with 0 summarySize
		if(r->summarySize == 0)
			continue;
		printf("['%s',%d,%d,%d,%d,%d,'%s'],", 
			chrInfo[r->chromIdx]->name, 
			r->bstart, r->bstop, 
			r->dstart, r->dstop, 
			r->summarySize, 
			(cgiVarExists("itemlist")||trigger_newitemlist)?r->name:""
			);
		}
	printf("],");
	/* beware
	   if atbplevel, the r->summarySize should not be used in JS,
	   as regionLst[x][5] is supposed to be plot width (# of pixels) atbplevel
	 */
	}

/***************************************
   displayed region is now determined
 ***************************************/



boolean trigger_decor9=cgiVarExists("decor9");
boolean trigger_decor10=cgiVarExists("decor10");


if(cgiVarExists("decor0")) parseTrackParam(cgiString("decor0"), FT_bed_n, &(hm.decor0));
if(cgiVarExists("decor1")) parseTrackParam(cgiString("decor1"), FT_bed_c, &(hm.decor1));
if(cgiVarExists("decor8")) parseTrackParam(cgiString("decor8"), FT_qdecor_n, &(hm.decor8));
if(cgiVarExists("decor4")) parseTrackParam(cgiString("decor4"), FT_sam_n, &(hm.decor4));
if(cgiVarExists("decor5")) parseTrackParam(cgiString("decor5"), FT_sam_c, &(hm.decor5));
if(trigger_decor9) parseTrackParam(cgiString("decor9"), FT_lr_n, &(hm.decor9));
if(trigger_decor10) parseTrackParam(cgiString("decor10"),FT_lr_c, &(hm.decor10));
if(cgiVarExists("decor17")) parseTrackParam(cgiString("decor17"),FT_bam_n, &(hm.decor17));
if(cgiVarExists("decor18")) parseTrackParam(cgiString("decor18"),FT_bam_c, &(hm.decor18));

/**
will apply dsp filter for longrange track
only for browser display, but not for hengeview
use "hengekey" to tell if doing hengeview
**/
struct track *tt;
dsp.dspFilter=NULL;
if((trigger_decor9 || trigger_decor10) && (!cgiVarExists("hengekey")))
	{
	boolean fullitemquery=FALSE;
	if(trigger_decor9)
		{
		for(tt=hm.decor9;tt!=NULL;tt=tt->next)
			{
			if(tt->mode==M_arc||tt->mode==M_trihm) fullitemquery=TRUE;
			}
		}
	if(trigger_decor10)
		{
		for(tt=hm.decor10;tt!=NULL;tt=tt->next)
			{
			if(tt->mode==M_arc||tt->mode==M_trihm) fullitemquery=TRUE;
			}
		}
	if(fullitemquery)
		{
		// first use all current regions as filters
		struct region *r;
		struct nnode *b;
		for(r=dsp.head; r!=NULL; r=r->next)
			{
			b=malloc(sizeof(struct nnode));
			b->chrIdx=r->chromIdx;
			b->start=r->dstart;
			b->stop=r->dstop;
			b->next=dsp.dspFilter;
			dsp.dspFilter=b;
			}
		if(cgiVarExists("existingDsp"))
			{
			// this came from existing regions when panning, shall be added to filter
			char delim[]=",";
			char *tok=strtok(cgiString("existingDsp"),delim);
			while(tok!=NULL)
				{
				b=malloc(sizeof(struct nnode));
				assert((b->chrIdx=strMayChrID(tok))!=-1);
				assert((tok=strtok(NULL,delim))!=NULL);
				assert((b->start=strMayPositiveInt(tok))!=-1);
				assert((tok=strtok(NULL,delim))!=NULL);
				assert((b->stop=strMayPositiveInt(tok))!=-1);
				tok=strtok(NULL,delim);
				b->next=dsp.dspFilter;
				dsp.dspFilter=b;
				}
			}
		}
	}
	

if(hm.decor0!=NULL || hm.decor1!=NULL || hm.decor9!=NULL || hm.decor10!=NULL || hm.decor8!=NULL || hm.decor4!=NULL || hm.decor5!=NULL || hm.decor17!=NULL || hm.decor18!=NULL)
    {
    printf("'decorData':[");
    for(tt=hm.decor0; tt!=NULL; tt=tt->next)
        {
		printf("{'name':'%s', 'ft':%d, 'mode':%d,'data':", tt->name, FT_bed_n, tt->mode);
		printJsonDecor(hm.dsp, tt);
		printf("},");
		}
    for(tt=hm.decor1; tt!=NULL; tt=tt->next)
        {
		printf("{'name':'%s', 'ft':%d, 'label':'%s', 'url':'%s', 'mode':%d,'data':",
			tt->name, FT_bed_c, tt->label, tt->urlpath, tt->mode);
		printJsonDecor(hm.dsp, tt);
		printf("},");
		}
    for(tt=hm.decor9; tt!=NULL; tt=tt->next)
        {
		printf("{'name':'%s', 'ft':%d, 'mode':%d,'data':", tt->name, FT_lr_n, tt->mode);
		printJsonDecor(hm.dsp, tt);
		if(__skipped_num>0) printf("'skipped':%d", __skipped_num);
		printf("},");
		}
    for(tt=hm.decor10; tt!=NULL; tt=tt->next)
        {
		printf("{'name':'%s', 'ft':%d, 'label':'%s', 'url':'%s', 'mode':%d,'data':",
			tt->name, FT_lr_c, tt->label, tt->urlpath, tt->mode);
		printJsonDecor(hm.dsp, tt);
		if(__skipped_num>0) printf("'skipped':%d,", __skipped_num);
		printf("'pfilterscore':%f, 'nfilterscore':%f},", tt->pscore, tt->nscore);
		}
    for(tt=hm.decor8; tt!=NULL; tt=tt->next)
        {
		printf("{'name':'%s', 'ft':%d, 'mode':%d,'data':", tt->name, FT_qdecor_n, tt->mode);
		printJsonDecor(hm.dsp, tt);
		printf("},");
		}
    for(tt=hm.decor4; tt!=NULL; tt=tt->next)
        {
		printf("{'name':'%s', 'ft':%d, 'mode':%d,'data':", tt->name, FT_sam_n, tt->mode);
		printJsonDecor(hm.dsp, tt);
		printf("},");
		}
    for(tt=hm.decor5; tt!=NULL; tt=tt->next)
        {
		printf("{'name':'%s', 'ft':%d, 'label':'%s', 'url':'%s', 'mode':%d,'data':",
			tt->name, FT_sam_c, tt->label, tt->urlpath, tt->mode);
		printJsonDecor(hm.dsp, tt);
		printf("},");
		}
    for(tt=hm.decor17; tt!=NULL; tt=tt->next)
        {
		printf("{'name':'%s', 'ft':%d, 'mode':%d,'data':", tt->name, FT_bam_n, tt->mode);
		printJsonDecor(hm.dsp, tt);
		printf("},");
		}
    for(tt=hm.decor18; tt!=NULL; tt=tt->next)
        {
		printf("{'name':'%s', 'ft':%d, 'label':'%s', 'url':'%s', 'mode':%d,'data':",
			tt->name, FT_bam_c, tt->label, tt->urlpath, tt->mode);
		printJsonDecor(hm.dsp, tt);
		printf("},");
		}
    printf("],");
    }





/*** make list of hmtk ***/
if(trigger_restoreStatus)
    {
    /* trackSl has already been made from saved status,
       only need to make file path for server tracks
     */
    struct track *tk;
    for(tk=hm.trackSl; tk!=NULL; tk=tk->next)
        {
		if(tk->ft==FT_bedgraph_c||tk->ft==FT_cat_c||tk->ft==FT_bigwighmtk_c)
			continue;
		tk->urlpath=makefilepath(tk->name,tk->ft);
		}
    if(SQUAWK) 
		fputs("track sl made\n", Squawk);
    }
else if(trigger_changeGF || trigger_zoom || trigger_move || trigger_imgAreaSelect || trigger_jump || trigger_htest || trigger_addtracks || trigger_getdata || trigger_newitemlist || trigger_updateitemlist)
    {
    /* trackSl will be made for all cases except following two:
       - restore status
       - decor only
     */
    struct track *trackSl = NULL, *tk;

	if(cgiVarExists("hmtk2"))
		{
		if(cgiVarExists("rpbr_splinter"))
			{
			/******
			special case for repeatbrowser splinters
			retrieves some quantitative experimental assay tracks
			use a "hard-coded" file path
			*/
			repeatbrowser_parseExpTrack(cgiString("hmtk2"), FT_bedgraph_n, &trackSl);
			}
		else
			{
			parseTrackParam(cgiString("hmtk2"), FT_bedgraph_n, &trackSl);
			}
		}
	if(cgiVarExists("hmtk3"))
		parseTrackParam(cgiString("hmtk3"), FT_bedgraph_c, &trackSl);
	if(cgiVarExists("hmtk14"))
		parseTrackParam(cgiString("hmtk14"), FT_bigwighmtk_n, &trackSl);
	if(cgiVarExists("hmtk15"))
		parseTrackParam(cgiString("hmtk15"), FT_bigwighmtk_c, &trackSl);
	if(cgiVarExists("hmtk12"))
		parseTrackParam(cgiString("hmtk12"), FT_cat_n, &trackSl);
	if(cgiVarExists("hmtk13"))
		parseTrackParam(cgiString("hmtk13"), FT_cat_c, &trackSl);


    hm.trackSl = trackSl;

    for(tk=hm.trackSl; tk!=NULL; tk=tk->next)
        tk->gid = 0;

    if(0) showTrackSl(hm.trackSl);
    if(SQUAWK) fputs("tttrack sl made\n", stderr);
    }



/* during change server track, if htest is on, custom track will have gid
   but as track attribute id will only be calculated on client side,
   this round won't carry out htest, so switch it off after parsing param.
   after json is passed back to client, it will issue htest()
 */
if(trigger_addtracks) trigger_htest = FALSE;



if(CHECKCPUTIME)
    {
    clock_t this = clock();
    fprintf(Squawk, "Elapsed time before fetching track data: %f\n", ((double)(this-cpuTimeStart))/CLOCKS_PER_SEC);
    cpuTimeTemp = clock();
    }




/*** fetch hmtk data ***/
if(hm.trackSl!=NULL && (trigger_changeGF || trigger_zoom || trigger_move || trigger_imgAreaSelect || trigger_jump || trigger_htest || trigger_addtracks || trigger_getdata || trigger_restoreStatus || trigger_newitemlist || trigger_updateitemlist))
	{
	/* _fork_ parallelization
	   for each track, fork to fetch data and write into a file at trash dir,
	   file name: session+track name, one value at each line,
	   the parent process will wait till all children processes finish,
	   and read in that folder data of each track,
	   then remove that folder
	   tk->pid as state flag, see "data structure.dia"
	 */
	char *line = malloc(1);
	size_t s = 1;
	/* preparation */
	struct track *tk;
	for(tk=hm.trackSl; tk!=NULL; tk=tk->next)
		{
		tk->pid = -1;
		assert(asprintf(&tk->tmpfile, "%s/%s.%s.%s", trashDir, session, horcrux, tk->name)>0);
		}

	/* fetch data */
	int i;
	boolean notfinished = TRUE;
	int processRunningNum = 0;
	boolean printdata = trigger_changeGF || trigger_zoom || trigger_move || trigger_imgAreaSelect || trigger_jump || trigger_addtracks || trigger_restoreStatus || trigger_newitemlist || trigger_updateitemlist;
	if(printdata) printf("'hmtkData':[");
	while(notfinished)
		{
		notfinished = FALSE;
		for(tk=hm.trackSl; tk!=NULL; tk=tk->next)
			{
			if(tk->pid == -1)
				{
				/* this track not processed */
				if(processRunningNum == MaxProcessNum)
					{
					notfinished = TRUE;
					if(0) fputs("reaches max, wait for empty slots...\n", Squawk);
					continue;
					}
				/* fork for this track */
				pid_t p = fork();
				if(p == 0)
					{
					/*** in a child process ***/
					if(tk->ft==FT_bedgraph_c || tk->ft==FT_bedgraph_n || tk->ft==FT_bigwighmtk_n || tk->ft==FT_bigwighmtk_c)
						{
						double *data= tabixQuery_bedgraph_dsp(hm.dsp, tk->urlpath, tk->ft);
						FILE *fout = fopen(tk->tmpfile, "w");
						if(fout == NULL) errabort("cannot open file to write");
						for(i=0; i<dsp.usedSummaryNumber; i++)
							fprintf(fout, "%f\n", data[i]);
						fclose(fout);
						}
					else if(tk->ft==FT_cat_c || tk->ft==FT_cat_n)
						{
						int *data=tabixQuery_categorical_dsp(hm.dsp, tk->urlpath);
						FILE *fout = fopen(tk->tmpfile, "w");
						if(fout == NULL) errabort("cannot open file to write");
						for(i=0; i<dsp.usedSummaryNumber; i++)
							fprintf(fout, "%d\n", data[i]);
						fclose(fout);
						}
					else 
						errabort("fetching hmtk data: unknown filetype");
					_exit(0); // exit without flushing buffer
					}
				else if(p < 0)
					{
					errabort("error spawning child process at track");
					}
				else
					{
					tk->pid = p;
					processRunningNum++;
					}
				notfinished = TRUE;
				}
			else if(tk->pid > 0)
				{
				/* child has been forked */
				int status;
				waitpid(tk->pid, &status, 0);
				if(status == 0)
					{
					/* child process has finished successfully */
					tk->pid = 0;
					/* cubersome ft-specific treatment */
					double *data = malloc(sizeof(double)*dsp.usedSummaryNumber);
					FILE *fin = fopen(tk->tmpfile, "r");
					if(fin == NULL) errabort("error opening file for reading");
					if(tk->ft==FT_bedgraph_c||tk->ft==FT_bedgraph_n||tk->ft==FT_bigwighmtk_n||tk->ft==FT_bigwighmtk_c)
						{
						for(i=0; i<dsp.usedSummaryNumber; i++)
							{
							if(getline(&line, &s, fin) == -1) errabort(tk->tmpfile);
							if(line[0]=='n')
								{
								// is nan
								data[i]=NAN;
								}
							else
								{
								data[i] = strtod(line, NULL);
								}
							}
						if(printdata)
							{
							// print out json for tkLst array element
							//fprintf(stderr, "json for %s\n", tk->name);
							printf("{'name':'%s','ft':%d,",tk->name, tk->ft);
							if(tk->ft==FT_bedgraph_c||tk->ft==FT_bigwighmtk_c)
								printf("'label':'%s','url':'%s',", tk->label,tk->urlpath);
							printf("'data':[");
							// track data of different regions are in separate arrays
							int i=0;
							struct region *r;
							for(r=dsp.head; r!=NULL; r=r->next)
								{
								int j;
								printf("[");
								for(j=0; j<r->summarySize; j++)
									{
									if(isnan(data[i+j]))
										printf("NaN,");
									else
										printf("%.2f,", data[i+j]);
									}
								printf("],");
								i += j;
								}
							printf("]},");
							}
						}
					else if(tk->ft==FT_cat_n||tk->ft==FT_cat_c)
						{
						for(i=0; i<dsp.usedSummaryNumber; i++)
							{
							assert(getline(&line, &s, fin)!=-1);
							data[i] = strtol(line, NULL, 0);
							}
						if(printdata)
							{
							// print out json for tkLst array element
							printf("{'name':'%s','ft':%d,",tk->name, tk->ft);
							if(isCustom(tk->ft))
								printf("'label:'%s','url':'%s',", tk->label,tk->urlpath);
							printf("'data':[");
							// track data of different regions are in separate arrays
							int i=0;
							struct region *r;
							for(r=dsp.head; r!=NULL; r=r->next)
								{
								int j;
								printf("[");
								for(j=0; j<r->summarySize; j++)
									printf("%d,", (int)data[i+j]);
								printf("],");
								i += j;
								}
							printf("]},");
							}
						}
					else errabort("collecting hmtk data: unknown ft");
					tk->data = data;
					unlink(tk->tmpfile);
					//free(tk->tmpfile);
					}
				else
					{
					fprintf(stderr, "%s(%d) unknown status for child process\n", tk->name, tk->ft);
					}
				processRunningNum--;
				}
			}
		}
	if(printdata) printf("],");
	if(SQUAWK) fputs("track data fetched\n", stderr);
	}

puts("}"); // json outmost brace

if(CHECKCPUTIME)
    {
    clock_t this = clock();
    fprintf(stderr, "Elapsed time spent on fetching track data: %f\n", ((double)(this-cpuTimeTemp))/CLOCKS_PER_SEC);
    cpuTimeTemp = clock();
    }




if(CHECKCPUTIME)
    {
    clock_t this = clock();
    fprintf(Squawk, "Elapsed time after fetching track data: %f\n", ((double)(this-cpuTimeTemp))/CLOCKS_PER_SEC);
    fprintf(Squawk, "(Parallel) Entire CPU time: %f\n", ((double)(this-cpuTimeStart))/CLOCKS_PER_SEC);
    }

done();
return 1;
}
